@@ -41,13 +41,16 @@ enum cpioErrorReturns {
 	CPIOERR_READLINK_FAILED	= (19   | CPIOERR_CHECK_ERRNO),
 	CPIOERR_READ_FAILED	= (20   | CPIOERR_CHECK_ERRNO),
 	CPIOERR_COPY_FAILED	= (21   | CPIOERR_CHECK_ERRNO),
-	CPIOERR_HDR_SIZE	= (22			),
-	CPIOERR_HDR_TRAILER	= (23			),
-	CPIOERR_UNKNOWN_FILETYPE= (24			),
-	CPIOERR_MISSING_HARDLINK= (25			),
-	CPIOERR_MD5SUM_MISMATCH	= (26			),
-	CPIOERR_INTERNAL	= (27			),
-	CPIOERR_UNMAPPED_FILE	= (28			)
+	CPIOERR_LSETFCON_FAILED	= (22   | CPIOERR_CHECK_ERRNO),
+	CPIOERR_HDR_SIZE	= (23			),
+	CPIOERR_HDR_TRAILER	= (24			),
+	CPIOERR_UNKNOWN_FILETYPE= (25			),
+	CPIOERR_MISSING_HARDLINK= (26			),
+	CPIOERR_MD5SUM_MISMATCH	= (27			),
+	CPIOERR_INTERNAL	= (28			),
+	CPIOERR_UNMAPPED_FILE	= (29			),
+	CPIOERR_ENOENT		= (30			),
+	CPIOERR_ENOTEMPTY	= (31			)
 };
 
 /** \ingroup payload
@@ -100,7 +103,7 @@ extern "C" {
  * @return		0 on success
  */
 int cpioTrailerWrite(FSM_t fsm)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, fileSystem, internalState @*/;
 
 /**
@@ -110,7 +113,7 @@ int cpioTrailerWrite(FSM_t fsm)
  * @return		0 on success
  */
 int cpioHeaderWrite(FSM_t fsm, struct stat * st)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, fileSystem, internalState @*/;
 
 /**
@@ -120,7 +123,7 @@ int cpioHeaderWrite(FSM_t fsm, struct stat * st)
  * @return		0 on success
  */
 int cpioHeaderRead(FSM_t fsm, struct stat * st)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, *st, fileSystem, internalState @*/;
 
 /** \ingroup payload
