@@ -5,9 +5,6 @@
 
 #include "system.h"
 
-#include <signal.h>
-#include <sys/signal.h>
-
 #include <rpmio_internal.h>
 #include <rpmlib.h>
 #include <rpmmacro.h>
@@ -31,7 +28,6 @@
 #include "rpmlead.h"		/* writeLead proto */
 #include "signature.h"		/* signature constants */
 #include "legacy.h"		/* XXX rpmfiBuildFNames() */
-#include "ugid.h"		/* XXX unameToUid() and gnameToGid() */
 #include "misc.h"		/* XXX stripTrailingChar() */
 #include "rpmdb.h"		/* XXX for db_chrootDone */
 #include "debug.h"
@@ -39,6 +35,8 @@
 #define	_PSM_DEBUG	0
 /*@unchecked@*/
 int _psm_debug = _PSM_DEBUG;
+/*@unchecked@*/
+int _psm_threads = 0;
 
 /*@access FD_t @*/		/* XXX void ptr args */
 /*@access rpmpsm @*/
@@ -55,15 +53,14 @@ int rpmVersionCompare(Header first, Header second)
 
     if (!headerGetEntry(first, RPMTAG_EPOCH, NULL, (void **) &epochOne, NULL))
 	epochOne = NULL;
-    if (!headerGetEntry(second, RPMTAG_EPOCH, NULL, (void **) &epochTwo,
-			NULL))
+    if (!headerGetEntry(second, RPMTAG_EPOCH, NULL, (void **) &epochTwo, NULL))
 	epochTwo = NULL;
 
-    if (epochOne && !epochTwo)
+    if (epochOne != NULL && epochTwo == NULL)
 	return 1;
-    else if (!epochOne && epochTwo)
+    else if (epochOne == NULL && epochTwo != NULL)
 	return -1;
-    else if (epochOne && epochTwo) {
+    else if (epochOne != NULL && epochTwo != NULL) {
 /*@-boundsread@*/
 	if (*epochOne < *epochTwo)
 	    return -1;
@@ -156,7 +153,7 @@ static int rpmInstallLoadMacros(rpmfi fi, Header h)
  */
 /*@-bounds@*/
 static rpmRC markReplacedFiles(const rpmpsm psm)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies psm, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     const rpmts ts = psm->ts;
@@ -244,14 +241,14 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     struct rpmpsm_s psmbuf;
     rpmpsm psm = &psmbuf;
     int isSource;
-    rpmRC rc;
+    rpmRC rpmrc;
     int i;
 
     memset(psm, 0, sizeof(*psm));
     psm->ts = rpmtsLink(ts, "InstallSourcePackage");
 
-    rc = rpmReadPackageFile(ts, fd, "InstallSourcePackage", &h);
-    switch (rc) {
+    rpmrc = rpmReadPackageFile(ts, fd, "InstallSourcePackage", &h);
+    switch (rpmrc) {
     case RPMRC_NOTTRUSTED:
     case RPMRC_NOKEY:
     case RPMRC_OK:
@@ -263,13 +260,13 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     if (h == NULL)
 	goto exit;
 
-    rc = RPMRC_OK;
+    rpmrc = RPMRC_OK;
 
     isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
 
     if (!isSource) {
 	rpmError(RPMERR_NOTSRPM, _("source package expected, binary found\n"));
-	rc = RPMRC_FAIL;
+	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
 
@@ -279,13 +276,18 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     h = headerFree(h);
 
     if (fi == NULL) {	/* XXX can't happen */
-	rc = RPMRC_FAIL;
+	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
 
 /*@-onlytrans@*/	/* FIX: te reference */
     fi->te = rpmtsElement(ts, 0);
 /*@=onlytrans@*/
+    if (fi->te == NULL) {	/* XXX can't happen */
+	rpmrc = RPMRC_FAIL;
+	goto exit;
+    }
+
 /*@-nullpass@*/		/* FIX fi->h may be null */
     fi->te->h = headerLink(fi->h);
 /*@=nullpass@*/
@@ -318,13 +320,6 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     fi->astriplen = 0;
     fi->striplen = 0;
 
-    fi->fuids = xcalloc(sizeof(*fi->fuids), fi->fc);
-    fi->fgids = xcalloc(sizeof(*fi->fgids), fi->fc);
-    for (i = 0; i < fi->fc; i++) {
-	fi->fuids[i] = fi->uid;
-	fi->fgids[i] = fi->gid;
-    }
-
     for (i = 0; i < fi->fc; i++)
 	fi->actions[i] = FA_CREATE;
 
@@ -348,16 +343,16 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     }
 
     _sourcedir = rpmGenPath(rpmtsRootDir(ts), "%{_sourcedir}", "");
-    rc = rpmMkdirPath(_sourcedir, "sourcedir");
-    if (rc) {
-	rc = RPMRC_FAIL;
+    rpmrc = rpmMkdirPath(_sourcedir, "sourcedir");
+    if (rpmrc) {
+	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
 
     _specdir = rpmGenPath(rpmtsRootDir(ts), "%{_specdir}", "");
-    rc = rpmMkdirPath(_specdir, "specdir");
-    if (rc) {
-	rc = RPMRC_FAIL;
+    rpmrc = rpmMkdirPath(_specdir, "specdir");
+    if (rpmrc) {
+	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
 
@@ -389,22 +384,22 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	specFile = t;
     } else {
 	rpmError(RPMERR_NOSPEC, _("source package contains no .spec file\n"));
-	rc = RPMRC_FAIL;
+	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
 
     psm->goal = PSM_PKGINSTALL;
 
     /*@-compmempass@*/	/* FIX: psm->fi->dnl should be owned. */
-    rc = rpmpsmStage(psm, PSM_PROCESS);
+    rpmrc = rpmpsmStage(psm, PSM_PROCESS);
 
     (void) rpmpsmStage(psm, PSM_FINI);
     /*@=compmempass@*/
 
-    if (rc) rc = RPMRC_FAIL;
+    if (rpmrc) rpmrc = RPMRC_FAIL;
 
 exit:
-    if (specFilePtr && specFile && rc == RPMRC_OK)
+    if (specFilePtr && specFile && rpmrc == RPMRC_OK)
 	*specFilePtr = specFile;
     else
 	specFile = _free(specFile);
@@ -433,7 +428,7 @@ exit:
 
     psm->ts = rpmtsFree(psm->ts);
 
-    return rc;
+    return rpmrc;
 }
 
 /*@observer@*/ /*@unchecked@*/
@@ -457,259 +452,6 @@ static /*@observer@*/ const char * const tag2sln(int tag)
     return "%unknownscript";
 }
 
-/**
- */
-/*@unchecked@*/
-static sigset_t caught;
-
-/**
- */
-/*@unchecked@*/
-static struct psmtbl_s {
-    int nalloced;
-    int npsms;
-/*@null@*/
-    rpmpsm * psms;
-} psmtbl = { 0, 0, NULL };
-
-/* forward ref */
-static void handler(int signum)
-	/*@globals caught, psmtbl, fileSystem @*/
-	/*@modifies caught, psmtbl, fileSystem @*/;
-
-/**
- */
-/*@unchecked@*/
-/*@-fullinitblock@*/
-static struct sigtbl_s {
-    int signum;
-    int active;
-    void (*handler) (int signum);
-    struct sigaction oact;
-} satbl[] = {
-    { SIGCHLD,	0, handler },
-    { -1,	0, NULL },
-};
-/*@=fullinitblock@*/
-
-/**
- */
-/*@-incondefs@*/
-static void handler(int signum)
-{
-    struct sigtbl_s * tbl;
-
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (tbl->signum != signum)
-	    continue;
-	if (!tbl->active)
-	    continue;
-	(void) sigaddset(&caught, signum);
-	switch (signum) {
-	case SIGCHLD:
-	    while (1) {
-		int status = 0;
-		pid_t reaped = waitpid(0, &status, WNOHANG);
-		int i;
-
-		if (reaped <= 0)
-		    /*@innerbreak@*/ break;
-
-		if (psmtbl.psms)
-		for (i = 0; i < psmtbl.npsms; i++) {
-		    rpmpsm psm = psmtbl.psms[i];
-		    if (psm->child != reaped)
-			/*@innercontinue@*/ continue;
-
-#if _PSM_DEBUG
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "      Reap: %p[%d:%d:%d] = %p child %d\n", psmtbl.psms, i, psmtbl.npsms, psmtbl.nalloced, psm, psm->child);
-/*@=modfilesys@*/
-#endif
-
-		    psm->reaped = reaped;
-		    psm->status = status;
-		    /*@innerbreak@*/ break;
-		}
-	    }
-	    /*@switchbreak@*/ break;
-	default:
-	    /*@switchbreak@*/ break;
-	}
-	break;
-    }
-}
-/*@=incondefs@*/
-
-/**
- * Enable a signal handler.
- */
-static int enableSignal(int signum)
-	/*@globals caught, satbl, fileSystem @*/
-	/*@modifies caught, satbl, fileSystem @*/
-{
-    sigset_t newMask, oldMask;
-    struct sigtbl_s * tbl;
-    struct sigaction act;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    for (tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (signum >= 0 && signum != tbl->signum)
-	    continue;
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "    Enable: %p[0:%d:%d] active %d\n", psmtbl.psms, psmtbl.npsms, psmtbl.nalloced, tbl->active);
-/*@=modfilesys@*/
-	if (tbl->active++ <= 0) {
-	    (void) sigdelset(&caught, tbl->signum);
-	    memset(&act, 0, sizeof(act));
-	    act.sa_handler = tbl->handler;
-	    (void) sigaction(tbl->signum, &act, &tbl->oact);
-	}
-	break;
-    }
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
-}
-
-/**
- * Disable a signal handler.
- */
-static int disableSignal(int signum)
-	/*@globals satbl, fileSystem @*/
-	/*@modifies satbl, fileSystem @*/
-{
-    sigset_t newMask, oldMask;
-    struct sigtbl_s * tbl;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    for (tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (signum >= 0 && signum != tbl->signum)
-	    continue;
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "   Disable: %p[0:%d:%d] active %d\n", psmtbl.psms, psmtbl.npsms, psmtbl.nalloced, tbl->active);
-/*@=modfilesys@*/
-	if (--tbl->active <= 0) {
-	    tbl->active = 0;		/* XXX just in case */
-	    (void) sigaction(tbl->signum, &tbl->oact, NULL);
-	}
-	break;
-    }
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
-}
-
-/**
- * Register a child reaper, then fork a child.
- * @param psm		package state machine data
- * @return		fork(2) pid
- */
-static pid_t psmRegisterFork(rpmpsm psm)
-	/*@globals psmtbl, fileSystem, internalState @*/
-	/*@modifies psm, psmtbl, fileSystem, internalState @*/
-{
-    sigset_t newMask, oldMask;
-    int empty = -1;
-    int i = psmtbl.npsms;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-
-    if (psmtbl.psms)
-    for (i = 0; i < psmtbl.npsms; i++) {
-	if (empty == -1 && psmtbl.psms[i] == NULL)
-	    empty = i;
-	if (psm != psmtbl.psms[i])
-	    continue;
-	break;
-    }
-    if (i == psmtbl.npsms) {
-	if (i >= psmtbl.nalloced) {
-	    if (psmtbl.nalloced == 0) psmtbl.nalloced = 5;
-	    while (psmtbl.nalloced < i)
-		psmtbl.nalloced += psmtbl.nalloced;
-	    psmtbl.psms = xrealloc(psmtbl.psms,
-			psmtbl.nalloced * sizeof(*psmtbl.psms));
-	}
-	empty = psmtbl.npsms++;
-    }
-    if (psmtbl.psms)	/* XXX can't happen */
-	psmtbl.psms[empty] = rpmpsmLink(psm, "psmRegister");
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "  Register: %p[%d:%d:%d] = %p\n", psmtbl.psms, empty, psmtbl.npsms, psmtbl.nalloced, psm);
-/*@=modfilesys@*/
-
-    (void) enableSignal(SIGCHLD);
-
-    psm->reaped = 0;
-    if ((psm->child = fork()) != 0) {
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "      Fork: %p[%d:%d:%d] = %p child %d\n", psmtbl.psms, 0, psmtbl.npsms, psmtbl.nalloced, psm, psm->child);
-/*@=modfilesys@*/
-    }
-
-    (void) sigprocmask(SIG_SETMASK, &oldMask, NULL);
-
-    return psm->child;
-}
-
-/**
- * Unregister a child reaper.
- */
-static int psmWaitUnregister(rpmpsm psm, pid_t child)
-	/*@globals psmtbl, fileSystem, internalState @*/
-	/*@modifies psmtbl, fileSystem, internalState @*/
-{
-    sigset_t newMask, oldMask;
-    int i = 0;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    
-    /*@-infloops@*/
-    while (psm->reaped != psm->child)
-	(void) sigsuspend(&oldMask);
-    /*@=infloops@*/
-
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "      Wait: %p[%d:%d:%d] = %p child %d\n", psmtbl.psms, 0, psmtbl.npsms, psmtbl.nalloced, psm, psm->child);
-/*@=modfilesys@*/
-
-    if (psmtbl.psms)
-    for (i = 0; i < psmtbl.npsms; i++) {
-	if (psmtbl.psms[i] == NULL)
-	    continue;
-	if (psm != psmtbl.psms[i])
-	    continue;
-	if (child != psm->child)
-	    continue;
-	break;
-    }
-
-    if (i < psmtbl.npsms) {
-	(void) disableSignal(SIGCHLD);
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "Unregister: %p[%d:%d:%d] = %p child %d\n", psmtbl.psms, i, psmtbl.npsms, psmtbl.nalloced, psm, child);
-/*@=modfilesys@*/
-	if (psmtbl.psms)	/* XXX can't happen */
-	    psmtbl.psms[i] = rpmpsmFree(psmtbl.psms[i]);
-	if (psmtbl.npsms == (i+1))
-	    psmtbl.npsms--;
-	if (psmtbl.npsms == 0) {
-	    psmtbl.psms = _free(psmtbl.psms);
-	    psmtbl.nalloced = 0;
-	}
-    }
-
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
-}
-
 /**
  * Wait for child process to be reaped.
  * @param psm		package state machine data
@@ -719,19 +461,20 @@ static pid_t psmWait(rpmpsm psm)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies psm, fileSystem, internalState @*/
 {
-    if (psm->reaper) {
-	(void) psmWaitUnregister(psm, psm->child);
-    } else {
-	do {
-	    psm->reaped = waitpid(psm->child, &psm->status, 0);
-	} while (psm->reaped >= 0 && psm->reaped != psm->child);
-    }
+    const rpmts ts = psm->ts;
+    rpmtime_t msecs;
 
-    rpmMessage(RPMMESS_DEBUG, _("%s: waitpid(%d) rc %d status %x\n"),
-	psm->stepName, (unsigned)psm->child,
-	(unsigned)psm->reaped, psm->status);
+    (void) rpmsqWait(&psm->sq);
+    msecs = psm->sq.op.usecs/1000;
+    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_SCRIPTLETS), &psm->sq.op);
 
-    return psm->reaped;
+    rpmMessage(RPMMESS_DEBUG,
+	_("%s: waitpid(%d) rc %d status %x secs %u.%03u\n"),
+	psm->stepName, (unsigned)psm->sq.child,
+	(unsigned)psm->sq.reaped, psm->sq.status,
+	(unsigned)msecs/1000, (unsigned)msecs%1000);
+
+    return psm->sq.reaped;
 }
 
 /**
@@ -763,7 +506,7 @@ static const char * ldconfig_path = "/sbin/ldconfig";
 static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 		int progArgc, const char ** progArgv, 
 		const char * script, int arg1, int arg2)
-	/*@globals ldconfig_done, rpmGlobalMacroContext,
+	/*@globals ldconfig_done, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState@*/
 	/*@modifies psm, ldconfig_done, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -782,23 +525,22 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     int len;
     char * prefixBuf = NULL;
     const char * fn = NULL;
-    int i, xx;
+    int xx;
+    int i;
     int freePrefixes = 0;
     FD_t scriptFd;
     FD_t out;
     rpmRC rc = RPMRC_OK;
-    const char *n, *v, *r;
+    const char *n, *v, *r, *a;
 
     if (progArgv == NULL && script == NULL)
 	return rc;
 
-    psm->child = 0;
-    psm->reaped = 0;
-    psm->status = 0;
-    psm->reaper = 1;
+    psm->sq.reaper = 1;
 
     /* XXX FIXME: except for %verifyscript, rpmteNEVR can be used. */
     xx = headerNVR(h, &n, &v, &r);
+    xx = hge(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
 
     /* XXX bash must have functional libtermcap.so.2 */
     if (!strcmp(n, "libtermcap"))
@@ -810,16 +552,16 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     if (ldconfig_path && progArgv && psm->unorderedSuccessor) {
  	if (ldconfig_done && !strcmp(progArgv[0], ldconfig_path)) {
 	    rpmMessage(RPMMESS_DEBUG,
-		_("%s: %s(%s-%s-%s) skipping redundant \"%s\".\n"),
-		psm->stepName, tag2sln(psm->scriptTag), n, v, r,
+		_("%s: %s(%s-%s-%s.%s) skipping redundant \"%s\".\n"),
+		psm->stepName, tag2sln(psm->scriptTag), n, v, r, a,
 		progArgv[0]);
 	    return rc;
 	}
     }
 
     rpmMessage(RPMMESS_DEBUG,
-		_("%s: %s(%s-%s-%s) %ssynchronous scriptlet start\n"),
-		psm->stepName, tag2sln(psm->scriptTag), n, v, r,
+		_("%s: %s(%s-%s-%s.%s) %ssynchronous scriptlet start\n"),
+		psm->stepName, tag2sln(psm->scriptTag), n, v, r, a,
 		(psm->unorderedSuccessor ? "a" : ""));
 
     if (!progArgv) {
@@ -835,6 +577,39 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 		? 1 : 0);
     }
 
+#if __ia64__
+    /* XXX This assumes that all interpreters are elf executables. */
+    if ((a != NULL && a[0] == 'i' && a[1] != '\0' && a[2] == '8' && a[3] == '6')
+     && strcmp(argv[0], "/sbin/ldconfig"))
+    {
+	const char * fmt = rpmGetPath("%{?_autorelocate_path}", NULL);
+	const char * errstr;
+	char * newPath;
+	char * t;
+
+	newPath = headerSprintf(h, fmt, rpmTagTable, rpmHeaderFormats, &errstr);
+	fmt = _free(fmt);
+
+	/* XXX On ia64, change leading /emul/ix86 -> /emul/ia32, ick. */
+ 	if (newPath != NULL && *newPath != '\0'
+	 && strlen(newPath) >= (sizeof("/emul/i386")-1)
+	 && newPath[0] == '/' && newPath[1] == 'e' && newPath[2] == 'm'
+	 && newPath[3] == 'u' && newPath[4] == 'l' && newPath[5] == '/'
+	 && newPath[6] == 'i' && newPath[8] == '8' && newPath[9] == '6')
+ 	{
+	    newPath[7] = 'a';
+	    newPath[8] = '3';
+	    newPath[9] = '2';
+	}
+
+	t = alloca(strlen(newPath) + strlen(argv[0]) + 1);
+	*t = '\0';
+	(void) stpcpy( stpcpy(t, newPath), argv[0]);
+	newPath = _free(newPath);
+	argv[0] = t;
+    }
+#endif
+
     if (hge(h, RPMTAG_INSTPREFIXES, &ipt, (void **) &prefixes, &numPrefixes)) {
 	freePrefixes = 1;
     } else if (hge(h, RPMTAG_INSTALLPREFIX, NULL, (void **) &oldPrefix, NULL)) {
@@ -845,6 +620,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     }
 
     maxPrefixLength = 0;
+    if (prefixes != NULL)
     for (i = 0; i < numPrefixes; i++) {
 	len = strlen(prefixes[i]);
 	if (len > maxPrefixLength) maxPrefixLength = len;
@@ -857,7 +633,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 
 	/*@-branchstate@*/
 	if (makeTempFile((!rpmtsChrootDone(ts) ? rootDir : "/"), &fn, &fd)) {
-	    if (freePrefixes) free(prefixes);
+	    if (prefixes != NULL && freePrefixes) free(prefixes);
 	    return RPMRC_FAIL;
 	}
 	/*@=branchstate@*/
@@ -914,10 +690,12 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     if (out == NULL) return RPMRC_FAIL;	/* XXX can't happen */
     
     /*@-branchstate@*/
-    (void) psmRegisterFork(psm);
-    if (psm->child == 0) {
+    xx = rpmsqFork(&psm->sq);
+    if (psm->sq.child == 0) {
 	const char * rootDir;
 	int pipes[2];
+	int flag;
+	int fdno;
 
 	pipes[0] = pipes[1] = 0;
 	/* make stdin inaccessible */
@@ -926,6 +704,15 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	xx = dup2(pipes[0], STDIN_FILENO);
 	xx = close(pipes[0]);
 
+	/* XXX Force FD_CLOEXEC on 1st 100 inherited fdno's. */
+	for (fdno = 3; fdno < 100; fdno++) {
+	    flag = fcntl(fdno, F_GETFD);
+	    if (flag == -1 || (flag & FD_CLOEXEC))
+		continue;
+	    xx = fcntl(fdno, F_SETFD, FD_CLOEXEC);
+	    /* XXX W2DO? debug msg for inheirited fdno w/o FD_CLOEXEC */
+	}
+
 	if (scriptFd != NULL) {
 	    int sfdno = Fileno(scriptFd);
 	    int ofdno = Fileno(out);
@@ -934,11 +721,14 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    if (ofdno != STDOUT_FILENO)
 		xx = dup2(ofdno, STDOUT_FILENO);
 	    /* make sure we don't close stdin/stderr/stdout by mistake! */
-	    if (ofdno > STDERR_FILENO && ofdno != sfdno) {
+	    if (ofdno > STDERR_FILENO && ofdno != sfdno)
 		xx = Fclose (out);
-	    }
-	    if (sfdno > STDERR_FILENO) {
+	    if (sfdno > STDERR_FILENO)
 		xx = Fclose (scriptFd);
+	    else {
+/*@-usereleased@*/
+		xx = Fclose(out);
+/*@=usereleased@*/
 	    }
 	}
 
@@ -954,6 +744,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    /*@=modobserver@*/
 	}
 
+	if (prefixes != NULL)
 	for (i = 0; i < numPrefixes; i++) {
 	    sprintf(prefixBuf, "RPM_INSTALL_PREFIX%d=%s", i, prefixes[i]);
 	    xx = doputenv(prefixBuf);
@@ -981,15 +772,23 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 		/*@=superuser =noeffect @*/
 	    }
 	    xx = chdir("/");
-	    rpmMessage(RPMMESS_DEBUG, _("%s: %s(%s-%s-%s)\texecv(%s) pid %d\n"),
-			psm->stepName, sln, n, v, r,
+	    rpmMessage(RPMMESS_DEBUG, _("%s: %s(%s-%s-%s.%s)\texecv(%s) pid %d\n"),
+			psm->stepName, sln, n, v, r, a,
 			argv[0], (unsigned)getpid());
-/*@-modfilesys@*/
-if (_psm_debug)
-fprintf(stderr, "      Exec: %s \"%s\"\n", sln, argv[0]);
-/*@=modfilesys@*/
+
+	    /* XXX Don't mtrace into children. */
 	    unsetenv("MALLOC_CHECK_");
+
+	    /* Permit libselinux to do the scriptlet exec. */
+	    if (rpmtsSELinuxEnabled(ts) == 1) {
+		xx = rpm_execcon(0, argv[0], argv, environ);
+		if (xx != 0)
+		    break;
+	    }
+
+/*@-nullstate@*/
 	    xx = execv(argv[0], (char *const *)argv);
+/*@=nullstate@*/
 	    break;
 	default:
 	    break;
@@ -1002,18 +801,21 @@ fprintf(stderr, "      Exec: %s \"%s\"\n", sln, argv[0]);
 
     (void) psmWait(psm);
 
-    if (psm->reaped < 0) {
+  /* XXX filter order dependent multilib "other" arch helper error. */
+  if (!(psm->sq.reaped >= 0 && !strcmp(argv[0], "/usr/sbin/glibc_post_upgrade") && WEXITSTATUS(psm->sq.status) == 110)) {
+    if (psm->sq.reaped < 0) {
 	rpmError(RPMERR_SCRIPT,
-		_("%s(%s-%s-%s) scriptlet failed, waitpid(%d) rc %d: %s\n"),
-		 sln, n, v, r, psm->child, psm->reaped, strerror(errno));
+		_("%s(%s-%s-%s.%s) scriptlet failed, waitpid(%d) rc %d: %s\n"),
+		 sln, n, v, r, a, psm->sq.child, psm->sq.reaped, strerror(errno));
 	rc = RPMRC_FAIL;
     } else
-    if (!WIFEXITED(psm->status) || WEXITSTATUS(psm->status)) {
+    if (!WIFEXITED(psm->sq.status) || WEXITSTATUS(psm->sq.status)) {
 	rpmError(RPMERR_SCRIPT,
-		_("%s(%s-%s-%s) scriptlet failed, exit status %d\n"),
-		sln, n, v, r, WEXITSTATUS(psm->status));
+		_("%s(%s-%s-%s.%s) scriptlet failed, exit status %d\n"),
+		sln, n, v, r, a, WEXITSTATUS(psm->sq.status));
 	rc = RPMRC_FAIL;
     }
+  }
 
     if (freePrefixes) prefixes = hfd(prefixes, ipt);
 
@@ -1036,7 +838,7 @@ fprintf(stderr, "      Exec: %s \"%s\"\n", sln, argv[0]);
  * @return		rpmRC return code
  */
 static rpmRC runInstScript(rpmpsm psm)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies psm, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmfi fi = psm->fi;
@@ -1091,7 +893,7 @@ exit:
 static rpmRC handleOneTrigger(const rpmpsm psm,
 			Header sourceH, Header triggeredH,
 			int arg2, unsigned char * triggersAlreadyRun)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState@*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState@*/
 	/*@modifies psm, sourceH, triggeredH, *triggersAlreadyRun,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
@@ -1105,11 +907,13 @@ static rpmRC handleOneTrigger(const rpmpsm psm,
     const char ** triggerProgs;
     int_32 * triggerIndices;
     const char * sourceName;
+    const char * triggerName;
     rpmRC rc = RPMRC_OK;
     int xx;
     int i;
 
     xx = headerNVR(sourceH, &sourceName, NULL, NULL);
+    xx = headerNVR(triggeredH, &triggerName, NULL, NULL);
 
     trigger = rpmdsInit(rpmdsNew(triggeredH, RPMTAG_TRIGGERNAME, scareMem));
     if (trigger == NULL)
@@ -1148,7 +952,7 @@ static rpmRC handleOneTrigger(const rpmpsm psm,
 	{   int arg1;
 	    int index;
 
-	    arg1 = rpmdbCountPackages(rpmtsGetRdb(ts), Name);
+	    arg1 = rpmdbCountPackages(rpmtsGetRdb(ts), triggerName);
 	    if (arg1 < 0) {
 		/* XXX W2DO? fails as "execution of script failed" */
 		rc = RPMRC_FAIL;
@@ -1189,7 +993,7 @@ static rpmRC handleOneTrigger(const rpmpsm psm,
  * @return		0 on success
  */
 static rpmRC runTriggers(rpmpsm psm)
-	/*@globals rpmGlobalMacroContext,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies psm, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -1208,7 +1012,7 @@ static rpmRC runTriggers(rpmpsm psm)
     if (numPackage < 0)
 	return RPMRC_NOTFOUND;
 
-    if (fi->h != NULL)	/* XXX can't happen */
+    if (fi != NULL && fi->h != NULL)	/* XXX can't happen */
     {	Header triggeredH;
 	rpmdbMatchIterator mi;
 	int countCorrection = psm->countCorrection;
@@ -1230,7 +1034,7 @@ static rpmRC runTriggers(rpmpsm psm)
  * @return		0 on success
  */
 static rpmRC runImmedTriggers(rpmpsm psm)
-	/*@globals rpmGlobalMacroContext,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies psm, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -1363,7 +1167,9 @@ rpmpsm rpmpsmFree(rpmpsm psm)
 #else
     psm->te = NULL;
 #endif
+/*@-internalglobs@*/
     psm->ts = rpmtsFree(psm->ts);
+/*@=internalglobs@*/
 
     (void) rpmpsmUnlink(psm, msg);
 
@@ -1396,6 +1202,26 @@ rpmpsm rpmpsmNew(rpmts ts, rpmte te, rpmfi fi)
     return rpmpsmLink(psm, msg);
 }
 
+static void * rpmpsmThread(void * arg)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies arg, rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    rpmpsm psm = arg;
+/*@-unqualifiedtrans@*/
+    return ((void *) rpmpsmStage(psm, psm->nstage));
+/*@=unqualifiedtrans@*/
+}
+
+static int rpmpsmNext(rpmpsm psm, pkgStage nstage)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies psm, rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    psm->nstage = nstage;
+    if (_psm_threads)
+	return rpmsqJoin( rpmsqThread(rpmpsmThread, psm) );
+    return rpmpsmStage(psm, psm->nstage);
+}
+
 /**
  * @todo Packages w/o files never get a callback, hence don't get displayed
  * on install with -v.
@@ -1439,16 +1265,16 @@ rpmRC rpmpsmStage(rpmpsm psm, pkgStage stage)
 
 assert(psm->mi == NULL);
 	    psm->mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(psm->te), 0);
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_EPOCH, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
 			rpmteE(psm->te));
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_VERSION, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
 			rpmteV(psm->te));
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_RELEASE, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
 			rpmteR(psm->te));
 	    if (tscolor) {
-		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_ARCH, RPMMIRE_DEFAULT,
+		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_ARCH, RPMMIRE_STRCMP,
 			rpmteA(psm->te));
-		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_OS, RPMMIRE_DEFAULT,
+		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_OS, RPMMIRE_STRCMP,
 			rpmteO(psm->te));
 	    }
 
@@ -1470,7 +1296,7 @@ assert(psm->mi == NULL);
 	    if (fc <= 0)				break;
 	
 	    /*
-	     * Old format relocateable packages need the entire default
+	     * Old format relocatable packages need the entire default
 	     * prefix stripped to form the cpio list, while all other packages
 	     * need the leading / stripped.
 	     */
@@ -1492,17 +1318,13 @@ assert(psm->mi == NULL);
 	    if (fi->fgroup == NULL)
 		xx = hge(fi->h, RPMTAG_FILEGROUPNAME, NULL,
 				(void **) &fi->fgroup, NULL);
-	    if (fi->fuids == NULL)
-		fi->fuids = xcalloc(sizeof(*fi->fuids), fc);
-	    if (fi->fgids == NULL)
-		fi->fgids = xcalloc(sizeof(*fi->fgids), fc);
 	    rc = RPMRC_OK;
 	}
 	if (psm->goal == PSM_PKGERASE || psm->goal == PSM_PKGSAVE) {
 	    psm->scriptArg = psm->npkgs_installed - 1;
 	
 	    /* Retrieve installed header. */
-	    rc = rpmpsmStage(psm, PSM_RPMDB_LOAD);
+	    rc = rpmpsmNext(psm, PSM_RPMDB_LOAD);
 if (rc == RPMRC_OK)
 if (psm->te)
 psm->te->h = headerLink(fi->h);
@@ -1542,7 +1364,7 @@ psm->te->h = headerLink(fi->h);
 #endif
 
 	/* Change root directory if requested and not already done. */
-	rc = rpmpsmStage(psm, PSM_CHROOT_IN);
+	rc = rpmpsmNext(psm, PSM_CHROOT_IN);
 
 	if (psm->goal == PSM_PKGINSTALL) {
 	    psm->scriptTag = RPMTAG_PREIN;
@@ -1553,7 +1375,7 @@ psm->te->h = headerLink(fi->h);
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPRE)) {
-		rc = rpmpsmStage(psm, PSM_SCRIPT);
+		rc = rpmpsmNext(psm, PSM_SCRIPT);
 		if (rc != RPMRC_OK) {
 		    rpmError(RPMERR_SCRIPT,
 			_("%s: %s scriptlet failed (%d), skipping %s\n"),
@@ -1572,16 +1394,16 @@ psm->te->h = headerLink(fi->h);
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERUN)) {
 		/* Run triggers in this package other package(s) set off. */
-		rc = rpmpsmStage(psm, PSM_IMMED_TRIGGERS);
+		rc = rpmpsmNext(psm, PSM_IMMED_TRIGGERS);
 		if (rc) break;
 
 		/* Run triggers in other package(s) this package sets off. */
-		rc = rpmpsmStage(psm, PSM_TRIGGERS);
+		rc = rpmpsmNext(psm, PSM_TRIGGERS);
 		if (rc) break;
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPREUN))
-		rc = rpmpsmStage(psm, PSM_SCRIPT);
+		rc = rpmpsmNext(psm, PSM_SCRIPT);
 	}
 	if (psm->goal == PSM_PKGSAVE) {
 	    int noArchiveSize = 0;
@@ -1627,7 +1449,7 @@ psm->te->h = headerLink(fi->h);
 
 	    /* Retrieve type of payload compression. */
 	    /*@-nullstate@*/	/* FIX: psm->oh may be NULL */
-	    rc = rpmpsmStage(psm, PSM_RPMIO_FLAGS);
+	    rc = rpmpsmNext(psm, PSM_RPMIO_FLAGS);
 	    /*@=nullstate@*/
 
 	    /* Write the lead section into the package. */
@@ -1690,7 +1512,6 @@ psm->te->h = headerLink(fi->h);
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 
 	if (psm->goal == PSM_PKGINSTALL) {
-	    int i;
 
 	    if (rpmtsFlags(ts) & RPMTRANS_FLAG_JUSTDB)	break;
 
@@ -1702,36 +1523,8 @@ psm->te->h = headerLink(fi->h);
 		break;
 	    }
 
-	    (void) rpmfiInit(fi, 0);
-	    while ((i = rpmfiNext(fi)) >= 0) {
-		uid_t uid;
-		gid_t gid;
-
-		uid = fi->uid;
-		gid = fi->gid;
-		if (fi->fuser && unameToUid(fi->fuser[i], &uid)) {
-		    rpmMessage(RPMMESS_WARNING,
-			_("user %s does not exist - using root\n"),
-			fi->fuser[i]);
-		    uid = 0;
-		    /* XXX this diddles header memory. */
-		    fi->fmodes[i] &= ~S_ISUID;	/* turn off the suid bit */
-		}
-
-		if (fi->fgroup && gnameToGid(fi->fgroup[i], &gid)) {
-		    rpmMessage(RPMMESS_WARNING,
-			_("group %s does not exist - using root\n"),
-			fi->fgroup[i]);
-		    gid = 0;
-		    /* XXX this diddles header memory. */
-		    fi->fmodes[i] &= ~S_ISGID;	/* turn off the sgid bit */
-		}
-		if (fi->fuids) fi->fuids[i] = uid;
-		if (fi->fgids) fi->fgids[i] = gid;
-	    }
-
 	    /* Retrieve type of payload compression. */
-	    rc = rpmpsmStage(psm, PSM_RPMIO_FLAGS);
+	    rc = rpmpsmNext(psm, PSM_RPMIO_FLAGS);
 
 	    if (rpmteFd(fi->te) == NULL) {	/* XXX can't happen */
 		rc = RPMRC_FAIL;
@@ -1748,6 +1541,10 @@ psm->te->h = headerLink(fi->h);
 
 	    rc = fsmSetup(fi->fsm, FSM_PKGINSTALL, ts, fi,
 			psm->cfd, NULL, &psm->failedFile);
+	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_UNCOMPRESS),
+			fdstat_op(psm->cfd, FDSTAT_READ));
+	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DIGEST),
+			fdstat_op(psm->cfd, FDSTAT_DIGEST));
 	    xx = fsmTeardown(fi->fsm);
 
 	    saveerrno = errno; /* XXX FIXME: Fclose with libio destroys errno */
@@ -1758,13 +1555,13 @@ psm->te->h = headerLink(fi->h);
 	    /*@=mods@*/
 
 	    if (!rc)
-		rc = rpmpsmStage(psm, PSM_COMMIT);
+		rc = rpmpsmNext(psm, PSM_COMMIT);
 
 	    /* XXX make sure progress is closed out */
 	    psm->what = RPMCALLBACK_INST_PROGRESS;
 	    psm->amount = (fi->archiveSize ? fi->archiveSize : 100);
 	    psm->total = psm->amount;
-	    xx = rpmpsmStage(psm, PSM_NOTIFY);
+	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 
 	    if (rc) {
 		rpmError(RPMERR_CPIO,
@@ -1778,7 +1575,7 @@ psm->te->h = headerLink(fi->h);
 		psm->what = RPMCALLBACK_UNPACK_ERROR;
 		psm->amount = 0;
 		psm->total = 0;
-		xx = rpmpsmStage(psm, PSM_NOTIFY);
+		xx = rpmpsmNext(psm, PSM_NOTIFY);
 
 		break;
 	    }
@@ -1793,7 +1590,7 @@ psm->te->h = headerLink(fi->h);
 	    psm->what = RPMCALLBACK_UNINST_START;
 	    psm->amount = fc;		/* XXX W2DO? looks wrong. */
 	    psm->total = fc;
-	    xx = rpmpsmStage(psm, PSM_NOTIFY);
+	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 
 	    rc = fsmSetup(fi->fsm, FSM_PKGERASE, ts, fi,
 			NULL, NULL, &psm->failedFile);
@@ -1802,7 +1599,7 @@ psm->te->h = headerLink(fi->h);
 	    psm->what = RPMCALLBACK_UNINST_STOP;
 	    psm->amount = 0;		/* XXX W2DO? looks wrong. */
 	    psm->total = fc;
-	    xx = rpmpsmStage(psm, PSM_NOTIFY);
+	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 
 	}
 	if (psm->goal == PSM_PKGSAVE) {
@@ -1827,6 +1624,10 @@ psm->te->h = headerLink(fi->h);
 
 	    rc = fsmSetup(fi->fsm, FSM_PKGBUILD, ts, fi, psm->cfd,
 			NULL, &psm->failedFile);
+	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_COMPRESS),
+			fdstat_op(psm->cfd, FDSTAT_WRITE));
+	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DIGEST),
+			fdstat_op(psm->cfd, FDSTAT_DIGEST));
 	    xx = fsmTeardown(fi->fsm);
 
 	    saveerrno = errno; /* XXX FIXME: Fclose with libio destroys errno */
@@ -1840,7 +1641,7 @@ psm->te->h = headerLink(fi->h);
 	    psm->what = RPMCALLBACK_INST_PROGRESS;
 	    psm->amount = (fi->archiveSize ? fi->archiveSize : 100);
 	    psm->total = psm->amount;
-	    xx = rpmpsmStage(psm, PSM_NOTIFY);
+	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 
 	    fi->action = action;
 	    fi->actions = actions;
@@ -1869,11 +1670,11 @@ psm->te->h = headerLink(fi->h);
 	     * the database before adding the new one.
 	     */
 	    if (fi->record && !(rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY)) {
-		rc = rpmpsmStage(psm, PSM_RPMDB_REMOVE);
+		rc = rpmpsmNext(psm, PSM_RPMDB_REMOVE);
 		if (rc) break;
 	    }
 
-	    rc = rpmpsmStage(psm, PSM_RPMDB_ADD);
+	    rc = rpmpsmNext(psm, PSM_RPMDB_ADD);
 	    if (rc) break;
 
 	    psm->scriptTag = RPMTAG_POSTIN;
@@ -1882,16 +1683,16 @@ psm->te->h = headerLink(fi->h);
 	    psm->countCorrection = 0;
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPOST)) {
-		rc = rpmpsmStage(psm, PSM_SCRIPT);
+		rc = rpmpsmNext(psm, PSM_SCRIPT);
 		if (rc) break;
 	    }
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERIN)) {
 		/* Run triggers in other package(s) this package sets off. */
-		rc = rpmpsmStage(psm, PSM_TRIGGERS);
+		rc = rpmpsmNext(psm, PSM_TRIGGERS);
 		if (rc) break;
 
 		/* Run triggers in this package other package(s) set off. */
-		rc = rpmpsmStage(psm, PSM_IMMED_TRIGGERS);
+		rc = rpmpsmNext(psm, PSM_IMMED_TRIGGERS);
 		if (rc) break;
 	    }
 
@@ -1907,30 +1708,30 @@ psm->te->h = headerLink(fi->h);
 	    psm->countCorrection = -1;
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPOSTUN)) {
-		rc = rpmpsmStage(psm, PSM_SCRIPT);
+		rc = rpmpsmNext(psm, PSM_SCRIPT);
 		/* XXX WTFO? postun failures don't cause erasure failure. */
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERPOSTUN)) {
 		/* Run triggers in other package(s) this package sets off. */
-		rc = rpmpsmStage(psm, PSM_TRIGGERS);
+		rc = rpmpsmNext(psm, PSM_TRIGGERS);
 		if (rc) break;
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY))
-		rc = rpmpsmStage(psm, PSM_RPMDB_REMOVE);
+		rc = rpmpsmNext(psm, PSM_RPMDB_REMOVE);
 	}
 	if (psm->goal == PSM_PKGSAVE) {
 	}
 
 	/* Restore root directory if changed. */
-	xx = rpmpsmStage(psm, PSM_CHROOT_OUT);
+	xx = rpmpsmNext(psm, PSM_CHROOT_OUT);
 	break;
     case PSM_UNDO:
 	break;
     case PSM_FINI:
 	/* Restore root directory if changed. */
-	xx = rpmpsmStage(psm, PSM_CHROOT_OUT);
+	xx = rpmpsmNext(psm, PSM_CHROOT_OUT);
 
 	if (psm->fd != NULL) {
 	    saveerrno = errno; /* XXX FIXME: Fclose with libio destroys errno */
@@ -1962,7 +1763,7 @@ psm->te->h = headerLink(fi->h);
 	    psm->amount = 0;
 	    psm->total = 0;
 	    /*@-nullstate@*/ /* FIX: psm->fd may be NULL. */
-	    xx = rpmpsmStage(psm, PSM_NOTIFY);
+	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 	    /*@=nullstate@*/
 	}
 
@@ -1980,8 +1781,6 @@ psm->te->h = headerFree(psm->te->h);
 	psm->rpmio_flags = _free(psm->rpmio_flags);
 	psm->failedFile = _free(psm->failedFile);
 
-	fi->fgids = _free(fi->fgids);
-	fi->fuids = _free(fi->fuids);
 	fi->fgroup = hfd(fi->fgroup, -1);
 	fi->fuser = hfd(fi->fuser, -1);
 	fi->apath = _free(fi->apath);
@@ -1995,11 +1794,11 @@ psm->te->h = headerFree(psm->te->h);
 	psm->rc = RPMRC_OK;
 	psm->stepName = pkgStageString(stage);
 
-	rc = rpmpsmStage(psm, PSM_INIT);
-	if (!rc) rc = rpmpsmStage(psm, PSM_PRE);
-	if (!rc) rc = rpmpsmStage(psm, PSM_PROCESS);
-	if (!rc) rc = rpmpsmStage(psm, PSM_POST);
-	xx = rpmpsmStage(psm, PSM_FINI);
+	rc = rpmpsmNext(psm, PSM_INIT);
+	if (!rc) rc = rpmpsmNext(psm, PSM_PRE);
+	if (!rc) rc = rpmpsmNext(psm, PSM_PROCESS);
+	if (!rc) rc = rpmpsmNext(psm, PSM_POST);
+	xx = rpmpsmNext(psm, PSM_FINI);
 	break;
     case PSM_PKGCOMMIT:
 	break;
@@ -2026,17 +1825,21 @@ psm->te->h = headerFree(psm->te->h);
     case PSM_CHROOT_IN:
     {	const char * rootDir = rpmtsRootDir(ts);
 	/* Change root directory if requested and not already done. */
-	if (rootDir != NULL && !rpmtsChrootDone(ts) && !psm->chrootDone) {
-	    static int _loaded = 0;
+	if (rootDir != NULL && !(rootDir[0] == '/' && rootDir[1] == '\0')
+	 && !rpmtsChrootDone(ts) && !psm->chrootDone)
+	{
+	    static int _pw_loaded = 0;
+	    static int _gr_loaded = 0;
 
-	    /*
-	     * This loads all of the name services libraries, in case we
-	     * don't have access to them in the chroot().
-	     */
-	    if (!_loaded) {
+	    if (!_pw_loaded) {
 		(void)getpwnam("root");
 		endpwent();
-		_loaded++;
+		_pw_loaded++;
+	    }
+	    if (!_gr_loaded) {
+		(void)getgrnam("root");
+		endgrent();
+		_gr_loaded++;
 	    }
 
 	    xx = chdir("/");
@@ -2106,17 +1909,21 @@ assert(psm->mi == NULL);
     case PSM_RPMDB_ADD:
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 	if (fi->h == NULL)	break;	/* XXX can't happen */
+	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
 	if (!(rpmtsVSFlags(ts) & RPMVSF_NOHDRCHK))
 	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
 				ts, headerCheck);
 	else
 	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
 				NULL, NULL);
+	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
 	break;
     case PSM_RPMDB_REMOVE:
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
+	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
 	rc = rpmdbRemove(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->record,
 				NULL, NULL);
+	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
 	break;
 
     default:
