@@ -366,6 +366,8 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
 	int rc;
 	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
 	    ec = rc;
+	if (qva->qva_source == RPMQV_DBOFFSET)
+	    break;
     }
     qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
     return ec;
@@ -480,7 +482,9 @@ restart:
 	    }
 	    
 	    /* Read list of packages from manifest. */
+/*@-nullstate@*/ /* FIX: *av may be NULL */
 	    res = rpmReadPackageManifest(fd, &ac, &av);
+/*@=nullstate@*/
 	    if (res != RPMRC_OK) {
 		rpmError(RPMERR_MANIFEST,
 			_("%s: not an rpm package (or package manifest): %s\n"),
@@ -516,9 +520,33 @@ restart:
 	    res = 1;
 	} else {
 	    if (arg != NULL)
-	    for (av = (const char **) arg; *av; av++) {
-		if (!rpmdbSetIteratorRE(qva->qva_mi, RPMTAG_NAME, RPMMIRE_DEFAULT, *av))
+	    for (av = (const char **) arg; *av != NULL; av++) {
+		int tag = RPMTAG_NAME;
+		const char * pat;
+		char * a, * ae;
+
+		pat = a = xstrdup(*av);
+		tag = RPMTAG_NAME;
+
+		/* Parse for "tag=pattern" args. */
+		if ((ae = strchr(a, '=')) != NULL) {
+		    *ae++ = '\0';
+		    tag = tagValue(a);
+		    if (tag < 0) {
+			rpmError(RPMERR_QUERYINFO,
+				_("unknown tag: \"%s\"\n"), a);
+			res = 1;
+		    }
+		    pat = ae;
+		}
+
+		if (!res)
+		    res = rpmdbSetIteratorRE(qva->qva_mi, tag, RPMMIRE_DEFAULT, pat);
+		a = _free(a);
+
+		if (res == 0)
 		    continue;
+
 		qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
 		res = 1;
 		/*@loopbreak@*/ break;
@@ -775,6 +803,22 @@ int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
     if (qva->qva_flags & VERIFY_HDRCHK)
 	vsflags |= RPMVSF_NOHDRCHK;
 
+#ifdef	NOTYET
+    /* Initialize security context patterns (if not already done). */
+    if (!(qva->qva_flags & VERIFY_CONTEXTS)) {
+	rpmsx sx = rpmtsREContext(ts);
+	if (sx == NULL) {
+	    arg = rpmGetPath("%{?_verify_file_context_path}", NULL);
+	    if (arg != NULL && *arg != '\0') {
+		sx = rpmsxNew(arg);
+		(void) rpmtsSetREContext(ts, sx);
+	    }
+	    arg = _free(arg);
+	}
+	sx = rpmsxFree(sx);
+    }
+#endif
+
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     if (qva->qva_source == RPMQV_ALL) {
 	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
