@@ -445,8 +445,18 @@ fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
 	    (void) rpmfiSetDX(fi, dx);
 
 	    /*@-assignexpose -dependenttrans -observertrans@*/
-	    dieNeedle->dirName = (char *) rpmfiDN(fi);
+	    {   const char * DN = rpmfiDN(fi);
+
+#if defined(__ia64__)
+/* XXX Make sure that autorelocated file dependencies are satisfied. */
+#define	DNPREFIX	"/emul/ia32-linux"
+		if (!strncmp(DN, DNPREFIX, sizeof(DNPREFIX)-1))
+		    DN += sizeof(DNPREFIX)-1;
+#endif
+		dieNeedle->dirName = DN;
+	    }
 	    /*@=assignexpose =dependenttrans =observertrans@*/
+
 	    dieNeedle->dirNameLen = (dieNeedle->dirName != NULL
 			? strlen(dieNeedle->dirName) : 0);
 	    die = bsearch(dieNeedle, al->dirs, origNumDirs,
@@ -598,11 +608,13 @@ void rpmalMakeIndex(rpmal al)
 
     ai->index = xrealloc(ai->index, ai->size * sizeof(*ai->index));
     ai->k = 0;
-
     for (i = 0; i < al->size; i++) {
 	alp = al->list + i;
 	rpmalAddProvides(al, (alKey)i, alp->provides, alp->tscolor);
     }
+
+    /* Reset size to the no. of provides added. */
+    ai->size = ai->k;
     qsort(ai->index, ai->size, sizeof(*ai->index), indexcmp);
 }
 
@@ -752,7 +764,7 @@ rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 
     if (al->list != NULL)	/* XXX always true */
     for (ret = NULL, found = 0;
-	 match <= ai->index + ai->size && indexcmp(match, needle) == 0;
+	 match < ai->index + ai->size && indexcmp(match, needle) == 0;
 	 match++)
     {
 	alp = al->list + alKey2Num(al, match->pkgKey);
