@@ -114,10 +114,177 @@ fprintf(stderr, "*** ds %p\t%s[%d]\n", ds, ds->Type, ds->Count);
     return NULL;
 }
 
-rpmds rpmdsNew(Header h, rpmTag tagN, int scareMem)
+/*@unchecked@*/ /*@observer@*/
+static const char * beehiveToken = "archfilter";
+
+/**
+ * Return archScore filter boolean.
+ * @param arch		beehive arch score token
+ * @returns		0 == false, 1 == true
+ */
+static int archFilter(const char * arch)
+	/*@*/
 {
+    static int oneshot = 0;
+    int negate = 0;	/* assume no negation. */
+    int rc = 0;		/* assume arch does not apply */
+
+    if (*arch == '!') {
+	negate = 1;
+	arch++;
+    }
+    if (*arch == '=') {
+	const char * myarch = NULL;
+	arch++;
+
+	if (oneshot <= 0) {
+	    rpmSetTables(RPM_MACHTABLE_INSTARCH, RPM_MACHTABLE_INSTOS);
+	    rpmSetMachine(NULL, NULL);
+	    oneshot++;
+	}
+	rpmGetMachine(&myarch, NULL);
+	if (myarch != NULL) {
+	    if (negate)
+		rc = (!strcmp(arch, myarch) ? 0 : 1);
+	    else
+		rc = (!strcmp(arch, myarch) ? 1 : 0);
+/*@-modfilesys@*/
+if (_rpmds_debug < 0)
+fprintf(stderr, "=== strcmp(\"%s\", \"%s\") negate %d rc %d\n", arch, myarch, negate, rc);
+/*@=modfilesys@*/
+	}
+    } else {
+	int archScore = rpmMachineScore(RPM_MACHTABLE_INSTARCH, arch);
+	if (negate)
+	    rc = (archScore > 0 ? 0 : 1);
+	else
+	    rc = (archScore > 0 ? 1 : 0);
+/*@-modfilesys@*/
+if (_rpmds_debug < 0)
+fprintf(stderr, "=== archScore(\"%s\") %d negate %d rc %d\n", arch, archScore, negate, rc);
+/*@=modfilesys@*/
+    }
+    return rc;
+}
+
+/**
+ * Filter dependency set, removing "foo(bar,i386,=s390,!sparcv8)" wrapper.
+ * @param ds		dependency set
+ * @param token		namespace string
+ * @returns		filtered dependency set
+ */
+static rpmds rpmdsFilter(/*@null@*/ /*@returned@*/ rpmds ds,
+		/*@null@*/ const char * token)
+	/*@modifies ds @*/
+{
+    size_t toklen;
+    rpmds fds;
+    int i;
+
+    if (ds == NULL || token == NULL || *token == '\0')
+	goto exit;
+
+    toklen = strlen(token);
+    fds = rpmdsLink(ds, ds->Type);
+    fds = rpmdsInit(ds);
+    if (fds != NULL)
+    while ((i = rpmdsNext(fds)) >= 0) {
+	const char * N = rpmdsN(fds);
+	const char * gN;
+	const char * f, * fe;
+	const char * g, * ge;
+	size_t len;
+	int ignore;
+	int state;
+	char buf[1024+1];
+	int nb;
+
+	if (N == NULL)
+	    continue;
+	len = strlen(N);
+	if (len < (toklen + (sizeof("()")-1)))
+	    continue;
+	if (strncmp(N, token, toklen))
+	    continue;
+	if (*(f = N + toklen) != '(')
+	    continue;
+	if (*(fe = N + len - 1) != ')')
+	    continue;
+/*@-modfilesys@*/
+if (_rpmds_debug < 0)
+fprintf(stderr, "*** f \"%s\"\n", f);
+/*@=modfilesys@*/
+	g = f + 1;
+	state = 0;
+	gN = NULL;
+	ignore = 1;	/* assume depedency will be skipped. */
+	for (ge = (char *) g; ge < fe; g = ++ge) {
+	    while (ge < fe && *ge != ':')
+		ge++;
+
+	    nb = (ge - g);
+	    if (nb < 0 || nb > (sizeof(buf)-1))
+		nb = sizeof(buf) - 1;
+	    (void) strncpy(buf, g, nb);
+	    buf[nb] = '\0';
+/*@-branchstate@*/
+	    switch (state) {
+	    case 0:		/* g is unwrapped N */
+		gN = xstrdup(buf);
+		/*@switchbreak@*/ break;
+	    default:		/* g is next arch score token. */
+		/* arch score tokens are compared assuming || */
+		if (archFilter(buf))
+		    ignore = 0;
+		/*@switchbreak@*/ break;
+	    }
+/*@=branchstate@*/
+	    state++;
+	}
+	if (ignore) {
+	    int Count = rpmdsCount(fds);
+/*@-modfilesys@*/
+if (_rpmds_debug < 0)
+fprintf(stderr, "***   deleting N[%d:%d] = \"%s\"\n", i, Count, N);
+/*@=modfilesys@*/
+	    if (i < (Count - 1)) {
+		memcpy((fds->N + i), (fds->N + i + 1), (Count - (i+1)) * sizeof(*fds->N));
+		if (fds->EVR != NULL)
+		    memcpy((fds->EVR + i), (fds->EVR + i + 1), (Count - (i+1)) * sizeof(*fds->EVR));
+		if (fds->Flags != NULL)
+		    memcpy((fds->Flags + i), (fds->Flags + i + 1), (Count - (i+1)) * sizeof(*fds->Flags));
+	 	fds->i--;
+	    }
+	    fds->Count--;
+	} else if (gN != NULL) {
+/*@-modobserver -observertrans@*/
+	    char * t = (char *) N;
+	    (void) strcpy(t, gN);
+/*@=modobserver =observertrans@*/
+/*@-modfilesys@*/
+if (_rpmds_debug < 0)
+fprintf(stderr, "*** unwrapping N[%d] = \"%s\"\n", i, N);
+/*@=modfilesys@*/
+	}
+	gN = _free(gN);
+    }
+    fds = rpmdsFree(fds);
+
+exit:
+    /*@-refcounttrans@*/
+    return ds;
+    /*@=refcounttrans@*/
+}
+
+rpmds rpmdsNew(Header h, rpmTag tagN, int flags)
+{
+    int scareMem = (flags & 0x1);
+    int nofilter = (flags & 0x2);
     HGE_t hge =
 	(scareMem ? (HGE_t) headerGetEntryMinMemory : (HGE_t) headerGetEntry);
+    rpmTag tagBT = RPMTAG_BUILDTIME;
+    rpmTagType BTt;
+    int_32 * BTp;
     rpmTag tagEVR, tagF;
     rpmds ds = NULL;
     const char * Type;
@@ -167,7 +334,7 @@ rpmds rpmdsNew(Header h, rpmTag tagN, int scareMem)
 	ds->N = N;
 	ds->Nt = Nt;
 	ds->Count = Count;
-	ds->nopromote = 0;
+	ds->nopromote = _rpmds_nopromote;
 
 	xx = hge(h, tagEVR, &ds->EVRt, (void **) &ds->EVR, NULL);
 	xx = hge(h, tagF, &ds->Ft, (void **) &ds->Flags, NULL);
@@ -175,6 +342,8 @@ rpmds rpmdsNew(Header h, rpmTag tagN, int scareMem)
 	if (!scareMem && ds->Flags != NULL)
 	    ds->Flags = memcpy(xmalloc(ds->Count * sizeof(*ds->Flags)),
                                 ds->Flags, ds->Count * sizeof(*ds->Flags));
+	xx = hge(h, tagBT, &BTt, (void **) &BTp, NULL);
+	ds->BT = (xx && BTp != NULL && BTt == RPM_INT32_TYPE ? *BTp : 0);
 /*@=boundsread@*/
 	ds->Color = xcalloc(Count, sizeof(*ds->Color));
 	ds->Refs = xcalloc(Count, sizeof(*ds->Refs));
@@ -189,8 +358,13 @@ fprintf(stderr, "*** ds %p\t%s[%d]\n", ds, ds->Type, ds->Count);
 
 exit:
     /*@-nullstate@*/ /* FIX: ds->Flags may be NULL */
-    return rpmdsLink(ds, (ds ? ds->Type : NULL));
+    ds = rpmdsLink(ds, (ds ? ds->Type : NULL));
     /*@=nullstate@*/
+
+    if (!nofilter)
+	ds = rpmdsFilter(ds, beehiveToken);
+
+    return ds;
 }
 
 char * rpmdsNewDNEVR(const char * dspfx, const rpmds ds)
@@ -347,6 +521,9 @@ rpmds rpmdsSingle(rpmTag tagN, const char * N, const char * EVR, int_32 Flags)
     ds->h = NULL;
     ds->Type = Type;
     ds->tagN = tagN;
+    {	time_t now = time(NULL);
+	ds->BT = now;
+    }
     ds->Count = 1;
     /*@-assignexpose@*/
 /*@-boundswrite@*/
@@ -452,6 +629,24 @@ rpmTag rpmdsTagN(const rpmds ds)
     return tagN;
 }
 
+time_t rpmdsBT(const rpmds ds)
+{
+    time_t BT = 0;
+    if (ds != NULL && ds->BT > 0)
+	BT = ds->BT;
+    return BT;
+}
+
+time_t rpmdsSetBT(const rpmds ds, time_t BT)
+{
+    time_t oBT = 0;
+    if (ds != NULL) {
+	oBT = ds->BT;
+	ds->BT = BT;
+    }
+    return oBT;
+}
+
 int rpmdsNoPromote(const rpmds ds)
 {
     int nopromote = 0;
@@ -605,10 +800,13 @@ assert(argv[ac] != NULL);
 	t = stpcpy(t, argv[ac]) + 1;
     }
     av[ac] = NULL;
+/*@-nullret@*/
     return av;
+/*@=nullret@*/
 }
 /*@=bounds@*/
 
+/*@null@*/
 static rpmds rpmdsDup(const rpmds ods)
 	/*@modifies ods @*/
 {
@@ -707,6 +905,8 @@ int save;
     }
 /*@=branchstate@*/
     ds = *dsp;
+    if (ds == NULL)
+	return -1;
 
     /*
      * Add new entries.
@@ -935,7 +1135,7 @@ void rpmdsProblem(rpmps ps, const char * pkgNEVR, const rpmds ds,
 
 int rpmdsAnyMatchesDep (const Header h, const rpmds req, int nopromote)
 {
-    int scareMem = 1;
+    int scareMem = 0;
     rpmds provides = NULL;
     int result = 0;
 
