@@ -85,8 +85,8 @@ static void delTE(rpmte p)
 static void addTE(rpmts ts, rpmte p, Header h,
 		/*@dependent@*/ /*@null@*/ fnpyKey key,
 		/*@null@*/ rpmRelocation * relocs)
-	/*@globals fileSystem @*/
-	/*@modifies ts, p, h, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies ts, p, h, rpmGlobalMacroContext, fileSystem @*/
 {
     int scareMem = 0;
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
@@ -106,10 +106,22 @@ static void addTE(rpmts ts, rpmte p, Header h,
 
     arch = NULL;
     xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
-    p->arch = (arch != NULL ? xstrdup(arch) : NULL);
+    if (arch != NULL) {
+	p->arch = xstrdup(arch);
+	p->archScore = rpmMachineScore(RPM_MACHTABLE_INSTARCH, arch);
+    } else {
+	p->arch = NULL;
+	p->archScore = 0;
+    }
     os = NULL;
     xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
-    p->os = (os != NULL ? xstrdup(os) : NULL);
+    if (os != NULL) {
+	p->os = xstrdup(os);
+	p->osScore = rpmMachineScore(RPM_MACHTABLE_INSTOS, os);
+    } else {
+	p->os = NULL;
+	p->osScore = 0;
+    }
 
     nb = strlen(p->NEVR) + 1;
     if (p->arch)
@@ -131,13 +143,15 @@ static void addTE(rpmts ts, rpmte p, Header h,
 	p->epoch = NULL;
 /*@=branchstate@*/
 
+    p->nrelocs = 0;
+    p->relocs = NULL;
     if (relocs != NULL) {
 	rpmRelocation * r;
 	int i;
 
-	for (i = 0, r = relocs; r->oldPath || r->newPath; i++, r++)
-	    {};
-	p->relocs = xmalloc((i + 1) * sizeof(*p->relocs));
+	for (r = relocs; r->oldPath || r->newPath; r++)
+	    p->nrelocs++;
+	p->relocs = xmalloc((p->nrelocs + 1) * sizeof(*p->relocs));
 
 	for (i = 0, r = relocs; r->oldPath || r->newPath; i++, r++) {
 	    p->relocs[i].oldPath = r->oldPath ? xstrdup(r->oldPath) : NULL;
@@ -145,9 +159,8 @@ static void addTE(rpmts ts, rpmte p, Header h,
 	}
 	p->relocs[i].oldPath = NULL;
 	p->relocs[i].newPath = NULL;
-    } else {
-	p->relocs = NULL;
     }
+    p->autorelocatex = -1;
 
     p->key = key;
     p->fd = NULL;
@@ -193,10 +206,10 @@ rpmte rpmteNew(const rpmts ts, Header h,
     int_32 * ep;
     int xx;
 
+    p->type = type;
     addTE(ts, p, h, key, relocs);
     switch (type) {
     case TR_ADDED:
-	p->type = type;
 	p->u.addedKey = pkgKey;
 	ep = NULL;
 	xx = headerGetEntry(h, RPMTAG_SIGSIZE, NULL, (void **)&ep, NULL);
@@ -205,7 +218,6 @@ rpmte rpmteNew(const rpmts ts, Header h,
 	    p->pkgFileSize += 96 + 256 + *ep;
 	break;
     case TR_REMOVED:
-	p->type = type;
 	p->u.removed.dependsOnKey = pkgKey;
 	p->u.removed.dboffset = dboffset;
 	break;
@@ -543,8 +555,10 @@ rpmtsi XrpmtsiFree(/*@only@*//*@null@*/ rpmtsi tsi,
 		const char * fn, unsigned int ln)
 {
     /* XXX watchout: a funky recursion segfaults here iff nrefs is wrong. */
+/*@-internalglobs@*/
     if (tsi)
 	tsi->ts = rpmtsFree(tsi->ts);
+/*@=internalglobs@*/
 
 /*@-modfilesys@*/
 if (_rpmte_debug)
@@ -574,7 +588,7 @@ fprintf(stderr, "*** tsi %p ++ %s:%d\n", tsi, fn, ln);
  * @param tsi		transaction element iterator
  * @return		transaction element, NULL on termination
  */
-static /*@dependent@*/ /*@null@*/
+static /*@null@*/ /*@dependent@*/
 rpmte rpmtsiNextElement(rpmtsi tsi)
 	/*@modifies tsi @*/
 {
