@@ -13,6 +13,7 @@
 #include "rpmal.h"
 #include "rpmds.h"
 #include "rpmfi.h"
+#include "rpmlock.h"
 
 #define	_RPMTE_INTERNAL		/* XXX te->h */
 #include "rpmte.h"
@@ -54,6 +55,7 @@ extern int statvfs (const char * file, /*@out@*/ struct statvfs * buf)
 
 /*@access rpmps @*/
 /*@access rpmDiskSpaceInfo @*/
+/*@access rpmsx @*/
 /*@access rpmte @*/
 /*@access rpmtsi @*/
 /*@access fnpyKey @*/
@@ -85,6 +87,29 @@ char * hGetNEVR(Header h, const char ** np)
     return NVR;
 }
 
+char * hGetNEVRA(Header h, const char ** np)
+{
+    const char * n, * v, * r, * a;
+    char * NVRA, * t;
+    int xx;
+
+    (void) headerNVR(h, &n, &v, &r);
+    xx = headerGetEntry(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
+    NVRA = t = xcalloc(1, strlen(n) + strlen(v) + strlen(r) + strlen(a) + sizeof("--."));
+/*@-boundswrite@*/
+    t = stpcpy(t, n);
+    t = stpcpy(t, "-");
+    t = stpcpy(t, v);
+    t = stpcpy(t, "-");
+    t = stpcpy(t, r);
+    t = stpcpy(t, ".");
+    t = stpcpy(t, a);
+    if (np)
+	*np = n;
+/*@=boundswrite@*/
+    return NVRA;
+}
+
 uint_32 hGetColor(Header h)
 {
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
@@ -133,6 +158,9 @@ int rpmtsCloseDB(rpmts ts)
     int rc = 0;
 
     if (ts->rdb != NULL) {
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBGET), &ts->rdb->db_getops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBPUT), &ts->rdb->db_putops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBDEL), &ts->rdb->db_delops);
 	rc = rpmdbClose(ts->rdb);
 	ts->rdb = NULL;
     }
@@ -164,16 +192,24 @@ int rpmtsOpenDB(rpmts ts, int dbmode)
 
 int rpmtsInitDB(rpmts ts, int dbmode)
 {
-    return rpmdbInit(ts->rootDir, dbmode);
+    void *lock = rpmtsAcquireLock(ts);
+    int rc = -1;
+    if (lock)
+	    rc = rpmdbInit(ts->rootDir, dbmode);
+    rpmtsFreeLock(lock);
+    return rc;
 }
 
 int rpmtsRebuildDB(rpmts ts)
 {
     int rc;
+    void *lock = rpmtsAcquireLock(ts);
+    if (!lock) return -1;
     if (!(ts->vsflags & RPMVSF_NOHDRCHK))
 	rc = rpmdbRebuild(ts->rootDir, ts, headerCheck);
     else
 	rc = rpmdbRebuild(ts->rootDir, NULL, NULL);
+    rpmtsFreeLock(lock);
     return rc;
 }
 
@@ -182,14 +218,17 @@ int rpmtsVerifyDB(rpmts ts)
     return rpmdbVerify(ts->rootDir);
 }
 
+/*@-boundsread@*/
 static int isArch(const char * arch)
 	/*@*/
 {
     const char ** av;
+/*@-nullassign@*/
+    /*@observer@*/
     static const char *arches[] = {
-	"i386", "i486", "i586", "i686", "athlon", "x86_64",
+	"i386", "i486", "i586", "i686", "athlon", "x86_64", "amd64", "ia32e",
 	"alpha", "alphaev5", "alphaev56", "alphapca56", "alphaev6", "alphaev67",
-	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv9",
+	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9",
 	"sparc64", "sun4u",
 	"mips", "mipsel", "IP",
 	"ppc", "ppciseries", "ppcpseries",
@@ -203,6 +242,7 @@ static int isArch(const char * arch)
 	"noarch",
 	NULL,
     };
+/*@=nullassign@*/
 
     for (av = arches; *av != NULL; av++) {
 	if (!strcmp(arch, *av))
@@ -210,7 +250,9 @@ static int isArch(const char * arch)
     }
     return 0;
 }
+/*@=boundsread@*/
 
+/*@-compdef@*/ /* keyp might no be defined. */
 rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 			const void * keyp, size_t keylen)
 {
@@ -222,7 +264,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	return NULL;
 
     /* Parse out "N(EVR).A" tokens from a label key. */
-/*@-branchstate@*/
+/*@-bounds -branchstate@*/
     if (rpmtag == RPMDBI_LABEL && keyp != NULL) {
 	const char * s = keyp;
 	const char *se;
@@ -236,7 +278,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	    switch (c) {
 	    default:
 		*t++ = c;
-		break;
+		/*@switchbreak@*/ break;
 	    case '(':
 		/* XXX Fail if nested parens. */
 		if (level++ != 0) {
@@ -245,7 +287,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 		}
 		/* Parse explicit epoch. */
 		for (se = s; *se && xisdigit(*se); se++)
-		    ;
+		    {};
 		if (*se == ':') {
 		    /* XXX skip explicit epoch's (for now) */
 		    *t++ = '-';
@@ -254,7 +296,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 		    /* No Epoch: found. Convert '(' to '-' and chug. */
 		    *t++ = '-';
 		}
-		break;
+		/*@switchbreak@*/ break;
 	    case ')':
 		/* XXX Fail if nested parens. */
 		if (--level != 0) {
@@ -262,7 +304,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 		    return NULL;
 		}
 		/* Don't copy trailing ')' */
-		break;
+		/*@switchbreak@*/ break;
 	    }
 	}
 	if (level) {
@@ -278,7 +320,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	   arch = t;
 	}
     }
-/*@=branchstate@*/
+/*@=bounds =branchstate@*/
 
     mi = rpmdbInitIterator(ts->rdb, rpmtag, keyp, keylen);
 
@@ -291,6 +333,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
     return mi;
 }
+/*@=compdef@*/
 
 rpmRC rpmtsFindPubkey(rpmts ts)
 {
@@ -411,6 +454,9 @@ int rpmtsCloseSDB(rpmts ts)
     int rc = 0;
 
     if (ts->sdb != NULL) {
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBGET), &ts->sdb->db_getops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBPUT), &ts->sdb->db_putops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBDEL), &ts->sdb->db_delops);
 	rc = rpmdbClose(ts->sdb);
 	ts->sdb = NULL;
     }
@@ -781,6 +827,8 @@ rpmts rpmtsFree(rpmts ts)
 
     (void) rpmtsCloseSDB(ts);
 
+    ts->sx = rpmsxFree(ts->sx);
+
     ts->removedPackages = _free(ts->removedPackages);
 
     ts->availablePackages = rpmalFree(ts->availablePackages);
@@ -923,13 +971,14 @@ void rpmtsSetScriptFd(rpmts ts, FD_t scriptFd)
     }
 }
 
+int rpmtsSELinuxEnabled(rpmts ts)
+{
+    return (ts != NULL ? (ts->selinuxEnabled > 0) : 0);
+}
+
 int rpmtsChrootDone(rpmts ts)
 {
-    int chrootDone = 0;
-    if (ts != NULL) {
-	chrootDone = ts->chrootDone;
-    }
-    return chrootDone;
+    return (ts != NULL ? ts->chrootDone : 0);
 }
 
 int rpmtsSetChrootDone(rpmts ts, int chrootDone)
@@ -944,6 +993,23 @@ int rpmtsSetChrootDone(rpmts ts, int chrootDone)
     return ochrootDone;
 }
 
+rpmsx rpmtsREContext(rpmts ts)
+{
+    return ( (ts && ts->sx ? rpmsxLink(ts->sx, __func__) : NULL) );
+}
+
+int rpmtsSetREContext(rpmts ts, rpmsx sx)
+{
+    int rc = -1;
+    if (ts != NULL) {
+	ts->sx = rpmsxFree(ts->sx);
+	ts->sx = rpmsxLink(sx, __func__);
+	if (ts->sx != NULL)
+	    rc = 0;
+    }
+    return rc;
+}
+
 int_32 rpmtsGetTid(rpmts ts)
 {
     int_32 tid = 0;
@@ -1406,6 +1472,8 @@ rpmts rpmtsCreate(void)
     ts->currDir = NULL;
     ts->chrootDone = 0;
 
+    ts->selinuxEnabled = is_selinux_enabled();
+
     ts->numAddedPackages = 0;
     ts->addedPackages = NULL;
 
