@@ -14,6 +14,8 @@
 
 #include "rpmds.h"
 
+#include "rpmlock.h"
+
 #define	_RPMFI_INTERNAL
 #include "rpmfi.h"
 
@@ -80,129 +82,12 @@ static int sharedCmp(const void * one, const void * two)
 }
 
 /**
- */
-/*@-boundsread@*/
-static fileAction decideFileFate(const rpmts ts,
-		const rpmfi ofi, rpmfi nfi)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies nfi, fileSystem, internalState @*/
-{
-    const char * fn = rpmfiFN(nfi);
-    int newFlags = rpmfiFFlags(nfi);
-    char buffer[1024];
-    fileTypes dbWhat, newWhat, diskWhat;
-    struct stat sb;
-    int save = (newFlags & RPMFILE_NOREPLACE) ? FA_ALTNAME : FA_SAVE;
-
-    if (lstat(fn, &sb)) {
-	/*
-	 * The file doesn't exist on the disk. Create it unless the new
-	 * package has marked it as missingok, or allfiles is requested.
-	 */
-	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_ALLFILES)
-	 && (newFlags & RPMFILE_MISSINGOK))
-	{
-	    rpmMessage(RPMMESS_DEBUG, _("%s skipped due to missingok flag\n"),
-			fn);
-	    return FA_SKIP;
-	} else {
-	    return FA_CREATE;
-	}
-    }
-
-    diskWhat = whatis((int_16)sb.st_mode);
-    dbWhat = whatis(rpmfiFMode(ofi));
-    newWhat = whatis(rpmfiFMode(nfi));
-
-    /*
-     * RPM >= 2.3.10 shouldn't create config directories -- we'll ignore
-     * them in older packages as well.
-     */
-    if (newWhat == XDIR)
-	return FA_CREATE;
-
-    if (diskWhat != newWhat)
-	return save;
-    else if (newWhat != dbWhat && diskWhat != dbWhat)
-	return save;
-    else if (dbWhat != newWhat)
-	return FA_CREATE;
-    else if (dbWhat != LINK && dbWhat != REG)
-	return FA_CREATE;
-
-    /*
-     * This order matters - we'd prefer to CREATE the file if at all
-     * possible in case something else (like the timestamp) has changed.
-     */
-    if (dbWhat == REG) {
-	const unsigned char * omd5, * nmd5;
-	if (domd5(fn, buffer, 0, NULL))
-	    return FA_CREATE;	/* assume file has been removed */
-	omd5 = rpmfiMD5(ofi);
-	if (omd5 && !memcmp(omd5, buffer, 16))
-	    return FA_CREATE;	/* unmodified config file, replace. */
-	nmd5 = rpmfiMD5(nfi);
-/*@-nullpass@*/
-	if (omd5 && nmd5 && !memcmp(omd5, nmd5, 16))
-	    return FA_SKIP;	/* identical file, don't bother. */
-/*@=nullpass@*/
-    } else /* dbWhat == LINK */ {
-	const char * oFLink, * nFLink;
-	memset(buffer, 0, sizeof(buffer));
-	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
-	    return FA_CREATE;	/* assume file has been removed */
-	oFLink = rpmfiFLink(ofi);
-	if (oFLink && !strcmp(oFLink, buffer))
-	    return FA_CREATE;	/* unmodified config file, replace. */
-	nFLink = rpmfiFLink(nfi);
-/*@-nullpass@*/
-	if (oFLink && nFLink && !strcmp(oFLink, nFLink))
-	    return FA_SKIP;	/* identical file, don't bother. */
-/*@=nullpass@*/
-    }
-
-    /*
-     * The config file on the disk has been modified, but
-     * the ones in the two packages are different. It would
-     * be nice if RPM was smart enough to at least try and
-     * merge the difference ala CVS, but...
-     */
-    return save;
-}
-/*@=boundsread@*/
-
-/**
- */
-/*@-boundsread@*/
-static int filecmp(rpmfi afi, rpmfi bfi)
-	/*@*/
-{
-    fileTypes awhat = whatis(rpmfiFMode(afi));
-    fileTypes bwhat = whatis(rpmfiFMode(bfi));
-
-    if (awhat != bwhat) return 1;
-
-    if (awhat == LINK) {
-	const char * alink = rpmfiFLink(afi);
-	const char * blink = rpmfiFLink(bfi);
-	if (alink == blink) return 0;
-	if (alink == NULL) return 1;
-	if (blink == NULL) return -1;
-	return strcmp(alink, blink);
-    } else if (awhat == REG) {
-	const unsigned char * amd5 = rpmfiMD5(afi);
-	const unsigned char * bmd5 = rpmfiMD5(bfi);
-	if (amd5 == bmd5) return 0;
-	if (amd5 == NULL) return 1;
-	if (bmd5 == NULL) return -1;
-	return memcmp(amd5, bmd5, 16);
-    }
-
-    return 0;
-}
-/*@=boundsread@*/
-
-/**
+ * @param ts		transaction set
+ * @param p
+ * @param fi		file info set
+ * @param shared
+ * @param sharedCount
+ * @param reportConflicts
  */
 /* XXX only ts->{probs,rpmdb} modified */
 /*@-bounds@*/
@@ -210,12 +95,12 @@ static int handleInstInstalledFiles(const rpmts ts,
 		rpmte p, rpmfi fi,
 		sharedFileInfo shared,
 		int sharedCount, int reportConflicts)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     uint_32 tscolor = rpmtsColor(ts);
     uint_32 otecolor, tecolor;
-    uint_32 oficolor, ficolor;
+    uint_32 oFColor, FColor;
     const char * altNEVR = NULL;
     rpmfi otherFi = NULL;
     int numReplaced = 0;
@@ -260,13 +145,13 @@ static int handleInstInstalledFiles(const rpmts ts,
 
 	otherFileNum = shared->otherFileNum;
 	(void) rpmfiSetFX(otherFi, otherFileNum);
-	oficolor = rpmfiFColor(otherFi);
-	oficolor &= tscolor;
+	oFColor = rpmfiFColor(otherFi);
+	oFColor &= tscolor;
 
 	fileNum = shared->pkgFileNum;
 	(void) rpmfiSetFX(fi, fileNum);
-	ficolor = rpmfiFColor(fi);
-	ficolor &= tscolor;
+	FColor = rpmfiFColor(fi);
+	FColor &= tscolor;
 
 	isCfgFile = ((rpmfiFFlags(otherFi) | rpmfiFFlags(fi)) & RPMFILE_CONFIG);
 
@@ -279,17 +164,32 @@ static int handleInstInstalledFiles(const rpmts ts,
 	if (XFA_SKIPPING(fi->actions[fileNum]))
 	    continue;
 
-	if (filecmp(otherFi, fi)) {
-	    /* Report conflicts only for packages/files of same color. */
-	    if (tscolor == 0 || (tecolor == otecolor && ficolor == oficolor))
-	    if (reportConflicts) {
+	if (rpmfiCompare(otherFi, fi)) {
+	    int rConflicts;
+
+	    rConflicts = reportConflicts;
+	    /* Resolve file conflicts to prefer Elf64 (if not forced). */
+	    if (tscolor != 0 && FColor != 0 && FColor != oFColor)
+	    {
+		if (oFColor & 0x2) {
+		    fi->actions[fileNum] = FA_SKIPCOLOR;
+		    rConflicts = 0;
+		} else
+		if (FColor & 0x2) {
+		    fi->actions[fileNum] = FA_CREATE;
+		    rConflicts = 0;
+		}
+	    }
+
+	    if (rConflicts) {
 		rpmpsAppend(ps, RPMPROB_FILE_CONFLICT,
 			rpmteNEVR(p), rpmteKey(p),
 			rpmfiDN(fi), rpmfiBN(fi),
 			altNEVR,
 			0);
 	    }
-	    if (!isCfgFile) {
+	    /* Save file identifier to mark as state REPLACED. */
+	    if ( !(isCfgFile || XFA_SKIPPING(fi->actions[fileNum])) ) {
 		/*@-assignexpose@*/ /* FIX: p->replaced, not fi */
 		if (!shared->isRemoved)
 		    fi->replaced[numReplaced++] = *shared;
@@ -297,9 +197,11 @@ static int handleInstInstalledFiles(const rpmts ts,
 	    }
 	}
 
+	/* Determine config file dispostion, skipping missing files (if any). */
 	if (isCfgFile) {
-	    fileAction action;
-	    action = decideFileFate(ts, otherFi, fi);
+	    int skipMissing =
+		((rpmtsFlags(ts) & RPMTRANS_FLAG_ALLFILES) ? 0 : 1);
+	    fileAction action = rpmfiDecideFate(otherFi, fi, skipMissing);
 	    fi->actions[fileNum] = action;
 	}
 	fi->replacedSizes[fileNum] = rpmfiFSize(otherFi);
@@ -322,7 +224,7 @@ static int handleInstInstalledFiles(const rpmts ts,
 /* XXX only ts->rpmdb modified */
 static int handleRmvdInstalledFiles(const rpmts ts, rpmfi fi,
 		sharedFileInfo shared, int sharedCount)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     HGE_t hge = fi->hge;
@@ -515,7 +417,7 @@ bingoFps->baseName);
 /* XXX only ts->{probs,di} modified */
 static void handleOverlappedFiles(const rpmts ts,
 		const rpmte p, rpmfi fi)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, fileSystem, internalState @*/
 {
     uint_32 fixupSize = 0;
@@ -527,6 +429,8 @@ static void handleOverlappedFiles(const rpmts ts,
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)
     while ((i = rpmfiNext(fi)) >= 0) {
+	uint_32 tscolor = rpmtsColor(ts);
+	uint_32 oFColor, FColor;
 	struct fingerPrint_s * fiFps;
 	int otherPkgNum, otherFileNum;
 	rpmfi otherFi;
@@ -542,6 +446,8 @@ static void handleOverlappedFiles(const rpmts ts,
 	fiFps = fi->fps + i;
 	FFlags = rpmfiFFlags(fi);
 	FMode = rpmfiFMode(fi);
+	FColor = rpmfiFColor(fi);
+	FColor &= tscolor;
 
 	fixupSize = 0;
 
@@ -603,10 +509,17 @@ static void handleOverlappedFiles(const rpmts ts,
 		/*@innerbreak@*/ break;
 	}
 
+	oFColor = rpmfiFColor(otherFi);
+	oFColor &= tscolor;
+
 /*@-boundswrite@*/
 	switch (rpmteType(p)) {
 	case TR_ADDED:
 	  { struct stat sb;
+	    int reportConflicts =
+		!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_REPLACENEWFILES);
+	    int done = 0;
+
 	    if (otherPkgNum < 0) {
 		/* XXX is this test still necessary? */
 		if (fi->actions[i] != FA_UNKNOWN)
@@ -623,14 +536,46 @@ static void handleOverlappedFiles(const rpmts ts,
 
 assert(otherFi != NULL);
 	    /* Mark added overlapped non-identical files as a conflict. */
-	    if (!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_REPLACENEWFILES)
-	     && filecmp(otherFi, fi))
-	    {
-		rpmpsAppend(ps, RPMPROB_NEW_FILE_CONFLICT,
+	    if (rpmfiCompare(otherFi, fi)) {
+		int rConflicts;
+
+		rConflicts = reportConflicts;
+		/* Resolve file conflicts to prefer Elf64 (if not forced) ... */
+		if (tscolor != 0) {
+		    if (FColor & 0x2) {
+			/* ... last Elf64 file is installed ... */
+			if (!XFA_SKIPPING(fi->actions[i])) {
+			    /* XXX static helpers are order dependent. Ick. */
+			    if (strcmp(fn, "/usr/sbin/libgcc_post_upgrade")
+			     && strcmp(fn, "/usr/sbin/glibc_post_upgrade"))
+				otherFi->actions[otherFileNum] = FA_SKIP;
+			}
+			fi->actions[i] = FA_CREATE;
+			rConflicts = 0;
+		    } else
+		    if (oFColor & 0x2) {
+			/* ... first Elf64 file is installed ... */
+			if (XFA_SKIPPING(fi->actions[i]))
+			    otherFi->actions[otherFileNum] = FA_CREATE;
+			fi->actions[i] = FA_SKIPCOLOR;
+			rConflicts = 0;
+		    } else
+		    if (FColor == 0 && oFColor == 0) {
+			/* ... otherwise, do both, last in wins. */
+			otherFi->actions[otherFileNum] = FA_CREATE;
+			fi->actions[i] = FA_CREATE;
+			rConflicts = 0;
+		    }
+		    done = 1;
+		}
+
+		if (rConflicts) {
+		    rpmpsAppend(ps, RPMPROB_NEW_FILE_CONFLICT,
 			rpmteNEVR(p), rpmteKey(p),
 			fn, NULL,
 			rpmteNEVR(otherFi->te),
 			0);
+		}
 	    }
 
 	    /* Try to get the disk accounting correct even if a conflict. */
@@ -641,7 +586,8 @@ assert(otherFi != NULL);
 		fi->actions[i] = (FFlags & RPMFILE_NOREPLACE)
 			? FA_ALTNAME : FA_SKIP;
 	    } else {
-		fi->actions[i] = FA_CREATE;
+		if (!done)
+		    fi->actions[i] = FA_CREATE;
 	    }
 	  } /*@switchbreak@*/ break;
 
@@ -748,11 +694,11 @@ static int ensureOlder(rpmts ts,
 /*@-mustmod@*/ /* FIX: fi->actions is modified. */
 /*@-bounds@*/
 static void skipFiles(const rpmts ts, rpmfi fi)
-	/*@globals rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies fi, rpmGlobalMacroContext @*/
 {
     uint_32 tscolor = rpmtsColor(ts);
-    uint_32 ficolor;
+    uint_32 FColor;
     int noConfigs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOCONFIGS);
     int noDocs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NODOCS);
     char ** netsharedPaths = NULL;
@@ -817,8 +763,8 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	}
 
 	/* Ignore colored files not in our rainbow. */
-	ficolor = rpmfiFColor(fi);
-	if (tscolor && ficolor && !(tscolor & ficolor)) {
+	FColor = rpmfiFColor(fi);
+	if (tscolor && FColor && !(tscolor & FColor)) {
 	    drc[ix]--;	dff[ix] = 1;
 	    fi->actions[i] = FA_SKIPCOLOR;
 	    continue;
@@ -864,7 +810,7 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	/*
 	 * Skip i18n language specific files.
 	 */
-	if (fi->flangs && languages && *fi->flangs[i]) {
+	if (languages != NULL && fi->flangs != NULL && *fi->flangs[i]) {
 	    const char **lang, *l, *le;
 	    for (lang = languages; *lang != NULL; lang++) {
 		if (!strcmp(*lang, "all"))
@@ -957,11 +903,13 @@ static void skipFiles(const rpmts ts, rpmfi fi)
 	}
     }
 
+/*@-dependenttrans@*/
     if (netsharedPaths) freeSplitString(netsharedPaths);
 #ifdef	DYING	/* XXX freeFi will deal with this later. */
     fi->flangs = _free(fi->flangs);
 #endif
     if (languages) freeSplitString((char **)languages);
+/*@=dependenttrans@*/
 }
 /*@=bounds@*/
 /*@=mustmod@*/
@@ -1012,12 +960,20 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     rpmtsi qi;	rpmte q;
     int numAdded;
     int numRemoved;
+    void * lock = NULL;
     int xx;
 
     /* XXX programmer error segfault avoidance. */
     if (rpmtsNElements(ts) <= 0)
 	return -1;
 
+    /* If we are in test mode, then there's no need for transaction lock. */
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
+	lock = rpmtsAcquireLock(ts);
+	if (lock == NULL)
+	    return -1;
+    }
+
     if (rpmtsFlags(ts) & RPMTRANS_FLAG_NOSCRIPTS)
 	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | _noTransScripts | _noTransTriggers));
     if (rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERS)
@@ -1034,9 +990,11 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 		? O_RDONLY : (O_RDWR|O_CREAT);
 
 	/* Open database RDWR for installing packages. */
-	if (rpmtsOpenDB(ts, dbmode))
+	if (rpmtsOpenDB(ts, dbmode)) {
+	    rpmtsFreeLock(lock);
 	    return -1;	/* XXX W2DO? */
     }
+    }
 
     ts->ignoreSet = ignoreSet;
     {	const char * currDir = currentDirectory();
@@ -1098,16 +1056,16 @@ rpmMessage(RPMMESS_DEBUG, _("sanity checking %d elements\n"), rpmtsNElements(ts)
 
 	if (!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_REPLACEPKG)) {
 	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
 				rpmteE(p));
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
 				rpmteV(p));
-	    xx = rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_DEFAULT,
+	    xx = rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
 				rpmteR(p));
 	    if (tscolor) {
-		xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT,
+		xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP,
 				rpmteA(p));
-		xx = rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_DEFAULT,
+		xx = rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP,
 				rpmteO(p));
 	    }
 
@@ -1151,6 +1109,7 @@ rpmMessage(RPMMESS_DEBUG, _("sanity checking %d elements\n"), rpmtsNElements(ts)
      * worth the trouble though.
      */
 rpmMessage(RPMMESS_DEBUG, _("computing %d file fingerprints\n"), totalFileCount);
+
     numAdded = numRemoved = 0;
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, 0)) != NULL) {
@@ -1206,6 +1165,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing %d file fingerprints\n"), totalFileCount)
 	    continue;	/* XXX can't happen */
 	fc = rpmfiFC(fi);
 
+	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), 0);
 	fpLookupList(fpc, fi->dnl, fi->bnl, fi->dil, fc, fi->fps);
 	/*@-branchstate@*/
  	fi = rpmfiInit(fi, 0);
@@ -1218,6 +1178,8 @@ rpmMessage(RPMMESS_DEBUG, _("computing %d file fingerprints\n"), totalFileCount)
 	    /*@=dependenttrans@*/
 	}
 	/*@=branchstate@*/
+	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), fc);
+
     }
     pi = rpmtsiFree(pi);
 
@@ -1246,10 +1208,12 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 
 	if (fc == 0) continue;
 
+	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), 0);
 	/* Extract file info for all files in this package from the database. */
 	matches = xcalloc(fc, sizeof(*matches));
 	if (rpmdbFindFpList(rpmtsGetRdb(ts), fi->fps, matches, fc)) {
 	    ps = rpmpsFree(ps);
+	    rpmtsFreeLock(lock);
 	    return 1;	/* XXX WTFO? */
 	}
 
@@ -1345,6 +1309,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 	case TR_REMOVED:
 	    /*@switchbreak@*/ break;
 	}
+	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), fc);
     }
     pi = rpmtsiFree(pi);
     ps = rpmpsFree(ps);
@@ -1386,6 +1351,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 		(okProbs != NULL || rpmpsTrim(ts->probs, okProbs)))
        )
     {
+	rpmtsFreeLock(lock);
 	return ts->orderCount;
     }
 
@@ -1394,6 +1360,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
      */
     if (rpmtsFlags(ts) & (RPMTRANS_FLAG_DIRSTASH | RPMTRANS_FLAG_REPACKAGE)) {
 	int progress;
+
 	progress = 0;
 	pi = rpmtsiInit(ts);
 	while ((p = rpmtsiNext(pi, 0)) != NULL) {
@@ -1416,6 +1383,8 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 			numRemoved, NULL, ts->notifyData));
 		progress++;
 
+		(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_REPACKAGE), 0);
+
 	/* XXX TR_REMOVED needs CPIO_MAP_{ABSOLUTE,ADDDOT} CPIO_ALL_HARDLINKS */
 		fi->mapflags |= CPIO_MAP_ABSOLUTE;
 		fi->mapflags |= CPIO_MAP_ADDDOT;
@@ -1427,6 +1396,8 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 		fi->mapflags &= ~CPIO_MAP_ADDDOT;
 		fi->mapflags &= ~CPIO_ALL_HARDLINKS;
 
+		(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_REPACKAGE), 0);
+
 		/*@switchbreak@*/ break;
 	    }
 	}
@@ -1454,15 +1425,19 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 	    continue;	/* XXX can't happen */
 	
 	psm = rpmpsmNew(ts, p, fi);
+assert(psm != NULL);
 	psm->unorderedSuccessor =
 		(rpmtsiOc(pi) >= rpmtsUnorderedSuccessors(ts, -1) ? 1 : 0);
 
 	switch (rpmteType(p)) {
 	case TR_ADDED:
+	    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_INSTALL), 0);
 
 	    pkgKey = rpmteAddedKey(p);
 
-	    rpmMessage(RPMMESS_DEBUG, "========== +++ %s\n", rpmteNEVR(p));
+	    rpmMessage(RPMMESS_DEBUG, "========== +++ %s %s-%s 0x%x\n",
+		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
+
 	    p->h = NULL;
 	    /*@-type@*/ /* FIX: rpmte not opaque */
 	    {
@@ -1513,7 +1488,9 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 
 		    fi->fstates = NULL;
 		    fi->actions = NULL;
+/*@-nullstate@*/ /* FIX: fi->actions is NULL */
 		    fi = rpmfiFree(fi);
+/*@=nullstate@*/
 
 		    savep = rpmtsSetRelocateElement(ts, p);
 		    fi = rpmfiNew(ts, p->h, RPMTAG_BASENAMES, 1);
@@ -1553,9 +1530,16 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 
 	    p->h = headerFree(p->h);
 
+	    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_INSTALL), 0);
+
 	    /*@switchbreak@*/ break;
+
 	case TR_REMOVED:
-	    rpmMessage(RPMMESS_DEBUG, "========== --- %s\n", rpmteNEVR(p));
+	    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_ERASE), 0);
+
+	    rpmMessage(RPMMESS_DEBUG, "========== --- %s %s-%s 0x%x\n",
+		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
+
 	    /*
 	     * XXX This has always been a hack, now mostly broken.
 	     * If install failed, then we shouldn't erase.
@@ -1564,6 +1548,9 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 		if (rpmpsmStage(psm, PSM_PKGERASE))
 		    ourrc++;
 	    }
+
+	    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_ERASE), 0);
+
 	    /*@switchbreak@*/ break;
 	}
 	xx = rpmdbSync(rpmtsGetRdb(ts));
@@ -1580,6 +1567,8 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
     /*@=branchstate@*/
     pi = rpmtsiFree(pi);
 
+    rpmtsFreeLock(lock);
+
     /*@-nullstate@*/ /* FIX: ts->flList may be NULL */
     if (ourrc)
     	return -1;
