@@ -32,6 +32,7 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
     rpmVerifyAttrs flags = rpmfiVFlags(fi);
     const char * fn = rpmfiFN(fi);
     const char * rootDir = rpmtsRootDir(ts);
+    int selinuxEnabled = rpmtsSELinuxEnabled(ts);
     struct stat sb;
     int rc;
 
@@ -77,6 +78,8 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	return 1;
     }
 
+    flags |= RPMVERIFY_CONTEXTS;	/* no disable from package. */
+
     /*
      * Not all attributes of non-regular files can be verified.
      */
@@ -112,7 +115,36 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
     /*
      * Don't verify any features in omitMask.
      */
-    flags &= ~(omitMask | RPMVERIFY_LSTATFAIL|RPMVERIFY_READFAIL|RPMVERIFY_READLINKFAIL);
+    flags &= ~(omitMask | RPMVERIFY_FAILURES);
+
+    /*
+     * Verify file security context.
+     */
+/*@-branchstate@*/
+    if (selinuxEnabled == 1 && (flags & RPMVERIFY_CONTEXTS)) {
+	security_context_t con;
+
+	rc = lgetfilecon(fn, &con);
+	if (rc == -1)
+	    *res |= (RPMVERIFY_LGETFILECONFAIL|RPMVERIFY_CONTEXTS);
+	else {
+	    rpmsx sx = rpmtsREContext(ts);
+	    const char * fcontext;
+
+	    if (sx != NULL) {
+		/* Get file security context from patterns. */
+		fcontext = rpmsxFContext(sx, fn, fmode);
+		sx = rpmsxFree(sx);
+	    } else {
+		/* Get file security context from package. */
+		fcontext = rpmfiFContext(fi);
+	    }
+	    if (fcontext == NULL || strcmp(fcontext, con))
+		*res |= RPMVERIFY_CONTEXTS;
+	    freecon(con);
+	}
+    }
+/*@=branchstate@*/
 
     if (flags & RPMVERIFY_MD5) {
 	unsigned char md5sum[16];
@@ -217,13 +249,16 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
  */
 static int rpmVerifyScript(/*@unused@*/ QVA_t qva, rpmts ts,
 		rpmfi fi, /*@null@*/ FD_t scriptFd)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, scriptFd, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
 {
     rpmpsm psm = rpmpsmNew(ts, NULL, fi);
     int rc = 0;
 
+    if (psm == NULL)	/* XXX can't happen */
+	return rc;
+
     if (scriptFd != NULL)
 	rpmtsSetScriptFd(psm->ts, scriptFd);
 
@@ -248,16 +283,17 @@ static int rpmVerifyScript(/*@unused@*/ QVA_t qva, rpmts ts,
  * @return		0 no problems, 1 problems found
  */
 static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fi, fileSystem, internalState  @*/
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies ts, fi, fileSystem, internalState  @*/
 {
-    char buf[BUFSIZ];
-    char * t, * te;
+    int selinuxEnabled = rpmtsSELinuxEnabled(ts);
     rpmVerifyAttrs verifyResult = 0;
     /*@-type@*/ /* FIX: union? */
     rpmVerifyAttrs omitMask = ((qva->qva_flags & VERIFY_ATTRS) ^ VERIFY_ATTRS);
     /*@=type@*/
     int ec = 0;		/* assume no problems */
+    char * t, * te;
+    char buf[BUFSIZ];
     int i;
 
     te = t = buf;
@@ -282,7 +318,7 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 /*@=boundswrite@*/
 	if (rc) {
 	    if (!(fileAttrs & (RPMFILE_MISSINGOK|RPMFILE_GHOST)) || rpmIsVerbose()) {
-		sprintf(te, _("missing  %c %s"),
+		sprintf(te, _("missing   %c %s"),
 			((fileAttrs & RPMFILE_CONFIG)	? 'c' :
 			 (fileAttrs & RPMFILE_DOC)	? 'd' :
 			 (fileAttrs & RPMFILE_GHOST)	? 'g' :
@@ -293,11 +329,12 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 		te += strlen(te);
 		ec = rc;
 	    }
-	} else if (verifyResult) {
+	} else if (verifyResult || rpmIsVerbose()) {
 	    const char * size, * MD5, * link, * mtime, * mode;
-	    const char * group, * user, * rdev;
+	    const char * group, * user, * rdev, *ctxt;
 	    /*@observer@*/ static const char *const aok = ".";
 	    /*@observer@*/ static const char *const unknown = "?";
+	    /*@observer@*/ static const char *const ctxt_ignore = " ";
 
 	    ec = 1;
 
@@ -309,6 +346,10 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 #define	_verifyfile(_RPMVERIFY_F, _C)	\
 	((verifyResult & RPMVERIFY_READFAIL) ? unknown : \
 	 (verifyResult & _RPMVERIFY_F) ? _C : aok)
+#define	_verifyctxt(_RPMVERIFY_F, _C)	\
+	((selinuxEnabled != 1 ? ctxt_ignore : \
+	 (verifyResult & RPMVERIFY_LGETFILECONFAIL) ? unknown : \
+	 (verifyResult & _RPMVERIFY_F) ? _C : aok))
 	
 	    MD5 = _verifyfile(RPMVERIFY_MD5, "5");
 	    size = _verify(RPMVERIFY_FILESIZE, "S");
@@ -318,13 +359,15 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	    user = _verify(RPMVERIFY_USER, "U");
 	    group = _verify(RPMVERIFY_GROUP, "G");
 	    mode = _verify(RPMVERIFY_MODE, "M");
+	    ctxt = _verifyctxt(RPMVERIFY_CONTEXTS, "C");
 
-#undef _verify
-#undef _verifylink
+#undef _verifyctxt
 #undef _verifyfile
+#undef _verifylink
+#undef _verify
 
-	    sprintf(te, "%s%s%s%s%s%s%s%s %c %s",
-			size, mode, MD5, rdev, link, user, group, mtime, 
+	    sprintf(te, "%s%s%s%s%s%s%s%s%s %c %s",
+			size, mode, MD5, rdev, link, user, group, mtime, ctxt,
 			((fileAttrs & RPMFILE_CONFIG)	? 'c' :
 			 (fileAttrs & RPMFILE_DOC)	? 'd' :
 			 (fileAttrs & RPMFILE_GHOST)	? 'g' :
@@ -359,7 +402,7 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
  */
 static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 		Header h)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmps ps;
@@ -479,6 +522,20 @@ int rpmcliVerify(rpmts ts, QVA_t qva, const char ** argv)
 	vsflags |= RPMVSF_NOHDRCHK;
     vsflags &= ~RPMVSF_NEEDPAYLOAD;
 
+    /* Initialize security context patterns (if not already done). */
+    if (qva->qva_flags & VERIFY_CONTEXTS) {
+	rpmsx sx = rpmtsREContext(ts);
+	if (sx == NULL) {
+	    arg = rpmGetPath("%{?_verify_file_context_path}", NULL);
+	    if (arg != NULL && *arg != '\0') {
+		sx = rpmsxNew(arg);
+		(void) rpmtsSetREContext(ts, sx);
+	    }
+	    arg = _free(arg);
+	}
+	sx = rpmsxFree(sx);
+    }
+
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     if (qva->qva_source == RPMQV_ALL) {
 	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
