@@ -1,15 +1,9 @@
-/** \ingroup python
+/** \ingroup py_c
  * \file python/header-py.c
  */
 
 #include "system.h"
 
-#include "Python.h"
-#ifdef __LCLINT__
-#undef  PyObject_HEAD
-#define PyObject_HEAD   int _PyObjectHead;
-#endif
-
 #include "rpmio_internal.h"
 #include "rpmcli.h"	/* XXX for rpmCheckSig */
 
@@ -25,14 +19,65 @@
 
 #include "debug.h"
 
+/** \ingroup python
+ * \class Rpm
+ * \brief START HERE / RPM base module for the Python API
+ *
+ * The rpm base module provides the main starting point for
+ * accessing RPM from Python. For most usage, call
+ * the TransactionSet method to get a transaction set (rpmts).
+ *
+ * For example:
+ * \code
+ *	import rpm
+ *
+ *	ts = rpm.TransactionSet()
+ * \endcode
+ *
+ * The transaction set will open the RPM database as needed, so
+ * in most cases, you do not need to explicitly open the
+ * database. The transaction set is the workhorse of RPM.
+ *
+ * You can open another RPM database, such as one that holds
+ * all packages for a given Linux distribution, to provide
+ * packages used to solve dependencies. To do this, use
+ * the following code:
+ *
+ * \code
+ * rpm.addMacro('_dbpath', '/path/to/alternate/database')
+ * solvets = rpm.TransactionSet()
+ * solvets.openDB()
+ * rpm.delMacro('_dbpath')
+ *
+ * # Open default database
+ * ts = rpm.TransactionSet()
+ * \endcode
+ *
+ * This code gives you access to two RPM databases through
+ * two transaction sets (rpmts): ts is a transaction set
+ * associated with the default RPM database and solvets
+ * is a transaction set tied to an alternate database, which
+ * is very useful for resolving dependencies.
+ *
+ * The rpm methods used here are:
+ *
+ * - addMacro(macro, value)
+ * @param macro   Name of macro to add
+ * @param value   Value for the macro
+ *
+ * - delMacro(macro)
+ * @param macro   Name of macro to delete
+ *
+ */
+
 /** \ingroup python
  * \class Rpmhdr
  * \brief A python header object represents an RPM package header.
- * 
+ *
  * All RPM packages have headers that provide metadata for the package.
  * Header objects can be returned by database queries or loaded from a
  * binary package on disk.
- * 
+ *
  * The ts.hdrFromFdno() function returns the package header from a
  * package on disk, verifying package signatures and digests of the
  * package while reading.
@@ -53,7 +98,7 @@
  *	else:
  *	   print "header is from a binary package"
  * \endcode
- * 
+ *
  * The Python interface to the header data is quite elegant.  It
  * presents the data in a dictionary form.  We'll take the header we
  * just loaded and access the data within it:
@@ -88,7 +133,7 @@
  */
 /*@{*/
 
-/** \ingroup python
+/** \ingroup py_c
  */
 struct hdrObject_s {
     PyObject_HEAD
@@ -110,7 +155,7 @@ struct hdrObject_s {
     return 0;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrKeyList(hdrObject * s, PyObject * args)
 	/*@*/
@@ -144,7 +189,7 @@ static PyObject * hdrKeyList(hdrObject * s, PyObject * args)
     return list;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
 	/*@*/
@@ -156,7 +201,7 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
     static char *kwlist[] = { "legacyHeader", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, keywords, "|i", kwlist, &legacy))
-	return NULL; 
+	return NULL;
 
     h = headerLink(s->h);
     /* XXX this legacy switch is a hack, needs to be removed. */
@@ -179,7 +224,7 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
     return rc;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrExpandFilelist(hdrObject * s, PyObject * args)
 	/*@*/
@@ -190,7 +235,7 @@ static PyObject * hdrExpandFilelist(hdrObject * s, PyObject * args)
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrCompressFilelist(hdrObject * s, PyObject * args)
 	/*@*/
@@ -202,7 +247,7 @@ static PyObject * hdrCompressFilelist(hdrObject * s, PyObject * args)
 }
 
 /* make a header with _all_ the tags we need */
-/** \ingroup python
+/** \ingroup py_c
  */
 static void mungeFilelist(Header h)
 	/*@*/
@@ -214,7 +259,7 @@ static void mungeFilelist(Header h)
 	|| !headerIsEntry (h, RPMTAG_DIRNAMES)
 	|| !headerIsEntry (h, RPMTAG_DIRINDEXES))
 	compressFilelist(h);
-    
+
     rpmfiBuildFNames(h, RPMTAG_BASENAMES, &fileNames, &count);
 
     if (fileNames == NULL || count <= 0)
@@ -227,7 +272,7 @@ static void mungeFilelist(Header h)
     fileNames = _free(fileNames);
 }
 
-/** 
+/**
  */
 static PyObject * rhnUnload(hdrObject * s, PyObject * args)
 	/*@*/
@@ -289,7 +334,7 @@ static PyObject * rhnUnload(hdrObject * s, PyObject * args)
     return rc;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrFullFilelist(hdrObject * s, PyObject * args)
 	/*@*/
@@ -303,7 +348,7 @@ static PyObject * hdrFullFilelist(hdrObject * s, PyObject * args)
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdrSprintf(hdrObject * s, PyObject * args)
 	/*@*/
@@ -341,7 +386,7 @@ static long hdr_hash(PyObject * h)
     return (long) h;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 /*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef hdr_methods[] = {
@@ -370,15 +415,20 @@ static struct PyMethodDef hdr_methods[] = {
     {NULL,		NULL}		/* sentinel */
 };
 
-/** \ingroup python
- */
-static PyObject * hdr_getattr(hdrObject * s, char * name)
+static PyObject * hdr_getattro(PyObject * o, PyObject * n)
 	/*@*/
 {
-    return Py_FindMethod(hdr_methods, (PyObject * ) s, name);
+    return PyObject_GenericGetAttr(o, n);
 }
 
-/** \ingroup python
+static int hdr_setattro(PyObject * o, PyObject * n, PyObject * v)
+	/*@*/
+{
+    return PyObject_GenericSetAttr(o, n, v);
+}
+
+
+/** \ingroup py_c
  */
 static void hdr_dealloc(hdrObject * s)
 	/*@*/
@@ -390,7 +440,7 @@ static void hdr_dealloc(hdrObject * s)
     PyObject_Del(s);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 long tagNumFromPyObject (PyObject *item)
 {
@@ -408,7 +458,7 @@ long tagNumFromPyObject (PyObject *item)
     return -1;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	/*@*/
@@ -440,6 +490,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	}
     }
 
+    /* Retrieve data from extension or header. */
     if (ext) {
         ext->u.tagFunction(s->h, &type, (const void **) &data, &count, &freeData);
     } else {
@@ -448,10 +499,22 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
             return NULL;
         }
         
-        if (!rpmHeaderGetEntry(s->h, tag, &type, &data, &count)) {
-            Py_INCREF(Py_None);
-            return Py_None;
-        }
+	if (!rpmHeaderGetEntry(s->h, tag, &type, &data, &count)) {
+	    switch (tag) {
+	    case RPMTAG_EPOCH:
+	    case RPMTAG_NAME:
+	    case RPMTAG_VERSION:
+	    case RPMTAG_RELEASE:
+	    case RPMTAG_ARCH:
+	    case RPMTAG_OS:
+		Py_INCREF(Py_None);
+		return Py_None;
+		break;
+	    default:
+		return PyList_New(0);
+		break;
+	    }
+	}
     }
 
     switch (tag) {
@@ -469,6 +532,18 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     case RPMTAG_ROOT:
     case RPMTAG_FILEUSERNAME:
     case RPMTAG_FILEGROUPNAME:
+    case RPMTAG_REQUIRENAME:
+    case RPMTAG_REQUIREFLAGS:
+    case RPMTAG_REQUIREVERSION:
+    case RPMTAG_PROVIDENAME:
+    case RPMTAG_PROVIDEFLAGS:
+    case RPMTAG_PROVIDEVERSION:
+    case RPMTAG_OBSOLETENAME:
+    case RPMTAG_OBSOLETEFLAGS:
+    case RPMTAG_OBSOLETEVERSION:
+    case RPMTAG_CONFLICTNAME:
+    case RPMTAG_CONFLICTFLAGS:
+    case RPMTAG_CONFLICTVERSION:
 	forceArray = 1;
 	break;
     case RPMTAG_SUMMARY:
@@ -567,7 +642,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     return o;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 /*@unchecked@*/ /*@observer@*/
 static PyMappingMethods hdr_as_mapping = {
@@ -581,7 +656,7 @@ static PyMappingMethods hdr_as_mapping = {
 static char hdr_doc[] =
 "";
 
-/** \ingroup python
+/** \ingroup py_c
  */
 /*@unchecked@*/ /*@observer@*/
 PyTypeObject hdr_Type = {
@@ -592,7 +667,7 @@ PyTypeObject hdr_Type = {
 	0,				/* tp_itemsize */
 	(destructor) hdr_dealloc, 	/* tp_dealloc */
 	0,				/* tp_print */
-	(getattrfunc) hdr_getattr, 	/* tp_getattr */
+	(getattrfunc) 0, 		/* tp_getattr */
 	0,				/* tp_setattr */
 	(cmpfunc) hdr_compare,		/* tp_compare */
 	0,				/* tp_repr */
@@ -602,8 +677,8 @@ PyTypeObject hdr_Type = {
 	hdr_hash,			/* tp_hash */
 	0,				/* tp_call */
 	0,				/* tp_str */
-	0,				/* tp_getattro */
-	0,				/* tp_setattro */
+	(getattrofunc) hdr_getattro,	/* tp_getattro */
+	(setattrofunc) hdr_setattro,	/* tp_setattro */
 	0,				/* tp_as_buffer */
 	Py_TPFLAGS_DEFAULT,		/* tp_flags */
 	hdr_doc,			/* tp_doc */
@@ -656,7 +731,7 @@ PyObject * hdrLoad(PyObject * self, PyObject * args)
     int len;
 
     if (!PyArg_ParseTuple(args, "s#", &obj, &len)) return NULL;
-    
+
     /* malloc is needed to avoid surprises from data swab in headerLoad(). */
     copy = malloc(len);
     if (copy == NULL) {
@@ -689,7 +764,7 @@ PyObject * rhnLoad(PyObject * self, PyObject * args)
     int len;
 
     if (!PyArg_ParseTuple(args, "s#", &obj, &len)) return NULL;
-    
+
     /* malloc is needed to avoid surprises from data swab in headerLoad(). */
     copy = malloc(len);
     if (copy == NULL) {
@@ -861,6 +936,7 @@ int rpmMergeHeaders(PyObject * list, FD_t fd, int matchTag)
 	}
 
 	headerFreeIterator(hi);
+	h = headerFree(h);
 
 	Py_BEGIN_ALLOW_THREADS
 	h = headerRead(fd, HEADER_MAGIC_YES);
@@ -913,34 +989,33 @@ PyObject * versionCompare (PyObject * self, PyObject * args)
 
 /**
  */
+static int compare_values(const char *str1, const char *str2)
+{
+    if (!str1 && !str2)
+	return 0;
+    else if (str1 && !str2)
+	return 1;
+    else if (!str1 && str2)
+	return -1;
+    return rpmvercmp(str1, str2);
+}
+
 PyObject * labelCompare (PyObject * self, PyObject * args)
 {
     char *v1, *r1, *e1, *v2, *r2, *e2;
     int rc;
 
     if (!PyArg_ParseTuple(args, "(zzz)(zzz)",
-			  &e1, &v1, &r1,
-			  &e2, &v2, &r2)) return NULL;
+			&e1, &v1, &r1, &e2, &v2, &r2))
+	return NULL;
 
-    if (e1 && !e2)
-	return Py_BuildValue("i", 1);
-    else if (!e1 && e2)
-	return Py_BuildValue("i", -1);
-    else if (e1 && e2) {
-	int ep1, ep2;
-	ep1 = atoi (e1);
-	ep2 = atoi (e2);
-	if (ep1 < ep2)
-	    return Py_BuildValue("i", -1);
-	else if (ep1 > ep2)
-	    return Py_BuildValue("i", 1);
+    rc = compare_values(e1, e2);
+    if (!rc) {
+	rc = compare_values(v1, v2);
+	if (!rc)
+	    rc = compare_values(r1, r2);
     }
-
-    rc = rpmvercmp(v1, v2);
-    if (rc)
-	return Py_BuildValue("i", rc);
-
-    return Py_BuildValue("i", rpmvercmp(r1, r2));
+    return Py_BuildValue("i", rc);
 }
 
 /*@}*/
