@@ -1,15 +1,9 @@
-/** \ingroup python
+/** \ingroup py_c
  * \file python/rpmfi-py.c
  */
 
 #include "system.h"
 
-#include "Python.h"
-#ifdef __LCLINT__
-#undef  PyObject_HEAD
-#define PyObject_HEAD   int _PyObjectHead;
-#endif
-
 #include <rpmlib.h>
 
 #include "header-py.h"
@@ -19,6 +13,7 @@
 
 /*@access rpmfi @*/
 
+/*@null@*/
 static PyObject *
 rpmfi_Debug(/*@unused@*/ rpmfiObject * s, PyObject * args)
 	/*@globals _Py_NoneStruct @*/
@@ -29,6 +24,7 @@ rpmfi_Debug(/*@unused@*/ rpmfiObject * s, PyObject * args)
     return Py_None;
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FC(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -37,6 +33,7 @@ rpmfi_FC(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFC(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FX(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -45,6 +42,7 @@ rpmfi_FX(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFX(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_DC(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -53,6 +51,7 @@ rpmfi_DC(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiDC(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_DX(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -61,6 +60,7 @@ rpmfi_DX(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiDX(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_BN(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -69,6 +69,7 @@ rpmfi_BN(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiBN(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_DN(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -77,6 +78,7 @@ rpmfi_DN(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiDN(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FN(rpmfiObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -85,6 +87,7 @@ rpmfi_FN(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiFN(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FFlags(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -93,6 +96,7 @@ rpmfi_FFlags(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFFlags(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_VFlags(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -101,6 +105,7 @@ rpmfi_VFlags(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiVFlags(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FMode(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -109,6 +114,7 @@ rpmfi_FMode(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFMode(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FState(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -118,6 +124,7 @@ rpmfi_FState(rpmfiObject * s, PyObject * args)
 }
 
 /* XXX rpmfiMD5 */
+/*@null@*/
 static PyObject *
 rpmfi_MD5(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -126,15 +133,17 @@ rpmfi_MD5(rpmfiObject * s, PyObject * args)
     char fmd5[33];
     char * t;
     int i;
-    
+
     if (!PyArg_ParseTuple(args, ":MD5")) return NULL;
     MD5 = rpmfiMD5(s->fi);
+    if (MD5 != NULL)
     for (i = 0, t = fmd5; i < 16; i++, t += 2)
 	sprintf(t, "%02x", MD5[i]);
     *t = '\0';
     return Py_BuildValue("s", xstrdup(fmd5));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FLink(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -143,6 +152,7 @@ rpmfi_FLink(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiFLink(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FSize(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -151,6 +161,7 @@ rpmfi_FSize(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFSize(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FRdev(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -159,6 +170,7 @@ rpmfi_FRdev(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFRdev(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FMtime(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -167,6 +179,7 @@ rpmfi_FMtime(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFMtime(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FUser(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -175,6 +188,7 @@ rpmfi_FUser(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiFUser(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FGroup(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -183,6 +197,7 @@ rpmfi_FGroup(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("s", xstrdup(rpmfiFGroup(s->fi)));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FColor(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -191,6 +206,7 @@ rpmfi_FColor(rpmfiObject * s, PyObject * args)
     return Py_BuildValue("i", rpmfiFColor(s->fi));
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_FClass(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -213,6 +229,7 @@ rpmfi_iter(rpmfiObject * s, /*@unused@*/ PyObject * args)
 }
 #endif
 
+/*@null@*/
 static PyObject *
 rpmfi_iternext(rpmfiObject * s)
 	/*@globals _Py_NoneStruct @*/
@@ -222,7 +239,7 @@ rpmfi_iternext(rpmfiObject * s)
 
     /* Reset loop indices on 1st entry. */
     if (!s->active) {
-	rpmfiInit(s->fi, 0);
+	s->fi = rpmfiInit(s->fi, 0);
 	s->active = 1;
     }
 
@@ -311,6 +328,7 @@ rpmfi_Next(rpmfiObject * s, /*@unused@*/ PyObject * args)
 }
 
 #ifdef	NOTYET
+/*@null@*/
 static PyObject *
 rpmfi_NextD(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -321,6 +339,7 @@ rpmfi_NextD(rpmfiObject * s, PyObject * args)
 	return Py_None;
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_InitD(rpmfiObject * s, PyObject * args)
 	/*@*/
@@ -410,17 +429,22 @@ rpmfi_print(rpmfiObject * s, FILE * fp, /*@unused@*/ int flags)
     if (!(s && s->fi))
 	return -1;
 
-    rpmfiInit(s->fi, 0);
+    s->fi = rpmfiInit(s->fi, 0);
     while (rpmfiNext(s->fi) >= 0)
 	fprintf(fp, "%s\n", rpmfiFN(s->fi));
     return 0;
 }
 
-static PyObject *
-rpmfi_getattr(rpmfiObject * s, char * name)
+static PyObject * rpmfi_getattro(PyObject * o, PyObject * n)
 	/*@*/
 {
-    return Py_FindMethod(rpmfi_methods, (PyObject *)s, name);
+    return PyObject_GenericGetAttr(o, n);
+}
+
+static int rpmfi_setattro(PyObject * o, PyObject * n, PyObject * v)
+	/*@*/
+{
+    return PyObject_GenericSetAttr(o, n, v);
 }
 
 static int
@@ -430,6 +454,7 @@ rpmfi_length(rpmfiObject * s)
     return rpmfiFC(s->fi);
 }
 
+/*@null@*/
 static PyObject *
 rpmfi_subscript(rpmfiObject * s, PyObject * key)
 	/*@modifies s @*/
@@ -469,7 +494,7 @@ PyTypeObject rpmfi_Type = {
 	/* methods */
 	(destructor) rpmfi_dealloc,	/* tp_dealloc */
 	(printfunc) rpmfi_print,	/* tp_print */
-	(getattrfunc) rpmfi_getattr,	/* tp_getattr */
+	(getattrfunc)0,			/* tp_getattr */
 	(setattrfunc)0,			/* tp_setattr */
 	(cmpfunc)0,			/* tp_compare */
 	(reprfunc)0,			/* tp_repr */
@@ -479,8 +504,8 @@ PyTypeObject rpmfi_Type = {
 	(hashfunc)0,			/* tp_hash */
 	(ternaryfunc)0,			/* tp_call */
 	(reprfunc)0,			/* tp_str */
-	0,				/* tp_getattro */
-	0,				/* tp_setattro */
+	(getattrofunc) rpmfi_getattro,	/* tp_getattro */
+	(setattrofunc) rpmfi_setattro,	/* tp_setattro */
 	0,				/* tp_as_buffer */
 	Py_TPFLAGS_DEFAULT,		/* tp_flags */
 	rpmfi_doc,			/* tp_doc */
