@@ -1,15 +1,9 @@
-/** \ingroup python
+/** \ingroup py_c
  * \file python/rpmts-py.c
  */
 
 #include "system.h"
 
-#include "Python.h"
-#ifdef __LCLINT__
-#undef  PyObject_HEAD
-#define PyObject_HEAD   int _PyObjectHead;
-#endif
-
 #include <rpmcli.h>
 #include <rpmpgp.h>
 #include <rpmdb.h>
@@ -43,7 +37,7 @@ static int _rpmts_debug = 0;
  * \name Class: Rpmts
  * \class Rpmts
  * \brief A python rpm.ts object represents an RPM transaction set.
- * 
+ *
  * The transaction set is the workhorse of RPM.  It performs the
  * installation and upgrade of packages.  The rpm.ts object is
  * instantiated by the TransactionSet function in the rpm module.
@@ -141,14 +135,14 @@ static int _rpmts_debug = 0;
  * - ts.setProbFilter(ignoreSet) Set transaction set problem filter.
  * @param problemSetFilter - control bit(s) to ignore classes of problems,
  *		a logical or of one or more of the following bit(s):
- *	- rpm.RPMPROB_FILTER_IGNOREOS - 
- *	- rpm.RPMPROB_FILTER_IGNOREARCH - 
- *	- rpm.RPMPROB_FILTER_REPLACEPKG - 
- *	- rpm.RPMPROB_FILTER_FORCERELOCATE - 
- *	- rpm.RPMPROB_FILTER_REPLACENEWFILES - 
- *	- rpm.RPMPROB_FILTER_REPLACEOLDFILES - 
- *	- rpm.RPMPROB_FILTER_OLDPACKAGE - 
- *	- rpm.RPMPROB_FILTER_DISKSPACE - 
+ *	- rpm.RPMPROB_FILTER_IGNOREOS -
+ *	- rpm.RPMPROB_FILTER_IGNOREARCH -
+ *	- rpm.RPMPROB_FILTER_REPLACEPKG -
+ *	- rpm.RPMPROB_FILTER_FORCERELOCATE -
+ *	- rpm.RPMPROB_FILTER_REPLACENEWFILES -
+ *	- rpm.RPMPROB_FILTER_REPLACEOLDFILES -
+ *	- rpm.RPMPROB_FILTER_OLDPACKAGE -
+ *	- rpm.RPMPROB_FILTER_DISKSPACE -
  * @return	previous ignoreSet
  *
  * - ts.run(callback,data) Attempt to execute a transaction set.
@@ -157,7 +151,7 @@ static int _rpmts_debug = 0;
  *	the ts.run() method.
  */
 
-/** \ingroup python
+/** \ingroup py_c
  */
 struct rpmtsCallbackType_s {
     PyObject * cb;
@@ -167,8 +161,9 @@ struct rpmtsCallbackType_s {
     PyThreadState *_save;
 };
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_Debug(/*@unused@*/ rpmtsObject * s, PyObject * args)
         /*@globals _Py_NoneStruct @*/
@@ -183,7 +178,7 @@ fprintf(stderr, "*** rpmts_Debug(%p) ts %p\n", s, s->ts);
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  * Add package to universe of possible packages to install in transaction set.
  * @param ts		transaction set
  * @param h		header
@@ -191,7 +186,8 @@ fprintf(stderr, "*** rpmts_Debug(%p) ts %p\n", s, s->ts);
  */
 static void rpmtsAddAvailableElement(rpmts ts, Header h,
 		/*@exposed@*/ /*@null@*/ fnpyKey key)
-	/*@modifies h, ts @*/
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies h, ts, rpmGlobalMacroContext @*/
 {
     int scareMem = 0;
     rpmds provides = rpmdsNew(h, RPMTAG_PROVIDENAME, scareMem);
@@ -208,8 +204,9 @@ fprintf(stderr, "\tAddAvailable(%p) list %p\n", ts, ts->availablePackages);
 
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_AddInstall(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -217,7 +214,7 @@ rpmts_AddInstall(rpmtsObject * s, PyObject * args)
 {
     hdrObject * h;
     PyObject * key;
-    char * how = NULL;
+    char * how = "u";	/* XXX default to upgrade element if missing */
     int isUpgrade = 0;
 
     if (!PyArg_ParseTuple(args, "O!O|s:AddInstall", &hdr_Type, &h, &key, &how))
@@ -252,9 +249,10 @@ fprintf(stderr, "*** rpmts_AddInstall(%p,%p,%p,%s) ts %p\n", s, h, key, how, s->
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  * @todo Permit finer control (i.e. not just --allmatches) of deleted elments.
  */
+/*@null@*/
 static PyObject *
 rpmts_AddErase(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -263,7 +261,7 @@ rpmts_AddErase(rpmtsObject * s, PyObject * args)
     PyObject * o;
     int count;
     rpmdbMatchIterator mi;
-    
+
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_AddErase(%p) ts %p\n", s, s->ts);
 
@@ -313,7 +311,7 @@ fprintf(stderr, "*** rpmts_AddErase(%p) ts %p\n", s, s->ts);
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static int
 rpmts_SolveCallback(rpmts ts, rpmds ds, const void * data)
@@ -350,8 +348,9 @@ fprintf(stderr, "*** rpmts_SolveCallback(%p,%p,%p) \"%s\"\n", ts, ds, data, rpmd
     return res;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_Check(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -394,7 +393,7 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 
     PyEval_RestoreThread(cbInfo._save);
 
-    if (ps) {
+    if (ps != NULL) {
 	list = PyList_New(0);
 
 	/* XXX TODO: rpmlib >= 4.0.3 can return multiple suggested keys. */
@@ -411,14 +410,19 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 				   conflicts[i].suggestedPkgs[0] : Py_None,
 			       conflicts[i].sense);
 #else
-	    char * byName, * byVersion, * byRelease;
+	    char * byName, * byVersion, * byRelease, * byArch;
 	    char * needsName, * needsOP, * needsVersion;
 	    int needsFlags, sense;
 	    fnpyKey key;
-	    
+
 	    p = ps->probs + i;
 
+	    /* XXX autorelocated i386 on ia64, fix system-config-packages! */
+	    if (p->type == RPMPROB_BADRELOCATE)
+		continue;
 	    byName = p->pkgNEVR;
+	    if ((byArch= strrchr(byName, '.')) != NULL)
+		*byArch++ = '\0';
 	    if ((byRelease = strrchr(byName, '-')) != NULL)
 		*byRelease++ = '\0';
 	    if ((byVersion = strrchr(byName, '-')) != NULL)
@@ -444,7 +448,7 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 		    else if (*needsOP == '=')	needsFlags |= RPMSENSE_EQUAL;
 		}
 	    }
-	    
+
 	    cf = Py_BuildValue("((sss)(ss)iOi)", byName, byVersion, byRelease,
 			       needsName, needsVersion, needsFlags,
 			       (key != NULL ? key : Py_None),
@@ -463,11 +467,13 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_Order(rpmtsObject * s, PyObject * args)
-	/*@modifies s @*/
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies s, rpmGlobalMacroContext @*/
 {
     int rc;
 
@@ -483,8 +489,9 @@ fprintf(stderr, "*** rpmts_Order(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_Clean(rpmtsObject * s, PyObject * args)
 	/*@globals _Py_NoneStruct @*/
@@ -501,8 +508,9 @@ fprintf(stderr, "*** rpmts_Clean(%p) ts %p\n", s, s->ts);
     return Py_None;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_IDTXload(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -521,29 +529,35 @@ fprintf(stderr, "*** rpmts_IDTXload(%p) ts %p\n", s, s->ts);
     idtx = IDTXload(s->ts, tag);
     Py_END_ALLOW_THREADS
 
+/*@-branchstate@*/
     if (idtx == NULL || idtx->nidt <= 0) {
 	Py_INCREF(Py_None);
 	result = Py_None;
     } else {
 	PyObject * tuple;
+	PyObject * ho;
 	IDT idt;
 	int i;
 
 	result = PyTuple_New(idtx->nidt);
 	for (i = 0; i < idtx->nidt; i++) {
 	    idt = idtx->idt + i;
-	    tuple = Py_BuildValue("(iOi)", idt->val.u32, hdr_Wrap(idt->h), idt->instance);
+	    ho = (PyObject *) hdr_Wrap(idt->h);
+	    tuple = Py_BuildValue("(iOi)", idt->val.u32, ho, idt->instance);
 	    PyTuple_SET_ITEM(result,  i, tuple);
+	    Py_DECREF(ho);
 	}
     }
+/*@=branchstate@*/
 
     idtx = IDTXfree(idtx);
 
     return result;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_IDTXglob(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -565,29 +579,35 @@ fprintf(stderr, "*** rpmts_IDTXglob(%p) ts %p\n", s, s->ts);
     globstr = _free(globstr);
     Py_END_ALLOW_THREADS
 
+/*@-branchstate@*/
     if (idtx == NULL || idtx->nidt <= 0) {
 	Py_INCREF(Py_None);
 	result = Py_None;
     } else {
 	PyObject * tuple;
+	PyObject * ho;
 	IDT idt;
 	int i;
 
 	result = PyTuple_New(idtx->nidt);
 	for (i = 0; i < idtx->nidt; i++) {
 	    idt = idtx->idt + i;
-	    tuple = Py_BuildValue("(iOs)", idt->val.u32, hdr_Wrap(idt->h), idt->key);
+	    ho = (PyObject *) hdr_Wrap(idt->h);
+	    tuple = Py_BuildValue("(iOs)", idt->val.u32, ho, idt->key);
 	    PyTuple_SET_ITEM(result,  i, tuple);
+	    Py_DECREF(ho);
 	}
     }
+/*@=branchstate@*/
 
     idtx = IDTXfree(idtx);
 
     return result;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_Rollback(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
@@ -622,8 +642,9 @@ fprintf(stderr, "*** rpmts_Rollback(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_OpenDB(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
@@ -641,8 +662,9 @@ fprintf(stderr, "*** rpmts_OpenDB(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rpmtsOpenDB(s->ts, s->ts->dbmode));
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_CloseDB(rpmtsObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -660,8 +682,9 @@ fprintf(stderr, "*** rpmts_CloseDB(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_InitDB(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
@@ -681,8 +704,9 @@ fprintf(stderr, "*** rpmts_InitDB(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_RebuildDB(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
@@ -702,8 +726,9 @@ fprintf(stderr, "*** rpmts_RebuildDB(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_VerifyDB(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
@@ -723,12 +748,13 @@ fprintf(stderr, "*** rpmts_VerifyDB(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_HdrFromFdno(rpmtsObject * s, PyObject * args)
-	/*@globals rpmGlobalMacroContext, _Py_NoneStruct, fileSystem @*/
-	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies s, rpmGlobalMacroContext, fileSystem @*/
 {
     PyObject * result = NULL;
     Header h;
@@ -745,6 +771,7 @@ rpmts_HdrFromFdno(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_HdrFromFdno(%p) ts %p rc %d\n", s, s->ts, rpmrc);
 
+/*@-branchstate@*/
     switch (rpmrc) {
     case RPMRC_OK:
 	if (h)
@@ -752,11 +779,6 @@ fprintf(stderr, "*** rpmts_HdrFromFdno(%p) ts %p rc %d\n", s, s->ts, rpmrc);
 	h = headerFree(h);	/* XXX ref held by result */
 	break;
 
-    case RPMRC_NOTFOUND:
-	Py_INCREF(Py_None);
-	result = Py_None;
-	break;
-
     case RPMRC_NOKEY:
 	PyErr_SetString(pyrpmError, "public key not available");
 	break;
@@ -765,17 +787,20 @@ fprintf(stderr, "*** rpmts_HdrFromFdno(%p) ts %p rc %d\n", s, s->ts, rpmrc);
 	PyErr_SetString(pyrpmError, "public key not trusted");
 	break;
 
+    case RPMRC_NOTFOUND:
     case RPMRC_FAIL:
     default:
 	PyErr_SetString(pyrpmError, "error reading package header");
 	break;
     }
+/*@=branchstate@*/
 
     return result;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_HdrCheck(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -829,8 +854,9 @@ fprintf(stderr, "*** rpmts_HdrCheck(%p) ts %p\n", s, s->ts);
     return result;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_SetVSFlags(rpmtsObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -847,8 +873,9 @@ fprintf(stderr, "*** rpmts_SetVSFlags(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rpmtsSetVSFlags(s->ts, vsflags));
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_SetColor(rpmtsObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -865,8 +892,9 @@ fprintf(stderr, "*** rpmts_SetColor(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rpmtsSetColor(s->ts, tscolor));
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_PgpPrtPkts(rpmtsObject * s, PyObject * args)
 	/*@globals _Py_NoneStruct @*/
@@ -897,8 +925,9 @@ fprintf(stderr, "*** rpmts_PgpPrtPkts(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_PgpImportPubkey(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -929,8 +958,9 @@ fprintf(stderr, "*** rpmts_PgpImportPubkey(%p) ts %p\n", s, s->ts);
     return Py_BuildValue("i", rc);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static PyObject *
 rpmts_GetKeys(rpmtsObject * s, PyObject * args)
 	/*@globals _Py_NoneStruct @*/
@@ -966,8 +996,9 @@ fprintf(stderr, "*** rpmts_GetKeys(%p) ts %p\n", s, s->ts);
     return tuple;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
+/*@null@*/
 static void *
 rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
 		         const unsigned long amount, const unsigned long total,
@@ -1026,7 +1057,7 @@ rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
 	fd = fdDup(fdno);
 if (_rpmts_debug)
 fprintf(stderr, "\t%p = fdDup(%d)\n", fd, fdno);
-	
+
 	return fd;
     } else
     if (what == RPMCALLBACK_INST_CLOSE_FILE) {
@@ -1044,7 +1075,7 @@ fprintf(stderr, "\t%ld:%ld key %p\n", amount, total, pkgKey);
     return NULL;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * rpmts_SetFlags(rpmtsObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -1060,7 +1091,7 @@ fprintf(stderr, "*** rpmts_SetFlags(%p) ts %p transFlags %x\n", s, s->ts, transF
     return Py_BuildValue("i", rpmtsSetFlags(s->ts, transFlags));
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * rpmts_SetProbFilter(rpmtsObject * s, PyObject * args)
 	/*@modifies s @*/
@@ -1080,7 +1111,7 @@ fprintf(stderr, "*** rpmts_SetProbFilter(%p) ts %p ignoreSet %x\n", s, s->ts, ig
     return Py_BuildValue("i", oignoreSet);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * rpmts_Run(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
@@ -1106,6 +1137,20 @@ static PyObject * rpmts_Run(rpmtsObject * s, PyObject * args)
 	(void) rpmtsSetNotifyCallback(s->ts, rpmtsCallback, (void *) &cbInfo);
     }
 
+    /* Initialize security context patterns (if not already done). */
+    if (!(s->ts->transFlags & RPMTRANS_FLAG_NOCONTEXTS)) {
+	rpmsx sx = rpmtsREContext(s->ts);
+	if (sx == NULL) {
+	    const char *fn = rpmGetPath("%{?_install_file_context_path}", NULL);
+	    if (fn != NULL && *fn != '\0') {
+		sx = rpmsxNew(fn);
+		(void) rpmtsSetREContext(s->ts, sx);
+	    }
+	    fn = _free(fn);
+	}
+	sx = rpmsxFree(sx);
+    } 
+
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Run(%p) ts %p ignore %x\n", s, s->ts, s->ignoreSet);
@@ -1163,6 +1208,7 @@ fprintf(stderr, "*** rpmts_iter(%p) ts %p\n", s, s->ts);
 /**
  * @todo Add TR_ADDED filter to iterator.
  */
+/*@null@*/
 static PyObject *
 rpmts_iternext(rpmtsObject * s)
 	/*@modifies s @*/
@@ -1182,12 +1228,14 @@ fprintf(stderr, "*** rpmts_iternext(%p) ts %p tsi %p %d\n", s, s->ts, s->tsi, s-
     }
 
     te = rpmtsiNext(s->tsi, s->tsiFilter);
+/*@-branchstate@*/
     if (te != NULL) {
 	result = (PyObject *) rpmte_Wrap(te);
     } else {
 	s->tsi = rpmtsiFree(s->tsi);
 	s->tsiFilter = 0;
     }
+/*@=branchstate@*/
 
     return result;
 }
@@ -1217,28 +1265,47 @@ fprintf(stderr, "*** rpmts_Next(%p) ts %p\n", s, s->ts);
 
 /**
  */
+/*@null@*/
 static rpmmiObject *
 rpmts_Match(rpmtsObject * s, PyObject * args)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
     PyObject *TagN = NULL;
+    PyObject *Key = NULL;
     char *key = NULL;
+    long lkey = 0;
     int len = 0;
     int tag = RPMDBI_PACKAGES;
-    
+
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "|Ozi", &TagN, &key, &len))
+    if (!PyArg_ParseTuple(args, "|OO", &TagN, &Key))
 	return NULL;
 
     if (TagN && (tag = tagNumFromPyObject (TagN)) == -1) {
 	PyErr_SetString(PyExc_TypeError, "unknown tag type");
 	return NULL;
+
+    }
+
+    if (Key) {
+	if (PyString_Check(Key)) {
+	    key = PyString_AsString(Key);
+	    len = PyString_Size(Key);
+	} else if (PyInt_Check(Key)) {
+	    lkey = PyInt_AsLong(Key);
+	    key = (char *)&lkey;
+	    len = sizeof(lkey);
+	} else {
+	    PyErr_SetString(PyExc_TypeError, "unknown key type");
+	    return NULL;
+	}
     }
 
     /* XXX If not already opened, open the database O_RDONLY now. */
+    /* XXX FIXME: lazy default rdonly open also done by rpmtsInitIterator(). */
     if (s->ts->rdb == NULL) {
 	int rc = rpmtsOpenDB(s->ts, O_RDONLY);
 	if (rc || s->ts->rdb == NULL) {
@@ -1250,7 +1317,7 @@ fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
     return rpmmi_Wrap( rpmtsInitIterator(s->ts, tag, key, len) );
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 /*@-fullinitblock@*/
 /*@unchecked@*/ /*@observer@*/
@@ -1341,7 +1408,7 @@ static struct PyMethodDef rpmts_methods[] = {
 };
 /*@=fullinitblock@*/
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static void rpmts_dealloc(/*@only@*/ rpmtsObject * s)
 	/*@modifies *s @*/
@@ -1349,7 +1416,7 @@ static void rpmts_dealloc(/*@only@*/ rpmtsObject * s)
 
 if (_rpmts_debug)
 fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, s->ts->rdb);
-    rpmtsFree(s->ts);
+    s->ts = rpmtsFree(s->ts);
 
     if (s->scriptFd) Fclose(s->scriptFd);
     /* this will free the keyList, and decrement the ref count of all
@@ -1358,29 +1425,29 @@ fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, s->ts->rdb);
     PyObject_Del((PyObject *)s);
 }
 
-/** \ingroup python
- */
-static PyObject * rpmts_getattr(rpmtsObject * o, char * name)
+static PyObject * rpmts_getattro(PyObject * o, PyObject * n)
 	/*@*/
 {
-    return Py_FindMethod(rpmts_methods, (PyObject *) o, name);
+    return PyObject_GenericGetAttr(o, n);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
-static int rpmts_setattr(rpmtsObject * o, char * name, PyObject * val)
-	/*@modifies o @*/
+static int rpmts_setattro(PyObject * o, PyObject * n, PyObject * v)
+	/*@*/
 {
-    int i;
+    rpmtsObject *s = (rpmtsObject *)o;
+    char * name = PyString_AsString(n);
+    int fdno;
 
     if (!strcmp(name, "scriptFd")) {
-	if (!PyArg_Parse(val, "i", &i)) return 0;
-	if (i < 0) {
+	if (!PyArg_Parse(v, "i", &fdno)) return 0;
+	if (fdno < 0) {
 	    PyErr_SetString(PyExc_TypeError, "bad file descriptor");
 	    return -1;
 	} else {
-	    o->scriptFd = fdDup(i);
-	    rpmtsSetScriptFd(o->ts, o->scriptFd);
+	    s->scriptFd = fdDup(fdno);
+	    rpmtsSetScriptFd(s->ts, s->scriptFd);
 	}
     } else {
 	PyErr_SetString(PyExc_AttributeError, name);
@@ -1390,7 +1457,7 @@ static int rpmts_setattr(rpmtsObject * o, char * name, PyObject * val)
     return 0;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static int rpmts_init(rpmtsObject * s, PyObject *args, PyObject *kwds)
 	/*@globals rpmGlobalMacroContext @*/
@@ -1408,22 +1475,22 @@ fprintf(stderr, "*** rpmts_init(%p,%p,%p)\n", s, args, kwds);
     s->ts = rpmtsCreate();
     (void) rpmtsSetRootDir(s->ts, rootDir);
     (void) rpmtsSetVSFlags(s->ts, vsflags);
-    s->keyList = PyList_New(0);         
-    s->scriptFd = NULL;                 
+    s->keyList = PyList_New(0);
+    s->scriptFd = NULL;
     s->tsi = NULL;
     s->tsiFilter = 0;
 
     return 0;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static void rpmts_free(/*@only@*/ rpmtsObject * s)
 	/*@modifies s @*/
 {
 if (_rpmts_debug)
 fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, s->ts->rdb);
-    rpmtsFree(s->ts);
+    s->ts = rpmtsFree(s->ts);
 
     if (s->scriptFd)
 	Fclose(s->scriptFd);
@@ -1432,10 +1499,10 @@ fprintf(stderr, "%p -- ts %p db %p\n", s, s->ts, s->ts->rdb);
        the items on the list as well :-) */
     Py_DECREF(s->keyList);
 
-   PyObject_Del((PyObject *)s);
+    PyObject_Del((PyObject *)s);
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * rpmts_alloc(PyTypeObject * subtype, int nitems)
 	/*@*/
@@ -1447,7 +1514,7 @@ fprintf(stderr, "*** rpmts_alloc(%p,%d) ret %p\n", subtype, nitems, s);
     return s;
 }
 
-/** \ingroup python
+/** \ingroup py_c
  */
 static PyObject * rpmts_new(PyTypeObject * subtype, PyObject *args, PyObject *kwds)
 	/*@globals rpmGlobalMacroContext @*/
@@ -1473,7 +1540,7 @@ fprintf(stderr, "%p ++ ts %p db %p\n", s, s->ts, s->ts->rdb);
 static char rpmts_doc[] =
 "";
 
-/** \ingroup python
+/** \ingroup py_c
  */
 /*@-fullinitblock@*/
 PyTypeObject rpmts_Type = {
@@ -1484,8 +1551,8 @@ PyTypeObject rpmts_Type = {
 	0,				/* tp_itemsize */
 	(destructor) rpmts_dealloc, 	/* tp_dealloc */
 	0,				/* tp_print */
-	(getattrfunc) rpmts_getattr, 	/* tp_getattr */
-	(setattrfunc) rpmts_setattr,	/* tp_setattr */
+	(getattrfunc)0, 		/* tp_getattr */
+	(setattrfunc)0,			/* tp_setattr */
 	0,				/* tp_compare */
 	0,				/* tp_repr */
 	0,				/* tp_as_number */
@@ -1494,8 +1561,8 @@ PyTypeObject rpmts_Type = {
 	0,				/* tp_hash */
 	0,				/* tp_call */
 	0,				/* tp_str */
-	0,				/* tp_getattro */
-	0,				/* tp_setattro */
+	(getattrofunc) rpmts_getattro, 	/* tp_getattro */
+	(setattrofunc) rpmts_setattro,	/* tp_setattro */
 	0,				/* tp_as_buffer */
 	Py_TPFLAGS_DEFAULT,		/* tp_flags */
 	rpmts_doc,			/* tp_doc */
@@ -1517,7 +1584,7 @@ PyTypeObject rpmts_Type = {
 	(initproc) rpmts_init,		/* tp_init */
 	(allocfunc) rpmts_alloc,	/* tp_alloc */
 	(newfunc) rpmts_new,		/* tp_new */
-	(destructor) rpmts_free,	/* tp_free */
+	rpmts_free,			/* tp_free */
 	0,				/* tp_is_gc */
 #endif
 };
