@@ -1,10 +1,8 @@
-/*@-branchstate@*/
+/*@-boundsread@*/
 /** \ingroup rpmrc rpmio
  * \file rpmio/macro.c
  */
 
-/*@unused@*/ static int _debug = 0;
-
 #include "system.h"
 #include <stdarg.h>
 
@@ -46,15 +44,26 @@ typedef	FILE * FD_t;
 #include <rpmmessages.h>
 #include <rpmerr.h>
 
+#ifdef	WITH_LUA
+#include <rpmlua.h>
+#endif
+
 #endif
 
 #include <rpmmacro.h>
 
 #include "debug.h"
 
+#if defined(__LCLINT__)
+/*@-exportheader@*/
+extern const unsigned short int **__ctype_b_loc (void) /*@*/;
+/*@=exportheader@*/
+#endif
+
 /*@access FD_t@*/		/* XXX compared with NULL */
 /*@access MacroContext@*/
 /*@access MacroEntry@*/
+/*@access rpmlua @*/
 
 static struct MacroContext_s rpmGlobalMacroContext_s;
 /*@-compmempass@*/
@@ -70,35 +79,35 @@ MacroContext rpmCLIMacroContext = &rpmCLIMacroContext_s;
  * Macro expansion state.
  */
 typedef /*@abstract@*/ struct MacroBuf_s {
-/*@shared@*/ const char * s;	/*!< Text to expand. */
-/*@shared@*/ char * t;		/*!< Expansion buffer. */
+/*@kept@*/ /*@exposed@*/
+    const char * s;		/*!< Text to expand. */
+/*@shared@*/
+    char * t;			/*!< Expansion buffer. */
     size_t nb;			/*!< No. bytes remaining in expansion buffer. */
     int depth;			/*!< Current expansion depth. */
     int macro_trace;		/*!< Pre-print macro to expand? */
     int expand_trace;		/*!< Post-print macro expansion? */
-/*@shared@*/ /*@null@*/ void * spec;	/*!< (future) %file expansion info?. */
-/*@dependent@*/ MacroContext mc;
+/*@kept@*/ /*@exposed@*/ /*@null@*/
+    void * spec;		/*!< (future) %file expansion info?. */
+/*@kept@*/ /*@exposed@*/
+    MacroContext mc;
 } * MacroBuf;
 
 #define SAVECHAR(_mb, _c) { *(_mb)->t = (_c), (_mb)->t++, (_mb)->nb--; }
 
 /*@-exportlocal -exportheadervar@*/
 
-#define	MAX_MACRO_DEPTH	16
+#define	_MAX_MACRO_DEPTH	16
 /*@unchecked@*/
-int max_macro_depth = MAX_MACRO_DEPTH;
+int max_macro_depth = _MAX_MACRO_DEPTH;
 
-#ifdef	DEBUG_MACROS
+#define	_PRINT_MACRO_TRACE	0
 /*@unchecked@*/
-int print_macro_trace = 0;
+int print_macro_trace = _PRINT_MACRO_TRACE;
+
+#define	_PRINT_EXPAND_TRACE	0
 /*@unchecked@*/
-int print_expand_trace = 0;
-#else
-/*@unchecked@*/
-int print_macro_trace = 0;
-/*@unchecked@*/
-int print_expand_trace = 0;
-#endif
+int print_expand_trace = _PRINT_EXPAND_TRACE;
 /*@=exportlocal =exportheadervar@*/
 
 #define	MACRO_CHUNK_SIZE	16
@@ -106,11 +115,9 @@ int print_expand_trace = 0;
 /* forward ref */
 static int expandMacro(MacroBuf mb)
 	/*@globals rpmGlobalMacroContext,
-		print_macro_trace, print_expand_trace,
-		fileSystem @*/
+		print_macro_trace, print_expand_trace, h_errno, fileSystem @*/
 	/*@modifies mb, rpmGlobalMacroContext,
-		print_macro_trace, print_expand_trace,
-		fileSystem @*/;
+		print_macro_trace, print_expand_trace, fileSystem @*/;
 
 /**
  * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
@@ -153,6 +160,7 @@ compareMacroName(const void * ap, const void * bp)
  * Enlarge macro table.
  * @param mc		macro context
  */
+/*@-boundswrite@*/
 static void
 expandMacroTable(MacroContext mc)
 	/*@modifies mc @*/
@@ -170,6 +178,7 @@ expandMacroTable(MacroContext mc)
     }
     memset(&mc->macroTable[mc->firstFree], 0, MACRO_CHUNK_SIZE * sizeof(*(mc->macroTable)));
 }
+/*@=boundswrite@*/
 
 /**
  * Sort entries in macro table.
@@ -233,28 +242,32 @@ rpmDumpMacroTable(MacroContext mc, FILE * fp)
  * Find entry in macro table.
  * @param mc		macro context
  * @param name		macro name
- * @param namelen	no. of byes
+ * @param namelen	no. of bytes
  * @return		address of slot in macro table with name (or NULL)
  */
-/*@-mustmod@*/ /* LCL: segfault with modifies nothing annotation */
-/*@dependent@*/ /*@null@*/ static MacroEntry *
+/*@-boundswrite@*/
+/*@dependent@*/ /*@null@*/
+static MacroEntry *
 findEntry(MacroContext mc, const char * name, size_t namelen)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies rpmGlobalMacroContext @*/
+	/*@*/
 {
     MacroEntry key, *ret;
     struct MacroEntry_s keybuf;
     char namebuf[1024];
 
+/*@-globs@*/
     if (mc == NULL) mc = rpmGlobalMacroContext;
+/*@=globs@*/
     if (mc->macroTable == NULL || mc->firstFree == 0)
 	return NULL;
 
+/*@-branchstate@*/
     if (namelen > 0) {
 	strncpy(namebuf, name, namelen);
 	namebuf[namelen] = '\0';
 	name = namebuf;
     }
+/*@=branchstate@*/
     
     key = &keybuf;
     memset(key, 0, sizeof(*key));
@@ -266,48 +279,74 @@ findEntry(MacroContext mc, const char * name, size_t namelen)
     /* XXX TODO: find 1st empty slot and return that */
     return ret;
 }
-/*@=mustmod@*/
+/*@=boundswrite@*/
 
 /* =============================================================== */
 
 /**
  * fgets(3) analogue that reads \ continuations. Last newline always trimmed.
+ * @param buf		input buffer
+ * @param size		inbut buffer size (bytes)
+ * @param fd		file handle
+ * @param escapes	permit escaped newlines?
+ * @return		buffer, or NULL on end-of-file
  */
-/*@dependent@*/ static char *
-rdcl(char * buf, size_t size, FD_t fd, int escapes)
+/*@-boundswrite@*/
+/*@null@*/
+static char *
+rdcl(/*@returned@*/ char * buf, size_t size, FD_t fd)
 	/*@globals fileSystem @*/
 	/*@modifies buf, fileSystem @*/
 {
-    char *q = buf;
+    char *q = buf - 1;		/* initialize just before buffer. */
     size_t nb = 0;
     size_t nread = 0;
     FILE * f = fdGetFILE(fd);
+    int pc = 0, bc = 0;
+    char *p = buf;
 
-    *q = '\0';
     if (f != NULL)
     do {
-	/* read next line */
-	if (fgets(q, size, f) == NULL)
+	*(++q) = '\0';			/* terminate and move forward. */
+	if (fgets(q, size, f) == NULL)	/* read next line. */
 	    break;
 	nb = strlen(q);
-	nread += nb;
+	nread += nb;			/* trim trailing \r and \n */
 	for (q += nb - 1; nb > 0 && iseol(*q); q--)
 	    nb--;
-	if (!(nb > 0 && *q == '\\')) {	/* continue? */
+	for (; p <= q; p++) {
+	    switch (*p) {
+		case '\\':
+		    switch (*(p+1)) {
+			case '\0': /*@switchbreak@*/ break;
+			default: p++; /*@switchbreak@*/ break;
+		    }
+		    /*@switchbreak@*/ break;
+		case '%':
+		    switch (*(p+1)) {
+			case '{': p++, bc++; /*@switchbreak@*/ break;
+			case '(': p++, pc++; /*@switchbreak@*/ break;
+			case '%': p++; /*@switchbreak@*/ break;
+		    }
+		    /*@switchbreak@*/ break;
+		case '{': if (bc > 0) bc++; /*@switchbreak@*/ break;
+		case '}': if (bc > 0) bc--; /*@switchbreak@*/ break;
+		case '(': if (pc > 0) pc++; /*@switchbreak@*/ break;
+		case ')': if (pc > 0) pc--; /*@switchbreak@*/ break;
+	    }
+	}
+	if (nb == 0 || (*q != '\\' && !bc && !pc) || *(q+1) == '\0') {
 	    *(++q) = '\0';		/* trim trailing \r, \n */
 	    break;
 	}
-	if (escapes) {			/* copy escape too */
-	    q++;
-	    nb++;
-	}
+	q++; p++; nb++;			/* copy newline too */
 	size -= nb;
 	if (*q == '\r')			/* XXX avoid \r madness */
 	    *q = '\n';
-	*(++q) = '\0';			/* next char in buf */
     } while (size > 0);
-    /*@-retalias@*/ return (nread > 0 ? buf : NULL); /*@=retalias@*/
+    return (nread > 0 ? buf : NULL);
 }
+/*@=boundswrite@*/
 
 /**
  * Return text between pl and matching pr characters.
@@ -316,6 +355,7 @@ rdcl(char * buf, size_t size, FD_t fd, int escapes)
  * @param pr		right char, i.e. ']', ')', '}', etc.
  * @return		address of last char before pr (or NULL)
  */
+/*@null@*/
 static const char *
 matchchar(const char * p, char pl, char pr)
 	/*@*/
@@ -425,33 +465,32 @@ printExpansion(MacroBuf mb, const char * t, const char * te)
 }
 
 #define	SKIPBLANK(_s, _c)	\
+	/*@-globs@*/	/* FIX: __ctype_b */ \
 	while (((_c) = *(_s)) && isblank(_c)) \
-		(_s)++;
+		(_s)++;		\
+	/*@=globs@*/
 
 #define	SKIPNONBLANK(_s, _c)	\
+	/*@-globs@*/	/* FIX: __ctype_b */ \
 	while (((_c) = *(_s)) && !(isblank(_c) || iseol(_c))) \
-		(_s)++;
+		(_s)++;		\
+	/*@=globs@*/
 
 #define	COPYNAME(_ne, _s, _c)	\
     {	SKIPBLANK(_s,_c);	\
+	/*@-boundswrite@*/	\
 	while(((_c) = *(_s)) && (xisalnum(_c) || (_c) == '_')) \
 		*(_ne)++ = *(_s)++; \
 	*(_ne) = '\0';		\
+	/*@=boundswrite@*/	\
     }
 
 #define	COPYOPTS(_oe, _s, _c)	\
-    {	while(((_c) = *(_s)) && (_c) != ')') \
+    {	/*@-boundswrite@*/	\
+	while(((_c) = *(_s)) && (_c) != ')') \
 		*(_oe)++ = *(_s)++; \
 	*(_oe) = '\0';		\
-    }
-
-#define	COPYBODY(_be, _s, _c)	\
-    {	while(((_c) = *(_s)) && !iseol(_c)) { \
-		if ((_c) == '\\') \
-			(_s)++;	\
-		*(_be)++ = *(_s)++; \
-	}			\
-	*(_be) = '\0';		\
+	/*@=boundswrite@*/	\
     }
 
 /**
@@ -463,8 +502,7 @@ printExpansion(MacroBuf mb, const char * t, const char * te)
  */
 static int
 expandT(MacroBuf mb, const char * f, size_t flen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem@*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem@*/
 	/*@modifies mb, rpmGlobalMacroContext, fileSystem @*/
 {
     char *sbuf;
@@ -492,8 +530,7 @@ expandT(MacroBuf mb, const char * f, size_t flen)
  */
 static int
 expandS(MacroBuf mb, char * tbuf, size_t tbuflen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem@*/
+	/*@globals rpmGlobalMacroContext, fileSystem@*/
 	/*@modifies mb, *tbuf, rpmGlobalMacroContext, fileSystem @*/
 {
     const char *t = mb->t;
@@ -516,10 +553,10 @@ expandS(MacroBuf mb, char * tbuf, size_t tbuflen)
  * @param ulen		no. bytes in u buffer
  * @return		result of expansion
  */
+/*@-boundswrite@*/
 static int
 expandU(MacroBuf mb, char * u, size_t ulen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem@*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem@*/
 	/*@modifies mb, *u, rpmGlobalMacroContext, fileSystem @*/
 {
     const char *s = mb->s;
@@ -531,9 +568,7 @@ expandU(MacroBuf mb, char * u, size_t ulen)
     tbuf = alloca(ulen + 1);
     memset(tbuf, 0, (ulen + 1));
 
-    /*@-temptrans -assignexpose@*/
     mb->s = u;
-    /*@=temptrans =assignexpose@*/
     mb->t = tbuf;
     mb->nb = ulen;
     rc = expandMacro(mb);
@@ -548,6 +583,7 @@ expandU(MacroBuf mb, char * u, size_t ulen)
 
     return rc;
 }
+/*@=boundswrite@*/
 
 /**
  * Expand output of shell command into target buffer.
@@ -556,12 +592,11 @@ expandU(MacroBuf mb, char * u, size_t ulen)
  * @param clen		no. bytes in shell command
  * @return		result of expansion
  */
+/*@-boundswrite@*/
 static int
 doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem @*/
-	/*@modifies mb, rpmGlobalMacroContext,
-		fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies mb, rpmGlobalMacroContext, fileSystem @*/
 {
     char pcmd[BUFSIZ];
     FILE *shf;
@@ -587,6 +622,7 @@ doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
     }
     return 0;
 }
+/*@=boundswrite@*/
 
 /**
  * Parse (and execute) new macro definition.
@@ -597,8 +633,8 @@ doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
  * @return		address to continue parsing
  */
 /*@dependent@*/ static const char *
-doDefine(MacroBuf mb, const char * se, int level, int expandbody)
-	/*@globals rpmGlobalMacroContext @*/
+doDefine(MacroBuf mb, /*@returned@*/ const char * se, int level, int expandbody)
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies mb, rpmGlobalMacroContext @*/
 {
     const char *s = se;
@@ -609,9 +645,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
     int oc = ')';
 
     /* Copy name */
-/*@-globs@*/
     COPYNAME(ne, s, c);
-/*@=globs@*/
 
     /* Copy opts (if present) */
     oe = ne + 1;
@@ -624,24 +658,55 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
 
     /* Copy body, skipping over escaped newlines */
     b = be = oe + 1;
-/*@-globs@*/
     SKIPBLANK(s, c);
-/*@=globs@*/
     if (c == '{') {	/* XXX permit silent {...} grouping */
 	if ((se = matchchar(s, c, '}')) == NULL) {
 	    rpmError(RPMERR_BADSPEC,
 		_("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
-	    /*@-retalias@*/ return se; /*@=retalias@*/
+	    return se;
 	}
 	s++;	/* XXX skip { */
+/*@-boundswrite@*/
 	strncpy(b, s, (se - s));
 	b[se - s] = '\0';
+/*@=boundswrite@*/
 	be += strlen(b);
 	se++;	/* XXX skip } */
 	s = se;	/* move scan forward */
     } else {	/* otherwise free-field */
-	COPYBODY(be, s, c);
+/*@-boundswrite@*/
+	int bc = 0, pc = 0;
+	while (*s && (bc || pc || !iseol(*s))) {
+	    switch (*s) {
+		case '\\':
+		    switch (*(s+1)) {
+			case '\0': /*@switchbreak@*/ break;
+			default: s++; /*@switchbreak@*/ break;
+		    }
+		    /*@switchbreak@*/ break;
+		case '%':
+		    switch (*(s+1)) {
+			case '{': *be++ = *s++; bc++; /*@switchbreak@*/ break;
+			case '(': *be++ = *s++; pc++; /*@switchbreak@*/ break;
+			case '%': *be++ = *s++; /*@switchbreak@*/ break;
+		    }
+		    /*@switchbreak@*/ break;
+		case '{': if (bc > 0) bc++; /*@switchbreak@*/ break;
+		case '}': if (bc > 0) bc--; /*@switchbreak@*/ break;
+		case '(': if (pc > 0) pc++; /*@switchbreak@*/ break;
+		case ')': if (pc > 0) pc--; /*@switchbreak@*/ break;
+	    }
+	    *be++ = *s++;
+	}
+	*be = '\0';
+
+	if (bc || pc) {
+	    rpmError(RPMERR_BADSPEC,
+		_("Macro %%%s has unterminated body\n"), n);
+	    se = s;	/* XXX W2DO? */
+	    return se;
+	}
 
 	/* Trim trailing blanks/newlines */
 /*@-globs@*/
@@ -649,6 +714,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
 	    {};
 /*@=globs@*/
 	*(++be) = '\0';	/* one too far */
+/*@=boundswrite@*/
     }
 
     /* Move scan over body */
@@ -660,30 +726,30 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
     if (!((c = *n) && (xisalpha(c) || c == '_') && (ne - n) > 2)) {
 	rpmError(RPMERR_BADSPEC,
 		_("Macro %%%s has illegal name (%%define)\n"), n);
-	/*@-retalias@*/ return se; /*@=retalias@*/
+	return se;
     }
 
     /* Options must be terminated with ')' */
     if (o && oc != ')') {
 	rpmError(RPMERR_BADSPEC, _("Macro %%%s has unterminated opts\n"), n);
-	/*@-retalias@*/ return se; /*@=retalias@*/
+	return se;
     }
 
     if ((be - b) < 1) {
 	rpmError(RPMERR_BADSPEC, _("Macro %%%s has empty body\n"), n);
-	/*@-retalias@*/ return se; /*@=retalias@*/
+	return se;
     }
 
 /*@-modfilesys@*/
     if (expandbody && expandU(mb, b, (&buf[sizeof(buf)] - b))) {
 	rpmError(RPMERR_BADSPEC, _("Macro %%%s failed to expand\n"), n);
-	/*@-retalias@*/ return se; /*@=retalias@*/
+	return se;
     }
 /*@=modfilesys@*/
 
     addMacro(mb->mc, n, o, b, (level - 1));
 
-    /*@-retalias@*/ return se; /*@=retalias@*/
+    return se;
 }
 
 /**
@@ -693,7 +759,7 @@ doDefine(MacroBuf mb, const char * se, int level, int expandbody)
  * @return		address to continue parsing
  */
 /*@dependent@*/ static const char *
-doUndefine(MacroContext mc, const char * se)
+doUndefine(MacroContext mc, /*@returned@*/ const char * se)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies mc, rpmGlobalMacroContext @*/
 {
@@ -701,9 +767,7 @@ doUndefine(MacroContext mc, const char * se)
     char buf[BUFSIZ], *n = buf, *ne = n;
     int c;
 
-/*@-globs@*/
     COPYNAME(ne, s, c);
-/*@=globs@*/
 
     /* Move scan over body */
     while (iseol(*s))
@@ -714,12 +778,12 @@ doUndefine(MacroContext mc, const char * se)
     if (!((c = *n) && (xisalpha(c) || c == '_') && (ne - n) > 2)) {
 	rpmError(RPMERR_BADSPEC,
 		_("Macro %%%s has illegal name (%%undefine)\n"), n);
-	/*@-retalias@*/ return se; /*@=retalias@*/
+	return se;
     }
 
     delMacro(mc, n);
 
-    /*@-retalias@*/ return se; /*@=retalias@*/
+    return se;
 }
 
 #ifdef	DYING
@@ -752,9 +816,7 @@ pushMacro(/*@out@*/ MacroEntry * mep,
 		/*@null@*/ const char * b, int level)
 	/*@modifies *mep @*/
 {
-    /*@-usedef@*/
     MacroEntry prev = (mep && *mep ? *mep : NULL);
-    /*@=usedef@*/
     MacroEntry me = (MacroEntry) xmalloc(sizeof(*me));
 
     /*@-assignexpose@*/
@@ -765,10 +827,14 @@ pushMacro(/*@out@*/ MacroEntry * mep,
     me->body = xstrdup(b ? b : "");
     me->used = 0;
     me->level = level;
+/*@-boundswrite@*/
+/*@-branchstate@*/
     if (mep)
 	*mep = me;
     else
 	me = _free(me);
+/*@=branchstate@*/
+/*@=boundswrite@*/
 }
 
 /**
@@ -781,16 +847,20 @@ popMacro(MacroEntry * mep)
 {
 	MacroEntry me = (*mep ? *mep : NULL);
 
+/*@-branchstate@*/
 	if (me) {
 		/* XXX cast to workaround const */
 		/*@-onlytrans@*/
+/*@-boundswrite@*/
 		if ((*mep = me->prev) == NULL)
 			me->name = _free(me->name);
+/*@=boundswrite@*/
 		me->opts = _free(me->opts);
 		me->body = _free(me->body);
 		me = _free(me);
 		/*@=onlytrans@*/
 	}
+/*@=branchstate@*/
 }
 
 /**
@@ -848,8 +918,10 @@ freeArgs(MacroBuf mb)
  * @param lastc		stop parsing at lastc
  * @return		address to continue parsing
  */
+/*@-bounds@*/
 /*@dependent@*/ static const char *
-grabArgs(MacroBuf mb, const MacroEntry me, const char * se, char lastc)
+grabArgs(MacroBuf mb, const MacroEntry me, /*@returned@*/ const char * se,
+		const char * lastc)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies mb, rpmGlobalMacroContext @*/
 {
@@ -870,7 +942,7 @@ grabArgs(MacroBuf mb, const MacroEntry me, const char * se, char lastc)
 
     /* Copy args into buf until lastc */
     *be++ = ' ';
-    while ((c = *se++) != '\0' && c != lastc) {
+    while ((c = *se++) != '\0' && (se-1) != lastc) {
 /*@-globs@*/
 	if (!isblank(c)) {
 	    *be++ = c;
@@ -936,24 +1008,27 @@ grabArgs(MacroBuf mb, const MacroEntry me, const char * se, char lastc)
 
 #ifdef __GLIBC__
     /*@-mods@*/
-    optind = 1;
+    optind = 0;		/* XXX but posix != glibc */
     /*@=mods@*/
+#else
+    optind = 1;
 #endif
 
     opts = me->opts;
 
     /* Define option macros. */
-    while((c = getopt(argc, (char **)argv, opts)) != -1) {
+/*@-nullstate@*/ /* FIX: argv[] can be NULL */
+    while((c = getopt(argc, (char **)argv, opts)) != -1)
+/*@=nullstate@*/
+    {
 	if (c == '?' || (o = strchr(opts, c)) == NULL) {
 	    rpmError(RPMERR_BADSPEC, _("Unknown option %c in %s(%s)\n"),
 			(char)c, me->name, opts);
-	    /*@-retalias@*/ return se; /*@=retalias@*/
+	    return se;
 	}
 	*be++ = '-';
 	*be++ = c;
-	/*@-usedef@*/
 	if (o[1] == ':') {
-	/*@=usedef@*/
 	    *be++ = ' ';
 	    be = stpcpy(be, optarg);
 	}
@@ -977,16 +1052,19 @@ grabArgs(MacroBuf mb, const MacroEntry me, const char * se, char lastc)
 	for (c = optind; c < argc; c++) {
 	    sprintf(aname, "%d", (c - optind + 1));
 	    addMacro(mb->mc, aname, NULL, argv[c], mb->depth);
-	    *be++ = ' ';
+	    if (be != b) *be++ = ' '; /* Add space between args */
+/*@-nullpass@*/ /* FIX: argv[] can be NULL */
 	    be = stpcpy(be, argv[c]);
+/*@=nullpass@*/
 	}
     }
 
     /* Add unexpanded args as macro. */
     addMacro(mb->mc, "*", NULL, b, mb->depth);
 
-    /*@-retalias@*/ return se; /*@=retalias@*/
+    return se;
 }
+/*@=bounds@*/
 
 /**
  * Perform macro message output
@@ -997,10 +1075,8 @@ grabArgs(MacroBuf mb, const MacroEntry me, const char * se, char lastc)
  */
 static void
 doOutput(MacroBuf mb, int waserror, const char * msg, size_t msglen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem @*/
-	/*@modifies mb, rpmGlobalMacroContext,
-		fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies mb, rpmGlobalMacroContext, fileSystem @*/
 {
     char buf[BUFSIZ];
 
@@ -1024,24 +1100,24 @@ doOutput(MacroBuf mb, int waserror, const char * msg, size_t msglen)
  */
 static void
 doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
-		const char * g, size_t glen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem, internalState @*/
-	/*@modifies mb, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
+		/*@null@*/ const char * g, size_t gn)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies mb, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     char buf[BUFSIZ], *b = NULL, *be;
     int c;
 
     buf[0] = '\0';
-    if (g) {
-	strncpy(buf, g, glen);
-	buf[glen] = '\0';
+    if (g != NULL) {
+	strncpy(buf, g, gn);
+	buf[gn] = '\0';
 	(void) expandU(mb, buf, sizeof(buf));
     }
     if (STREQ("basename", f, fn)) {
 	if ((b = strrchr(buf, '/')) == NULL)
 	    b = buf;
+	else
+	    b++;
 #if NOTYET
     /* XXX watchout for conflict with %dir */
     } else if (STREQ("dirname", f, fn)) {
@@ -1061,7 +1137,9 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    b = (rpmIsVerbose() ? buf : NULL);
     } else if (STREQ("url2path", f, fn) || STREQ("u2p", f, fn)) {
 	(void)urlPath(buf, (const char **)&b);
+/*@-branchstate@*/
 	if (*b == '\0') b = "/";
+/*@=branchstate@*/
     } else if (STREQ("uncompress", f, fn)) {
 	rpmCompressedMagic compressed = COMPRESSED_OTHER;
 /*@-globs@*/
@@ -1125,11 +1203,9 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 static int
 expandMacro(MacroBuf mb)
 	/*@globals rpmGlobalMacroContext,
-		print_macro_trace, print_expand_trace,
-		fileSystem @*/
+		print_macro_trace, print_expand_trace, h_errno, fileSystem @*/
 	/*@modifies mb, rpmGlobalMacroContext,
-		print_macro_trace, print_expand_trace,
-		fileSystem @*/
+		print_macro_trace, print_expand_trace, fileSystem @*/
 {
     MacroEntry *mep;
     MacroEntry me;
@@ -1141,7 +1217,7 @@ expandMacro(MacroBuf mb)
     int c;
     int rc = 0;
     int negate;
-    char grab;
+    const char * lastc;
     int chkexist;
 
     if (++mb->depth > max_macro_depth) {
@@ -1153,6 +1229,7 @@ expandMacro(MacroBuf mb)
 	return 1;
     }
 
+/*@-branchstate@*/
     while (rc == 0 && mb->nb > 0 && (c = *s) != '\0') {
 	s++;
 	/* Copy text until next macro */
@@ -1174,7 +1251,7 @@ expandMacro(MacroBuf mb)
 	if (mb->depth > 1)	/* XXX full expansion for outermost level */
 		t = mb->t;	/* save expansion pointer for printExpand */
 	negate = 0;
-	grab = '\0';
+	lastc = NULL;
 	chkexist = 0;
 	switch ((c = *s)) {
 	default:		/* %name substitution */
@@ -1209,7 +1286,8 @@ expandMacro(MacroBuf mb)
 		/* For "%name " macros ... */
 /*@-globs@*/
 		if ((c = *fe) && isblank(c))
-			grab = '\n';
+			if ((lastc = strchr(fe,'\n')) == NULL)
+                lastc = strchr(fe, '\0');
 /*@=globs@*/
 		/*@switchbreak@*/ break;
 	case '(':		/* %(...) shell escape */
@@ -1256,7 +1334,7 @@ expandMacro(MacroBuf mb)
 			ge = se - 1;
 			/*@innerbreak@*/ break;
 		case ' ':
-			grab = se[-1];
+			lastc = se-1;
 			/*@innerbreak@*/ break;
 		default:
 			/*@innerbreak@*/ break;
@@ -1302,7 +1380,7 @@ expandMacro(MacroBuf mb)
 		int waserror = 0;
 		if (STREQ("error", f, fn))
 			waserror = 1;
-		if (g < ge)
+		if (g != NULL && g < ge)
 			doOutput(mb, waserror, g, gn);
 		else
 			doOutput(mb, waserror, f, fn);
@@ -1329,6 +1407,34 @@ expandMacro(MacroBuf mb)
 		continue;
 	}
 
+#ifdef	WITH_LUA
+	if (STREQ("lua", f, fn)) {
+		rpmlua lua = NULL; /* Global state. */
+		const char *ls = s+sizeof("{lua:")-1;
+		const char *lse = se-sizeof("}")+1;
+		char *scriptbuf = (char *)xmalloc((lse-ls)+1);
+		const char *printbuf;
+		memcpy(scriptbuf, ls, lse-ls);
+		scriptbuf[lse-ls] = '\0';
+		rpmluaSetPrintBuffer(lua, 1);
+		if (rpmluaRunScript(lua, scriptbuf, NULL) == -1)
+		    rc = 1;
+		printbuf = rpmluaGetPrintBuffer(lua);
+		if (printbuf) {
+		    int len = strlen(printbuf);
+		    if (len > mb->nb)
+			len = mb->nb;
+		    memcpy(mb->t, printbuf, len);
+		    mb->t += len;
+		    mb->nb -= len;
+		}
+		rpmluaSetPrintBuffer(lua, 0);
+		free(scriptbuf);
+		s = se;
+		continue;
+	}
+#endif
+
 	/* XXX necessary but clunky */
 	if (STREQ("basename", f, fn) ||
 	    STREQ("suffix", f, fn) ||
@@ -1410,8 +1516,8 @@ expandMacro(MacroBuf mb)
 
 	/* Setup args for "%name " macros with opts */
 	if (me && me->opts != NULL) {
-		if (grab != '\0') {
-			se = grabArgs(mb, me, fe, grab);
+		if (lastc != NULL) {
+			se = grabArgs(mb, me, fe, lastc);
 		} else {
 			addMacro(mb->mc, "**", NULL, "", mb->depth);
 			addMacro(mb->mc, "*", NULL, "", mb->depth);
@@ -1422,9 +1528,7 @@ expandMacro(MacroBuf mb)
 
 	/* Recursively expand body of macro */
 	if (me->body && *me->body) {
-		/*@-onlytrans@*/
 		mb->s = me->body;
-		/*@=onlytrans@*/
 		rc = expandMacro(mb);
 		if (rc == 0)
 			me->used++;	/* Mark macro as used */
@@ -1436,6 +1540,7 @@ expandMacro(MacroBuf mb)
 
 	s = se;
     }
+/*@=branchstate@*/
 
     *mb->t = '\0';
     mb->s = s;
@@ -1445,6 +1550,268 @@ expandMacro(MacroBuf mb)
     return rc;
 }
 
+/* =============================================================== */
+/* XXX dupe'd to avoid change in linkage conventions. */
+
+#define POPT_ERROR_NOARG        -10     /*!< missing argument */
+#define POPT_ERROR_BADQUOTE     -15     /*!< error in paramter quoting */
+#define POPT_ERROR_MALLOC       -21     /*!< memory allocation failed */
+
+#define POPT_ARGV_ARRAY_GROW_DELTA 5
+
+/*@-boundswrite@*/
+static int XpoptDupArgv(int argc, const char **argv,
+		int * argcPtr, const char *** argvPtr)
+	/*@modifies *argcPtr, *argvPtr @*/
+{
+    size_t nb = (argc + 1) * sizeof(*argv);
+    const char ** argv2;
+    char * dst;
+    int i;
+
+    if (argc <= 0 || argv == NULL)	/* XXX can't happen */
+	return POPT_ERROR_NOARG;
+    for (i = 0; i < argc; i++) {
+	if (argv[i] == NULL)
+	    return POPT_ERROR_NOARG;
+	nb += strlen(argv[i]) + 1;
+    }
+	
+    dst = malloc(nb);
+    if (dst == NULL)			/* XXX can't happen */
+	return POPT_ERROR_MALLOC;
+    argv2 = (void *) dst;
+    dst += (argc + 1) * sizeof(*argv);
+
+    /*@-branchstate@*/
+    for (i = 0; i < argc; i++) {
+	argv2[i] = dst;
+	dst += strlen(strcpy(dst, argv[i])) + 1;
+    }
+    /*@=branchstate@*/
+    argv2[argc] = NULL;
+
+    if (argvPtr) {
+	*argvPtr = argv2;
+    } else {
+	free(argv2);
+	argv2 = NULL;
+    }
+    if (argcPtr)
+	*argcPtr = argc;
+    return 0;
+}
+/*@=boundswrite@*/
+
+/*@-bounds@*/
+static int XpoptParseArgvString(const char * s, int * argcPtr, const char *** argvPtr)
+	/*@modifies *argcPtr, *argvPtr @*/
+{
+    const char * src;
+    char quote = '\0';
+    int argvAlloced = POPT_ARGV_ARRAY_GROW_DELTA;
+    const char ** argv = malloc(sizeof(*argv) * argvAlloced);
+    int argc = 0;
+    int buflen = strlen(s) + 1;
+    char * buf = memset(alloca(buflen), 0, buflen);
+    int rc = POPT_ERROR_MALLOC;
+
+    if (argv == NULL) return rc;
+    argv[argc] = buf;
+
+    for (src = s; *src != '\0'; src++) {
+	if (quote == *src) {
+	    quote = '\0';
+	} else if (quote != '\0') {
+	    if (*src == '\\') {
+		src++;
+		if (!*src) {
+		    rc = POPT_ERROR_BADQUOTE;
+		    goto exit;
+		}
+		if (*src != quote) *buf++ = '\\';
+	    }
+	    *buf++ = *src;
+	} else if (isspace(*src)) {
+	    if (*argv[argc] != '\0') {
+		buf++, argc++;
+		if (argc == argvAlloced) {
+		    argvAlloced += POPT_ARGV_ARRAY_GROW_DELTA;
+		    argv = realloc(argv, sizeof(*argv) * argvAlloced);
+		    if (argv == NULL) goto exit;
+		}
+		argv[argc] = buf;
+	    }
+	} else switch (*src) {
+	  case '"':
+	  case '\'':
+	    quote = *src;
+	    /*@switchbreak@*/ break;
+	  case '\\':
+	    src++;
+	    if (!*src) {
+		rc = POPT_ERROR_BADQUOTE;
+		goto exit;
+	    }
+	    /*@fallthrough@*/
+	  default:
+	    *buf++ = *src;
+	    /*@switchbreak@*/ break;
+	}
+    }
+
+    if (strlen(argv[argc])) {
+	argc++, buf++;
+    }
+
+    rc = XpoptDupArgv(argc, argv, argcPtr, argvPtr);
+
+exit:
+    if (argv) free(argv);
+    return rc;
+}
+/*@=bounds@*/
+/* =============================================================== */
+/*@unchecked@*/
+static int _debug = 0;
+
+int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
+{
+    int ac = 0;
+    const char ** av = NULL;
+    int argc = 0;
+    const char ** argv = NULL;
+    char * globRoot = NULL;
+#ifdef ENABLE_NLS	
+    const char * old_collate = NULL;
+    const char * old_ctype = NULL;
+    const char * t;
+#endif
+	size_t maxb, nb;
+    int i, j;
+    int rc;
+
+    rc = XpoptParseArgvString(patterns, &ac, &av);
+    if (rc)
+	return rc;
+#ifdef ENABLE_NLS
+/*@-branchstate@*/
+	t = setlocale(LC_COLLATE, NULL);
+	if (t)
+	    old_collate = xstrdup(t);
+	t = setlocale(LC_CTYPE, NULL);
+	if (t)
+	    old_ctype = xstrdup(t);
+/*@=branchstate@*/
+	(void) setlocale(LC_COLLATE, "C");
+	(void) setlocale(LC_CTYPE, "C");
+#endif
+	
+    if (av != NULL)
+    for (j = 0; j < ac; j++) {
+	const char * globURL;
+	const char * path;
+	int ut = urlPath(av[j], &path);
+	glob_t gl;
+
+	if (!Glob_pattern_p(av[j], 0) && strchr(path, '~') == NULL) {
+	    argv = xrealloc(argv, (argc+2) * sizeof(*argv));
+	    argv[argc] = xstrdup(av[j]);
+if (_debug)
+fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, argv[argc]);
+	    argc++;
+	    continue;
+	}
+	
+	gl.gl_pathc = 0;
+	gl.gl_pathv = NULL;
+	rc = Glob(av[j], GLOB_TILDE, Glob_error, &gl);
+	if (rc)
+	    goto exit;
+
+	/* XXX Prepend the URL leader for globs that have stripped it off */
+	maxb = 0;
+	for (i = 0; i < gl.gl_pathc; i++) {
+	    if ((nb = strlen(&(gl.gl_pathv[i][0]))) > maxb)
+		maxb = nb;
+	}
+	
+	nb = ((ut > URL_IS_DASH && ut != URL_IS_FTP) ? (path - av[j]) : 0);
+	maxb += nb;
+	maxb += 1;
+	globURL = globRoot = xmalloc(maxb);
+
+	switch (ut) {
+	case URL_IS_HTTP:
+	case URL_IS_PATH:
+	case URL_IS_DASH:
+	    strncpy(globRoot, av[j], nb);
+	    /*@switchbreak@*/ break;
+	case URL_IS_FTP:
+	case URL_IS_UNKNOWN:
+	    /*@switchbreak@*/ break;
+	}
+	globRoot += nb;
+	*globRoot = '\0';
+if (_debug)
+fprintf(stderr, "*** GLOB maxb %d diskURL %d %*s globURL %p %s\n", (int)maxb, (int)nb, (int)nb, av[j], globURL, globURL);
+	
+	argv = xrealloc(argv, (argc+gl.gl_pathc+1) * sizeof(*argv));
+
+	if (argv != NULL)
+	for (i = 0; i < gl.gl_pathc; i++) {
+	    const char * globFile = &(gl.gl_pathv[i][0]);
+	    if (globRoot > globURL && globRoot[-1] == '/')
+		while (*globFile == '/') globFile++;
+	    strcpy(globRoot, globFile);
+if (_debug)
+fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, globURL);
+	    argv[argc++] = xstrdup(globURL);
+	}
+	/*@-immediatetrans@*/
+	Globfree(&gl);
+	/*@=immediatetrans@*/
+	globURL = _free(globURL);
+    }
+
+    if (argv != NULL && argc > 0) {
+	argv[argc] = NULL;
+	if (argvPtr)
+	    *argvPtr = argv;
+	if (argcPtr)
+	    *argcPtr = argc;
+	rc = 0;
+    } else
+	rc = 1;
+
+
+exit:
+#ifdef ENABLE_NLS
+/*@-branchstate@*/
+    if (old_collate) {
+	(void) setlocale(LC_COLLATE, old_collate);
+	old_collate = _free(old_collate);
+    }
+    if (old_ctype) {
+	(void) setlocale(LC_CTYPE, old_ctype);
+	old_ctype = _free(old_ctype);
+    }
+/*@=branchstate@*/
+#endif
+    av = _free(av);
+/*@-branchstate@*/
+    if (rc || argvPtr == NULL) {
+/*@-dependenttrans -unqualifiedtrans@*/
+	if (argv != NULL)
+	for (i = 0; i < argc; i++)
+	    argv[i] = _free(argv[i]);
+	argv = _free(argv);
+/*@=dependenttrans =unqualifiedtrans@*/
+    }
+/*@=branchstate@*/
+    return rc;
+}
+
 /* =============================================================== */
 
 int
@@ -1461,19 +1828,15 @@ expandMacros(void * spec, MacroContext mc, char * sbuf, size_t slen)
     tbuf = alloca(slen + 1);
     memset(tbuf, 0, (slen + 1));
 
-    /*@-temptrans -assignexpose@*/
     mb->s = sbuf;
-    /*@=temptrans =assignexpose@*/
     mb->t = tbuf;
     mb->nb = slen;
     mb->depth = 0;
     mb->macro_trace = print_macro_trace;
     mb->expand_trace = print_expand_trace;
 
-    /*@-temptrans -assignexpose@*/
     mb->spec = spec;	/* (future) %file expansion info */
     mb->mc = mc;
-    /*@=temptrans =assignexpose@*/
 
     rc = expandMacro(mb);
 
@@ -1535,9 +1898,7 @@ rpmDefineMacro(MacroContext mc, const char * macro, int level)
 
     memset(mb, 0, sizeof(*mb));
     /* XXX just enough to get by */
-    /*@-temptrans -assignexpose@*/
     mb->mc = (mc ? mc : rpmGlobalMacroContext);
-    /*@=temptrans =assignexpose@*/
     (void) doDefine(mb, macro, level, 0);
     return 0;
 }
@@ -1564,10 +1925,43 @@ rpmLoadMacros(MacroContext mc, int level)
     }
 }
 
-void
-rpmInitMacros(/*@unused@*/ MacroContext mc, const char *macrofiles)
+int
+rpmLoadMacroFile(MacroContext mc, const char * fn)
 {
-    char *m, *mfile, *me;
+    FD_t fd = Fopen(fn, "r.fpio");
+    char buf[BUFSIZ];
+    int rc = -1;
+
+    if (fd == NULL || Ferror(fd)) {
+	if (fd) (void) Fclose(fd);
+	return rc;
+    }
+
+    /* XXX Assume new fangled macro expansion */
+    /*@-mods@*/
+    max_macro_depth = 16;
+    /*@=mods@*/
+
+    buf[0] = '\0';
+    while(rdcl(buf, sizeof(buf), fd) != NULL) {
+	char c, *n;
+
+	n = buf;
+	SKIPBLANK(n, c);
+
+	if (c != '%')
+		continue;
+	n++;	/* skip % */
+	rc = rpmDefineMacro(mc, n, RMIL_MACROFILES);
+    }
+    rc = Fclose(fd);
+    return rc;
+}
+
+void
+rpmInitMacros(MacroContext mc, const char * macrofiles)
+{
+    char *mfiles, *m, *me;
 
     if (macrofiles == NULL)
 	return;
@@ -1575,11 +1969,14 @@ rpmInitMacros(/*@unused@*/ MacroContext mc, const char *macrofiles)
     if (mc == NULL) mc = rpmGlobalMacroContext;
 #endif
 
-    for (mfile = m = xstrdup(macrofiles); mfile && *mfile != '\0'; mfile = me) {
-	FD_t fd;
-	char buf[BUFSIZ];
+    mfiles = xstrdup(macrofiles);
+    for (m = mfiles; m && *m != '\0'; m = me) {
+	const char ** av;
+	int ac;
+	int i;
 
-	for (me = mfile; (me = strchr(me, ':')) != NULL; me++) {
+	for (me = m; (me = strchr(me, ':')) != NULL; me++) {
+	    /* Skip over URI's. */
 	    if (!(me[1] == '/' && me[2] == '/'))
 		/*@innerbreak@*/ break;
 	}
@@ -1587,48 +1984,21 @@ rpmInitMacros(/*@unused@*/ MacroContext mc, const char *macrofiles)
 	if (me && *me == ':')
 	    *me++ = '\0';
 	else
-	    me = mfile + strlen(mfile);
+	    me = m + strlen(m);
 
-	/* Expand ~/ to $HOME */
-	buf[0] = '\0';
-	if (mfile[0] == '~' && mfile[1] == '/') {
-	    char *home;
-	    if ((home = getenv("HOME")) != NULL) {
-		mfile += 2;
-		strncpy(buf, home, sizeof(buf));
-		strncat(buf, "/", sizeof(buf) - strlen(buf));
-	    }
-	}
-	strncat(buf, mfile, sizeof(buf) - strlen(buf));
-	buf[sizeof(buf)-1] = '\0';
-
-	fd = Fopen(buf, "r.fpio");
-	if (fd == NULL || Ferror(fd)) {
-	    if (fd) (void) Fclose(fd);
+	/* Glob expand the macro file path element, expanding ~ to $HOME. */
+	ac = 0;
+	av = NULL;
+	i = rpmGlob(m, &ac, &av);
+        if (i != 0)
 	    continue;
-	}
 
-	/* XXX Assume new fangled macro expansion */
-	/*@-mods@*/
-	max_macro_depth = 16;
-	/*@=mods@*/
-
-	while(rdcl(buf, sizeof(buf), fd, 1) != NULL) {
-	    char c, *n;
-
-	    n = buf;
-/*@-globs@*/
-	    SKIPBLANK(n, c);
-/*@=globs@*/
-
-	    if (c != '%')
-		/*@innercontinue@*/ continue;
-	    n++;	/* skip % */
-	    (void) rpmDefineMacro(NULL, n, RMIL_MACROFILES);
-	}
-	(void) Fclose(fd);
+	/* Read macros from each file. */
+	for (i = 0; i < ac; i++)
+	    (void) rpmLoadMacroFile(mc, av[i]);
+	av = _free(av);
     }
-    m = _free(m);
+    mfiles = _free(mfiles);
 
     /* Reload cmdline macros */
     /*@-mods@*/
@@ -1819,7 +2189,7 @@ char *rpmCleanPath(char * path)
 		/*@switchbreak@*/ break;
 	    }
 	    /* Trim embedded ./ , trailing /. */
-	    if ((t[-1] == '/' && s[1] == '\0') || (t != path && s[1] == '/')) {
+	    if ((t[-1] == '/' && s[1] == '\0') || (t > path && t[-1] == '/' && s[1] == '/')) {
 		s++;
 		continue;
 	    }
@@ -1849,7 +2219,7 @@ char *rpmCleanPath(char * path)
     *t = '\0';
 
 /*fprintf(stderr, "\t%s\n", path); */
-    /*@-temptrans -retalias@*/ return path; /*@=temptrans =retalias@*/
+    return path;
 }
 
 /* Return concatenated and expanded canonical path. */
@@ -1932,12 +2302,14 @@ if (_debug) fprintf(stderr, "*** RGP ut %d file %s nurl %d\n", ut, file, nurl);
 #endif
     }
 
+/*@-branchstate@*/
     if (url && nurl > 0) {
 	char *t = strncpy(alloca(nurl+1), url, nurl);
 	t[nurl] = '\0';
 	url = t;
     } else
 	url = "";
+/*@=branchstate@*/
 
     result = rpmGetPath(url, root, "/", mdir, "/", file, NULL);
 
@@ -2012,7 +2384,7 @@ main(int argc, char *argv[])
     rpmDumpMacroTable(NULL, NULL);
 
     if ((fp = fopen(testfile, "r")) != NULL) {
-	while(rdcl(buf, sizeof(buf), fp, 1)) {
+	while(rdcl(buf, sizeof(buf), fp)) {
 	    x = expandMacros(NULL, NULL, buf, sizeof(buf));
 	    fprintf(stderr, "%d->%s\n", x, buf);
 	    memset(buf, 0, sizeof(buf));
@@ -2020,7 +2392,7 @@ main(int argc, char *argv[])
 	fclose(fp);
     }
 
-    while(rdcl(buf, sizeof(buf), stdin, 1)) {
+    while(rdcl(buf, sizeof(buf), stdin)) {
 	x = expandMacros(NULL, NULL, buf, sizeof(buf));
 	fprintf(stderr, "%d->%s\n <-\n", x, buf);
 	memset(buf, 0, sizeof(buf));
@@ -2031,4 +2403,4 @@ main(int argc, char *argv[])
 }
 #endif	/* EVAL_MACROS */
 #endif	/* DEBUG_MACROS */
-/*@=branchstate@*/
+/*@=boundsread@*/
