@@ -18,19 +18,21 @@
 
 /**
  */
-#ifdef	NOTYET
 typedef /*@abstract@*/ struct pgpDig_s * pgpDig;
-#endif
+
+/**
+ */
+typedef /*@abstract@*/ struct pgpDigParams_s * pgpDigParams;
 
 /** \ingroup rpmio
  * Hide libio API lossage.
  * The libio interface changed after glibc-2.1.3 to pass the seek offset
  * argument as a pointer rather than as an off_t. The snarl below defines
  * typedefs to isolate the lossage.
- * API unchanged.
  */
 /*@{*/
-#if !defined(__LCLINT__) && defined(__GLIBC__) && __GLIBC__ == 2 && __GLIBC_MINOR__ == 2
+#if !defined(__LCLINT__) && defined(__GLIBC__) && \
+	(__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 2))
 #define USE_COOKIE_SEEK_POINTER 1
 typedef _IO_off64_t 	_libio_off_t;
 typedef _libio_off_t *	_libio_pos_t;
@@ -60,26 +62,28 @@ extern "C" {
 /**
  */
 typedef ssize_t (*fdio_read_function_t) (void *cookie, char *buf, size_t nbytes)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies *cookie, errno, fileSystem @*/
+	/*@requires maxSet(buf) >= (nbytes - 1) @*/
+	/*@ensures maxRead(buf) == result @*/ ;
 
 /**
  */
 typedef ssize_t (*fdio_write_function_t) (void *cookie, const char *buf, size_t nbytes)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies *cookie, errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_seek_function_t) (void *cookie, _libio_pos_t pos, int whence)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies *cookie, errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_close_function_t) (void *cookie)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
+	/*@globals errno, fileSystem, systemState @*/
+	/*@modifies *cookie, errno, fileSystem, systemState @*/;
 
 
 /**
@@ -115,8 +119,8 @@ typedef int (*fdio_fileno_function_t) (void * cookie)
 /**
  */
 typedef FD_t (*fdio_open_function_t) (const char * path, int flags, mode_t mode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
 /**
  */
@@ -146,51 +150,51 @@ typedef int (*fdio_fflush_function_t) (FD_t fd)
 /**
  */
 typedef int (*fdio_mkdir_function_t) (const char * path, mode_t mode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_chdir_function_t) (const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_rmdir_function_t) (const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_rename_function_t) (const char * oldpath, const char * newpath)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_unlink_function_t) (const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 /*@-typeuse@*/
 
 /**
  */
 typedef int (*fdio_stat_function_t) (const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem @*/
-	/*@modifies *st, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies *st, errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_lstat_function_t) (const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem @*/
-	/*@modifies *st, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies *st, errno, fileSystem @*/;
 
 /**
  */
 typedef int (*fdio_access_function_t) (const char * path, int amode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 /*@=typeuse@*/
 /*@}*/
 
@@ -229,24 +233,29 @@ struct FDIO_s {
 /**
  * strerror(3) clone.
  */
-/*@-redecl@*/
 /*@observer@*/ const char * Fstrerror(/*@null@*/ FD_t fd)
 	/*@*/;
-/*@=redecl@*/
 
 /**
  * fread(3) clone.
  */
+/*@-incondefs@*/
 size_t Fread(/*@out@*/ void * buf, size_t size, size_t nmemb, FD_t fd)
 	/*@globals fileSystem @*/
-	/*@modifies fd, *buf, fileSystem @*/;
+	/*@modifies fd, *buf, fileSystem @*/
+	/*@requires maxSet(buf) >= (nmemb - 1) @*/
+	/*@ensures maxRead(buf) == result @*/;
+/*@=incondefs@*/
 
 /**
  * fwrite(3) clone.
  */
+/*@-incondefs@*/
 size_t Fwrite(const void * buf, size_t size, size_t nmemb, FD_t fd)
 	/*@globals fileSystem @*/
-	/*@modifies fd, fileSystem @*/;
+	/*@modifies fd, fileSystem @*/
+	/*@requires maxRead(buf) >= nmemb @*/;
+/*@=incondefs@*/
 
 /**
  * fseek(3) clone.
@@ -259,8 +268,8 @@ int Fseek(FD_t fd, _libio_off_t offset, int whence)
  * fclose(3) clone.
  */
 int Fclose( /*@killref@*/ FD_t fd)
-	/*@globals fileSystem @*/
-	/*@modifies fd, fileSystem @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fd, fileSystem, internalState @*/;
 
 /**
  */
@@ -273,8 +282,8 @@ int Fclose( /*@killref@*/ FD_t fd)
  */
 /*@null@*/ FD_t	Fopen(/*@null@*/ const char * path,
 			/*@null@*/ const char * fmode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
 
 
 /**
@@ -300,9 +309,10 @@ int Fileno(FD_t fd)
 /**
  * fcntl(2) clone.
  */
+/*@unused@*/
 int Fcntl(FD_t fd, int op, void *lip)
-	/*@globals fileSystem @*/
-	/*@modifies fd, *lip, fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies fd, *lip, errno, fileSystem @*/;
 
 /*@}*/
 
@@ -315,72 +325,87 @@ int Fcntl(FD_t fd, int op, void *lip)
  * mkdir(2) clone.
  */
 int Mkdir(const char * path, mode_t mode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * chdir(2) clone.
  */
 int Chdir(const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * rmdir(2) clone.
  */
 int Rmdir(const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * rename(2) clone.
  */
 int Rename(const char * oldpath, const char * newpath)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * link(2) clone.
  */
 int Link(const char * oldpath, const char * newpath)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * unlink(2) clone.
  */
 int Unlink(const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * readlink(2) clone.
  */
+/*@-incondefs@*/
 int Readlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
-	/*@globals fileSystem @*/
-	/*@modifies *buf, fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies *buf, errno, fileSystem, internalState @*/
+	/*@requires maxSet(buf) >= (bufsiz - 1) @*/
+	/*@ensures maxRead(buf) <= bufsiz @*/;
+/*@=incondefs@*/
 
 /**
  * stat(2) clone.
  */
 int Stat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem @*/
-	/*@modifies *st, fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, errno, fileSystem, internalState @*/;
 
 /**
  * lstat(2) clone.
  */
 int Lstat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals fileSystem @*/
-	/*@modifies *st, fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, errno, fileSystem, internalState @*/;
 
 /**
  * access(2) clone.
  */
 int Access(const char * path, int amode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
 
+/**
+ * glob_pattern_p(3) clone.
+ */
+int Glob_pattern_p (const char *pattern, int quote)
+	/*@*/;
+
+/**
+ * glob_error(3) clone.
+ */
+int Glob_error(const char * epath, int eerrno)
+	/*@*/;
 
 /**
  * glob(3) clone.
@@ -404,22 +429,23 @@ void Globfree( /*@only@*/ glob_t * pglob)
  */
 /*@null@*/
 DIR * Opendir(const char * path)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, h_errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * readdir(3) clone.
  */
-/*@null@*/ struct dirent * Readdir(DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies *dir, fileSystem @*/;
+/*@dependent@*/ /*@null@*/
+struct dirent * Readdir(DIR * dir)
+	/*@globals errno, fileSystem @*/
+	/*@modifies *dir, errno, fileSystem @*/;
 
 /**
  * closedir(3) clone.
  */
-int	Closedir(/*@only@*/ DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies *dir, fileSystem @*/;
+int Closedir(/*@only@*/ DIR * dir)
+	/*@globals errno, fileSystem @*/
+	/*@modifies *dir, errno, fileSystem @*/;
 
 /*@}*/
 
@@ -438,8 +464,8 @@ off_t	fdSize(FD_t fd)
 /**
  */
 /*@null@*/ FD_t fdDup(int fdno)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem@*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
 
 #ifdef UNUSED
 /*@null@*/ FILE *fdFdopen( /*@only@*/ void * cookie, const char * mode);
@@ -459,30 +485,34 @@ off_t	fdSize(FD_t fd)
 /**
  */
 /*@null@*/ FD_t fdOpen(const char *path, int flags, mode_t mode)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies errno, fileSystem, internalState @*/;
 #define	fdOpen(_path, _flags, _mode)	fdio->_open((_path), (_flags), (_mode))
 
 /**
  */
+/*@-incondefs@*/
 ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, *buf, fileSystem @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *cookie, *buf, errno, fileSystem, internalState @*/
+	/*@requires maxSet(buf) >= (count - 1) @*/
+	/*@ensures maxRead(buf) == result @*/ ;
 #define	fdRead(_fd, _buf, _count)	fdio->read((_fd), (_buf), (_count))
+/*@=incondefs@*/
 
 /**
  */
 ssize_t	fdWrite(void * cookie, const char * buf, size_t count)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *cookie, errno, fileSystem, internalState @*/;
 #define	fdWrite(_fd, _buf, _count)	fdio->write((_fd), (_buf), (_count))
 
 /**
  */
 int fdClose( /*@only@*/ void * cookie)
-	/*@globals fileSystem @*/
-	/*@modifies *cookie, fileSystem @*/;
-#define	fdCLose(_fd)		fdio->close(_fd)
+	/*@globals errno, fileSystem, systemState, internalState @*/
+	/*@modifies *cookie, errno, fileSystem, systemState, internalState @*/;
+#define	fdClose(_fd)		fdio->close(_fd)
 
 /**
  */
@@ -524,6 +554,18 @@ int fdReadable(FD_t fd, int secs)
 	/*@modifies fd, errno @*/;
 /*@=exportlocal@*/
 
+/**
+ * Insure that directories in path exist, creating as needed.
+ * @param path		diretory path
+ * @param mode		directory mode (if created)
+ * @param uid		directory uid (if created), or -1 to skip
+ * @param gid		directory uid (if created), or -1 to skip
+ * @return		0 on success, errno (or -1) on error
+ */
+int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
+
 /**
  * FTP and HTTP error codes.
  */
@@ -547,7 +589,8 @@ typedef enum ftperrCode_e {
 /**
  */
 /*@-redecl@*/
-/*@observer@*/ const char *const ftpStrerror(int errorNumber)	/*@*/;
+/*@observer@*/ const char *const ftpStrerror(int errorNumber)
+	/*@*/;
 /*@=redecl@*/
 
 /**
@@ -560,7 +603,9 @@ typedef enum ftperrCode_e {
  */
 /*@-redecl@*/
 /*@unused@*/
-/*@observer@*/ const char * urlStrerror(const char * url)	/*@*/;
+/*@observer@*/ const char * urlStrerror(const char * url)
+	/*@globals h_errno, internalState @*/
+	/*@modifies internalState @*/;
 /*@=redecl@*/
 
 /**
@@ -574,8 +619,8 @@ int ufdCopy(FD_t sfd, FD_t tfd)
 /**
  */
 int ufdGetFile( /*@killref@*/ FD_t sfd, FD_t tfd)
-	/*@globals fileSystem @*/
-	/*@modifies sfd, tfd, fileSystem @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies sfd, tfd, fileSystem, internalState @*/;
 
 /**
  */
