@@ -8,51 +8,125 @@
 
 #include <rpmio.h>
 #include <rpmurl.h>
+
+#if HAVE_BEECRYPT_API_H
+#include <api.h>
+#else
+#include <beecrypt.api.h>
+#endif
+
 #include <rpmpgp.h>
+#include <rpmsw.h>
+
+/* Drag in the beecrypt includes. */
+#include <beecrypt.h>
+#include <base64.h>
+#include <dsa.h>
+#include <endianness.h>
+#include <md5.h>
+#include <mp.h>
+#include <rsa.h>
+#include <rsapk.h>
+#include <sha1.h>
+
+/** \ingroup rpmio
+ * Values parsed from OpenPGP signature/pubkey packet(s).
+ */
+struct pgpDigParams_s {
+/*@only@*/ /*@null@*/
+    const char * userid;
+/*@only@*/ /*@null@*/
+    const byte * hash;
+    const char * params[4];
+    byte tag;
+
+    byte version;		/*!< version number. */
+    byte time[4];		/*!< time that the key was created. */
+    byte pubkey_algo;		/*!< public key algorithm. */
+
+    byte hash_algo;
+    byte sigtype;
+    byte hashlen;
+    byte signhash16[2];
+    byte signid[8];
+    byte saved;
+#define	PGPDIG_SAVED_TIME	(1 << 0)
+#define	PGPDIG_SAVED_ID		(1 << 1)
+
+};
+
+/** \ingroup rpmio
+ * Container for values parsed from an OpenPGP signature and public key.
+ */
+struct pgpDig_s {
+    struct pgpDigParams_s signature;
+    struct pgpDigParams_s pubkey;
+
+    size_t nbytes;		/*!< No. bytes of plain text. */
+
+/*@only@*/ /*@null@*/
+    DIGEST_CTX sha1ctx;		/*!< (dsa) sha1 hash context. */
+/*@only@*/ /*@null@*/
+    DIGEST_CTX hdrsha1ctx;	/*!< (dsa) header sha1 hash context. */
+/*@only@*/ /*@null@*/
+    void * sha1;		/*!< (dsa) V3 signature hash. */
+    size_t sha1len;		/*!< (dsa) V3 signature hash length. */
+
+/*@only@*/ /*@null@*/
+    DIGEST_CTX md5ctx;		/*!< (rsa) md5 hash context. */
+#ifdef	NOTYET
+/*@only@*/ /*@null@*/
+    DIGEST_CTX hdrmd5ctx;	/*!< (rsa) header md5 hash context. */
+#endif
+/*@only@*/ /*@null@*/
+    void * md5;			/*!< (rsa) V3 signature hash. */
+    size_t md5len;		/*!< (rsa) V3 signature hash length. */
+
+    /* DSA parameters. */
+    mpbarrett p;
+    mpbarrett q;
+    mpnumber g;
+    mpnumber y;
+    mpnumber hm;
+    mpnumber r;
+    mpnumber s;
+
+    /* RSA parameters. */
+    rsapk rsa_pk;
+    mpnumber m;
+    mpnumber c;
+    mpnumber rsahm;
+};
 
 /** \ingroup rpmio
  */
 typedef struct _FDSTACK_s {
+/*@exposed@*/
     FDIO_t		io;
-/*@dependent@*/ void *	fp;
+/*@dependent@*/
+    void *		fp;
     int			fdno;
 } FDSTACK_t;
 
-/** \ingroup rpmio
- * Cumulative statistics for an I/O operation.
- */
-typedef struct {
-    int			count;	/*!< Number of operations. */
-    off_t		bytes;	/*!< Number of bytes transferred. */
-    time_t		msecs;	/*!< Number of milli-seconds. */
-} OPSTAT_t;
-
 /** \ingroup rpmio
  * Identify per-desciptor I/O operation statistics.
  */
-enum FDSTAT_e {
+typedef enum fdOpX_e {
     FDSTAT_READ		= 0,	/*!< Read statistics index. */
     FDSTAT_WRITE	= 1,	/*!< Write statistics index. */
     FDSTAT_SEEK		= 2,	/*!< Seek statistics index. */
-    FDSTAT_CLOSE	= 3	/*!< Close statistics index */
-};
+    FDSTAT_CLOSE	= 3,	/*!< Close statistics index */
+    FDSTAT_DIGEST	= 4,	/*!< Digest statistics index. */
+    FDSTAT_MAX		= 5
+} fdOpX;
 
 /** \ingroup rpmio
  * Cumulative statistics for a descriptor.
  */
 typedef	/*@abstract@*/ struct {
-    struct timeval	create;	/*!< Structure creation time. */
-    struct timeval	begin;	/*!< Operation start time. */
-    OPSTAT_t		ops[4];	/*!< Cumulative statistics. */
+    struct rpmop_s	ops[FDSTAT_MAX];	/*!< Cumulative statistics. */
 } * FDSTAT_t;
 
-/** \ingroup rpmio
- * Bit(s) to control digest operation.
- */
-typedef enum rpmDigestFlags_e {
-    RPMDIGEST_NONE	= 0
-} rpmDigestFlags;
-
 /** \ingroup rpmio
  */
 typedef struct _FDDIGEST_s {
@@ -60,56 +134,12 @@ typedef struct _FDDIGEST_s {
     DIGEST_CTX		hashctx;
 } * FDDIGEST_t;
 
-/** \ingroup rpmio
- * Duplicate a digest context.
- * @param ctx		existing digest context
- * @return		duplicated digest context
- */
-/*@only@*/ /*@unused@*/
-DIGEST_CTX rpmDigestDup(DIGEST_CTX octx)
-	/*@*/;
-
-/** \ingroup rpmio
- * Initialize digest.
- * Set bit count to 0 and buffer to mysterious initialization constants.
- * @param flags		bit(s) to control digest operation
- * @return		digest context
- */
-/*@only@*/
-DIGEST_CTX rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
-	/*@*/;
-
-/** \ingroup rpmio
- * Update context with next plain text buffer.
- * @param ctx		digest context
- * @param data		next data buffer
- * @param len		no. bytes of data
- * @return		0 on success
- */
-int rpmDigestUpdate(DIGEST_CTX ctx, const void * data, size_t len)
-	/*@modifies ctx @*/;
-
-/** \ingroup rpmio
- * Return digest and destroy context.
- * Final wrapup - pad to 64-byte boundary with the bit pattern 
- * 1 0* (64-bit count of bits processed, MSB-first)
- *
- * @param ctx		digest context
- * @retval datap	address of returned digest
- * @retval lenp		address of digest length
- * @param asAscii	return digest as ascii string?
- * @return		0 on success
- */
-int rpmDigestFinal(/*@only@*/ DIGEST_CTX ctx,
-	/*@null@*/ /*@out@*/ void ** datap,
-	/*@null@*/ /*@out@*/ size_t * lenp, int asAscii)
-		/*@modifies *datap, *lenp @*/;
-
 /** \ingroup rpmio
  * The FD_t File Handle data structure.
  */
 struct _FD_s {
-/*@refs@*/ int	nrefs;
+/*@refs@*/
+    int		nrefs;
     int		flags;
 #define	RPMIO_DEBUG_IO		0x40000000
 #define	RPMIO_DEBUG_REFS	0x20000000
@@ -119,7 +149,8 @@ struct _FD_s {
     FDSTACK_t	fps[8];
     int		urlType;	/* ufdio: */
 
-/*@dependent@*/ void *	url;	/* ufdio: URL info */
+/*@dependent@*/
+    void *	url;		/* ufdio: URL info */
     int		rd_timeoutsecs;	/* ufdRead: per FD_t timer */
     ssize_t	bytesRemain;	/* ufdio: */
     ssize_t	contentLength;	/* ufdio: */
@@ -127,7 +158,8 @@ struct _FD_s {
     int		wr_chunked;	/* ufdio: */
 
     int		syserrno;	/* last system errno encountered */
-/*@observer@*/ const void *errcookie;	/* gzdio/bzdio/ufdio: */
+/*@observer@*/
+    const void *errcookie;	/* gzdio/bzdio/ufdio: */
 
     FDSTAT_t	stats;		/* I/O statistics */
 
@@ -149,11 +181,23 @@ struct _FD_s {
 extern int _rpmio_debug;
 /*@=redecl@*/
 
-#define DBG(_f, _m, _x) \
-    if ((_rpmio_debug | ((_f) ? ((FD_t)(_f))->flags : 0)) & (_m)) fprintf _x
+/*@-redecl@*/
+/*@unchecked@*/
+extern int _ftp_debug;
+/*@=redecl@*/
 
+#define DBG(_f, _m, _x) \
+    /*@-modfilesys@*/ \
+    if ((_rpmio_debug | ((_f) ? ((FD_t)(_f))->flags : 0)) & (_m)) fprintf _x \
+    /*@=modfilesys@*/
+
+#if defined(__LCLINT__XXX)
+#define DBGIO(_f, _x)
+#define DBGREFS(_f, _x)
+#else
 #define DBGIO(_f, _x)   DBG((_f), RPMIO_DEBUG_IO, _x)
 #define DBGREFS(_f, _x) DBG((_f), RPMIO_DEBUG_REFS, _x)
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -169,35 +213,37 @@ int fdFgets(FD_t fd, char * buf, size_t len)
  */
 /*@null@*/ FD_t ftpOpen(const char *url, /*@unused@*/ int flags,
                 /*@unused@*/ mode_t mode, /*@out@*/ urlinfo *uret)
-	/*@globals fileSystem @*/
-	/*@modifies *uret, fileSystem @*/;
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies *uret, fileSystem, internalState @*/;
 
 /** \ingroup rpmio
  */
 int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
-	/*@globals fileSystem @*/
-	/*@modifies data, fileSystem @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies data, fileSystem, internalState @*/;
 
 /** \ingroup rpmio
  */
 int ftpCmd(const char * cmd, const char * url, const char * arg2)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
 
 /** \ingroup rpmio
  */
 int ufdClose( /*@only@*/ void * cookie)
-	/*@globals fileSystem @*/
-	/*@modifies cookie, fileSystem @*/;
+	/*@globals fileSystem, internalState @*/
+	/*@modifies cookie, fileSystem, internalState @*/;
 
 /** \ingroup rpmio
  */
 /*@unused@*/ static inline
-/*@null@*/ const FDIO_t fdGetIo(FD_t fd)
+/*@null@*/ FDIO_t fdGetIo(FD_t fd)
 	/*@*/
 {
     FDSANE(fd);
+/*@-boundsread@*/
     return fd->fps[fd->nfps].io;
+/*@=boundsread@*/
 }
 
 /** \ingroup rpmio
@@ -208,9 +254,11 @@ void fdSetIo(FD_t fd, /*@kept@*/ /*@null@*/ FDIO_t io)
 	/*@modifies fd @*/
 {
     FDSANE(fd);
+/*@-boundswrite@*/
     /*@-assignexpose@*/
     fd->fps[fd->nfps].io = io;
     /*@=assignexpose@*/
+/*@=boundswrite@*/
 }
 /*@=nullstate@*/
 
@@ -221,9 +269,11 @@ void fdSetIo(FD_t fd, /*@kept@*/ /*@null@*/ FDIO_t io)
 	/*@*/
 {
     FDSANE(fd);
+/*@-boundsread@*/
     /*@+voidabstract@*/
     return ((FILE *)fd->fps[fd->nfps].fp);
     /*@=voidabstract@*/
+/*@=boundsread@*/
 }
 
 /** \ingroup rpmio
@@ -233,7 +283,9 @@ void fdSetIo(FD_t fd, /*@kept@*/ /*@null@*/ FDIO_t io)
 	/*@*/
 {
     FDSANE(fd);
+/*@-boundsread@*/
     return fd->fps[fd->nfps].fp;
+/*@=boundsread@*/
 }
 
 /** \ingroup rpmio
@@ -244,9 +296,11 @@ void fdSetFp(FD_t fd, /*@kept@*/ /*@null@*/ void * fp)
 	/*@modifies fd @*/
 {
     FDSANE(fd);
+/*@-boundswrite@*/
     /*@-assignexpose@*/
     fd->fps[fd->nfps].fp = fp;
     /*@=assignexpose@*/
+/*@=boundswrite@*/
 }
 /*@=nullstate@*/
 
@@ -257,7 +311,9 @@ int fdGetFdno(FD_t fd)
 	/*@*/
 {
     FDSANE(fd);
+/*@-boundsread@*/
     return fd->fps[fd->nfps].fdno;
+/*@=boundsread@*/
 }
 
 /** \ingroup rpmio
@@ -267,7 +323,9 @@ void fdSetFdno(FD_t fd, int fdno)
 	/*@modifies fd @*/
 {
     FDSANE(fd);
+/*@-boundswrite@*/
     fd->fps[fd->nfps].fdno = fdno;
+/*@=boundswrite@*/
 }
 
 /** \ingroup rpmio
@@ -297,7 +355,8 @@ void fdPush(FD_t fd, FDIO_t io, void * fp, int fdno)
 
 /** \ingroup rpmio
  */
-/*@unused@*/ static inline void fdPop(FD_t fd)
+/*@unused@*/ static inline
+void fdPop(FD_t fd)
 	/*@modifies fd @*/
 {
     FDSANE(fd);
@@ -310,79 +369,74 @@ void fdPush(FD_t fd, FDIO_t io, void * fp, int fdno)
 
 /** \ingroup rpmio
  */
-/*@unused@*/ static inline void fdstat_enter(/*@null@*/ FD_t fd, int opx)
-	/*@modifies fd @*/
+/*@unused@*/ static inline /*@null@*/
+rpmop fdstat_op(/*@null@*/ FD_t fd, fdOpX opx)
+	/*@*/
 {
-    if (fd == NULL || fd->stats == NULL) return;
-    fd->stats->ops[opx].count++;
-    (void) gettimeofday(&fd->stats->begin, NULL);
+    rpmop op = NULL;
+
+/*@-boundsread@*/
+    if (fd != NULL && fd->stats != NULL && opx >= 0 && opx < FDSTAT_MAX)
+        op = fd->stats->ops + opx;
+/*@=boundsread@*/
+    return op;
 }
 
 /** \ingroup rpmio
  */
 /*@unused@*/ static inline
-time_t tvsub(/*@null@*/ const struct timeval * etv,
-		/*@null@*/ const struct timeval * btv)
-	/*@*/
+void fdstat_enter(/*@null@*/ FD_t fd, int opx)
+	/*@globals internalState @*/
+	/*@modifies internalState @*/
 {
-    time_t secs, usecs;
-    if (etv == NULL  || btv == NULL) return 0;
-    secs = etv->tv_sec - btv->tv_sec;
-    for (usecs = etv->tv_usec - btv->tv_usec; usecs < 0; usecs += 1000000)
-	secs++;
-    return ((secs * 1000) + (usecs/1000));
+    if (fd == NULL) return;
+    if (fd->stats != NULL)
+	(void) rpmswEnter(fdstat_op(fd, opx), 0);
 }
 
 /** \ingroup rpmio
  */
 /*@unused@*/ static inline
 void fdstat_exit(/*@null@*/ FD_t fd, int opx, ssize_t rc)
-	/*@modifies fd @*/
+	/*@globals internalState @*/
+	/*@modifies fd, internalState @*/
 {
-    struct timeval end;
     if (fd == NULL) return;
-    if (rc == -1) fd->syserrno = errno;
-    if (fd->stats == NULL) return;
-    (void) gettimeofday(&end, NULL);
-    if (rc >= 0) {
-	switch(opx) {
-	case FDSTAT_SEEK:
-	    fd->stats->ops[opx].bytes = rc;
-	    break;
-	default:
-	    fd->stats->ops[opx].bytes += rc;
-	    if (fd->bytesRemain > 0) fd->bytesRemain -= rc;
-	    break;
-	}
-    }
-    fd->stats->ops[opx].msecs += tvsub(&end, &fd->stats->begin);
-    fd->stats->begin = end;	/* structure assignment */
+    if (rc == -1)
+	fd->syserrno = errno;
+    else if (rc > 0 && fd->bytesRemain > 0)
+	fd->bytesRemain -= rc;
+    if (fd->stats != NULL)
+	(void) rpmswExit(fdstat_op(fd, opx), rc);
 }
 
 /** \ingroup rpmio
  */
+/*@-boundsread@*/
 /*@unused@*/ static inline
 void fdstat_print(/*@null@*/ FD_t fd, const char * msg, FILE * fp)
 	/*@globals fileSystem @*/
 	/*@modifies *fp, fileSystem @*/
 {
+    static int usec_scale = (1000*1000);
     int opx;
+
     if (fd == NULL || fd->stats == NULL) return;
     for (opx = 0; opx < 4; opx++) {
-	OPSTAT_t *ops = &fd->stats->ops[opx];
-	if (ops->count <= 0) continue;
+	rpmop op = &fd->stats->ops[opx];
+	if (op->count <= 0) continue;
 	switch (opx) {
 	case FDSTAT_READ:
 	    if (msg) fprintf(fp, "%s:", msg);
-	    fprintf(fp, "%8d reads, %8ld total bytes in %d.%03d secs\n",
-		ops->count, (long)ops->bytes,
-		(int)(ops->msecs/1000), (int)(ops->msecs%1000));
+	    fprintf(fp, "%8d reads, %8ld total bytes in %d.%06d secs\n",
+		op->count, (long)op->bytes,
+		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
 	    /*@switchbreak@*/ break;
 	case FDSTAT_WRITE:
 	    if (msg) fprintf(fp, "%s:", msg);
-	    fprintf(fp, "%8d writes, %8ld total bytes in %d.%03d secs\n",
-		ops->count, (long)ops->bytes,
-		(int)(ops->msecs/1000), (int)(ops->msecs%1000));
+	    fprintf(fp, "%8d writes, %8ld total bytes in %d.%06d secs\n",
+		op->count, (long)op->bytes,
+		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
 	    /*@switchbreak@*/ break;
 	case FDSTAT_SEEK:
 	    /*@switchbreak@*/ break;
@@ -391,6 +445,7 @@ void fdstat_print(/*@null@*/ FD_t fd, const char * msg, FILE * fp)
 	}
     }
 }
+/*@=boundsread@*/
 
 /** \ingroup rpmio
  */
@@ -453,13 +508,16 @@ FD_t c2f(/*@null@*/ void * cookie)
  */
 /*@unused@*/ static inline
 void fdInitDigest(FD_t fd, pgpHashAlgo hashalgo, int flags)
-	/*@modifies fd @*/
+	/*@globals internalState @*/
+	/*@modifies fd, internalState @*/
 {
     FDDIGEST_t fddig = fd->digests + fd->ndigests;
     if (fddig != (fd->digests + FDDIGEST_MAX)) {
 	fd->ndigests++;
 	fddig->hashalgo = hashalgo;
+	fdstat_enter(fd, FDSTAT_DIGEST);
 	fddig->hashctx = rpmDigestInit(hashalgo, flags);
+	fdstat_exit(fd, FDSTAT_DIGEST, 0);
     }
 }
 
@@ -467,8 +525,9 @@ void fdInitDigest(FD_t fd, pgpHashAlgo hashalgo, int flags)
  * Update digest(s) attached to fd.
  */
 /*@unused@*/ static inline
-void fdUpdateDigests(FD_t fd, const byte * buf, ssize_t buflen)
-	/*@modifies fd @*/
+void fdUpdateDigests(FD_t fd, const unsigned char * buf, ssize_t buflen)
+	/*@globals internalState @*/
+	/*@modifies fd, internalState @*/
 {
     int i;
 
@@ -477,7 +536,9 @@ void fdUpdateDigests(FD_t fd, const byte * buf, ssize_t buflen)
 	FDDIGEST_t fddig = fd->digests + i;
 	if (fddig->hashctx == NULL)
 	    continue;
+	fdstat_enter(fd, FDSTAT_DIGEST);
 	(void) rpmDigestUpdate(fddig->hashctx, buf, buflen);
+	fdstat_exit(fd, FDSTAT_DIGEST, buflen);
     }
 }
 
@@ -488,7 +549,8 @@ void fdFiniDigest(FD_t fd, pgpHashAlgo hashalgo,
 		/*@null@*/ /*@out@*/ void ** datap,
 		/*@null@*/ /*@out@*/ size_t * lenp,
 		int asAscii)
-	/*@modifies fd, *datap, *lenp @*/
+	/*@globals internalState @*/
+	/*@modifies fd, *datap, *lenp, internalState @*/
 {
     int imax = -1;
     int i;
@@ -500,18 +562,22 @@ void fdFiniDigest(FD_t fd, pgpHashAlgo hashalgo,
 	if (i > imax) imax = i;
 	if (fddig->hashalgo != hashalgo)
 	    continue;
+	fdstat_enter(fd, FDSTAT_DIGEST);
 	(void) rpmDigestFinal(fddig->hashctx, datap, lenp, asAscii);
+	fdstat_exit(fd, FDSTAT_DIGEST, 0);
 	fddig->hashctx = NULL;
 	break;
     }
+/*@-boundswrite@*/
     if (i < 0) {
 	if (datap) *datap = NULL;
 	if (lenp) *lenp = 0;
-	return;
-    } else if (i == imax)
-	fd->ndigests = imax - 1;
-    else
-	fd->ndigests = imax;
+    }
+/*@=boundswrite@*/
+
+    fd->ndigests = imax;
+    if (i < imax)
+	fd->ndigests++;		/* convert index to count */
 }
 
 /*@-shadow@*/
@@ -524,16 +590,23 @@ int fdFileno(/*@null@*/ void * cookie)
     FD_t fd;
     if (cookie == NULL) return -2;
     fd = c2f(cookie);
+/*@-boundsread@*/
     return fd->fps[0].fdno;
+/*@=boundsread@*/
 }
 /*@=shadow@*/
 
 /**
+ * Read an entire file into a buffer.
+ * @param fn		file name to read
+ * @retval *bp		(malloc'd) buffer address
+ * @retval *blenp	(malloc'd) buffer length
+ * @return		0 on success
  */
 int rpmioSlurp(const char * fn,
-                /*@out@*/ const byte ** bp, /*@out@*/ ssize_t * blenp)
-        /*@globals fileSystem @*/
-        /*@modifies *bp, *blenp, fileSystem @*/;
+                /*@out@*/ const unsigned char ** bp, /*@out@*/ ssize_t * blenp)
+        /*@globals h_errno, fileSystem, internalState @*/
+        /*@modifies *bp, *blenp, fileSystem, internalState @*/;
 
 #ifdef __cplusplus
 }
