@@ -324,7 +324,7 @@ const char * pgpMpiHex(const byte *p)
  * @return		0 on success
  */
 static int pgpHexSet(const char * pre, int lbits,
-		/*@out@*/ mp32number * mpn, const byte * p, const byte * pend)
+		/*@out@*/ mpnumber * mpn, const byte * p, const byte * pend)
 	/*@globals fileSystem @*/
 	/*@modifies *mpn, fileSystem @*/
 {
@@ -348,10 +348,10 @@ fprintf(stderr, "*** mbits %u nbits %u nbytes %u t %p[%d] ix %u\n", mbits, nbits
     strcpy(t+ix, pgpMpiHex(p));
 if (_debug)
 fprintf(stderr, "*** %s %s\n", pre, t);
-    mp32nsethex(mpn, t);
+    mpnsethex(mpn, t);
     t = _free(t);
 if (_debug && _print)
-fprintf(stderr, "\t %s ", pre), mp32println(stderr, mpn->size, mpn->data);
+fprintf(stderr, "\t %s ", pre), mpfprintln(stderr, mpn->size, mpn->data);
     return 0;
 }
 /*@=boundswrite@*/
@@ -476,25 +476,22 @@ static int pgpPrtSigParams(/*@unused@*/ pgpTag tag, byte pubkey_algo, byte sigty
     for (i = 0; p < pend; i++, p += pgpMpiLen(p)) {
 	if (pubkey_algo == PGPPUBKEYALGO_RSA) {
 	    if (i >= 1) break;
-	    /*@-mods@*/
 	    if (_dig &&
 	(sigtype == PGPSIGTYPE_BINARY || sigtype == PGPSIGTYPE_TEXT))
 	    {
 		switch (i) {
 		case 0:		/* m**d */
-		    mp32nsethex(&_dig->c, pgpMpiHex(p));
+		    mpnsethex(&_dig->c, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t  m**d = "),  mp32println(stderr, _dig->c.size, _dig->c.data);
+fprintf(stderr, "\t  m**d = "),  mpfprintln(stderr, _dig->c.size, _dig->c.data);
 		    /*@switchbreak@*/ break;
 		default:
 		    /*@switchbreak@*/ break;
 		}
 	    }
-	    /*@=mods@*/
 	    pgpPrtStr("", pgpSigRSA[i]);
 	} else if (pubkey_algo == PGPPUBKEYALGO_DSA) {
 	    if (i >= 2) break;
-	    /*@-mods@*/
 	    if (_dig &&
 	(sigtype == PGPSIGTYPE_BINARY || sigtype == PGPSIGTYPE_TEXT))
 	    {
@@ -513,7 +510,6 @@ fprintf(stderr, "\t  m**d = "),  mp32println(stderr, _dig->c.size, _dig->c.data)
 		}
 		if (xx) return xx;
 	    }
-	    /*@=mods@*/
 	    pgpPrtStr("", pgpSigDSA[i]);
 	} else {
 	    if (_print)
@@ -527,6 +523,8 @@ fprintf(stderr, "\t  m**d = "),  mp32println(stderr, _dig->c.size, _dig->c.data)
 }
 
 int pgpPrtSig(pgpTag tag, const byte *h, unsigned int hlen)
+	/*@globals _digp @*/
+	/*@modifies *_digp @*/
 {
     byte version = h[0];
     byte * p;
@@ -555,7 +553,6 @@ int pgpPrtSig(pgpTag tag, const byte *h, unsigned int hlen)
 	pgpPrtHex(" signhash16", v->signhash16, sizeof(v->signhash16));
 	pgpPrtNL();
 
-/*@-mods@*/
 	if (_digp && _digp->pubkey_algo == 0) {
 	    _digp->version = v->version;
 	    _digp->hashlen = v->hashlen;
@@ -567,7 +564,6 @@ int pgpPrtSig(pgpTag tag, const byte *h, unsigned int hlen)
 	    _digp->hash_algo = v->hash_algo;
 	    memcpy(_digp->signhash16, v->signhash16, sizeof(_digp->signhash16));
 	}
-/*@=mods@*/
 
 	p = ((byte *)v) + sizeof(*v);
 	rc = pgpPrtSigParams(tag, v->pubkey_algo, v->sigtype, p, h, hlen);
@@ -590,12 +586,10 @@ int pgpPrtSig(pgpTag tag, const byte *h, unsigned int hlen)
 
 if (_debug && _print)
 fprintf(stderr, "   hash[%u] -- %s\n", plen, pgpHexStr(p, plen));
-/*@-mods@*/
 	if (_digp && _digp->pubkey_algo == 0) {
 	    _digp->hashlen = sizeof(*v) + plen;
 	    _digp->hash = memcpy(xmalloc(_digp->hashlen), v, _digp->hashlen);
 	}
-/*@=mods@*/
 	(void) pgpPrtSubType(p, plen, v->sigtype);
 	p += plen;
 
@@ -614,7 +608,6 @@ fprintf(stderr, " unhash[%u] -- %s\n", plen, pgpHexStr(p, plen));
 	pgpPrtHex(" signhash16", p, 2);
 	pgpPrtNL();
 
-/*@-mods@*/
 	if (_digp && _digp->pubkey_algo == 0) {
 	    _digp->version = v->version;
 	    _digp->sigtype = v->sigtype;
@@ -622,7 +615,6 @@ fprintf(stderr, " unhash[%u] -- %s\n", plen, pgpHexStr(p, plen));
 	    _digp->hash_algo = v->hash_algo;
 	    memcpy(_digp->signhash16, p, sizeof(_digp->signhash16));
 	}
-/*@=mods@*/
 
 	p += 2;
 	if (p > (h + hlen))
@@ -686,24 +678,23 @@ static const char * pgpSecretELGAMAL[] = {
 
 static const byte * pgpPrtPubkeyParams(byte pubkey_algo,
 		/*@returned@*/ const byte *p, const byte *h, unsigned int hlen)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/
 {
     int i;
 
     for (i = 0; p < &h[hlen]; i++, p += pgpMpiLen(p)) {
 	if (pubkey_algo == PGPPUBKEYALGO_RSA) {
 	    if (i >= 2) break;
-	    /*@-mods@*/
 	    if (_dig) {
 		switch (i) {
 		case 0:		/* n */
-		    mp32bsethex(&_dig->rsa_pk.n, pgpMpiHex(p));
+		    mpbsethex(&_dig->rsa_pk.n, pgpMpiHex(p));
 		    /* Get the keyid */
 		    if (_digp) {
-			uint32* np = _dig->rsa_pk.n.modl;
-			uint32  nsize = _dig->rsa_pk.n.size;
-			uint32 keyid[2];
+			uint32_t* np = _dig->rsa_pk.n.modl;
+			size_t nsize = _dig->rsa_pk.n.size;
+			uint32_t keyid[2];
 			#if WORDS_BIGENDIAN
 			keyid[0] = np[nsize-2];
 			keyid[1] = np[nsize-1];
@@ -714,49 +705,46 @@ static const byte * pgpPrtPubkeyParams(byte pubkey_algo,
 			memcpy(_digp->signid, keyid, sizeof(_digp->signid));
 		    }
 if (_debug && _print)
-fprintf(stderr, "\t     n = "),  mp32println(stderr, _dig->rsa_pk.n.size, _dig->rsa_pk.n.modl);
+fprintf(stderr, "\t     n = "),  mpfprintln(stderr, _dig->rsa_pk.n.size, _dig->rsa_pk.n.modl);
 		    /*@switchbreak@*/ break;
 		case 1:		/* e */
-		    mp32nsethex(&_dig->rsa_pk.e, pgpMpiHex(p));
+		    mpnsethex(&_dig->rsa_pk.e, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t     e = "),  mp32println(stderr, _dig->rsa_pk.e.size, _dig->rsa_pk.e.data);
+fprintf(stderr, "\t     e = "),  mpfprintln(stderr, _dig->rsa_pk.e.size, _dig->rsa_pk.e.data);
 		    /*@switchbreak@*/ break;
 		default:
 		    /*@switchbreak@*/ break;
 		}
 	    }
-	    /*@=mods@*/
 	    pgpPrtStr("", pgpPublicRSA[i]);
 	} else if (pubkey_algo == PGPPUBKEYALGO_DSA) {
 	    if (i >= 4) break;
-	    /*@-mods@*/
 	    if (_dig) {
 		switch (i) {
 		case 0:		/* p */
-		    mp32bsethex(&_dig->p, pgpMpiHex(p));
+		    mpbsethex(&_dig->p, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t     p = "),  mp32println(stderr, _dig->p.size, _dig->p.modl);
+fprintf(stderr, "\t     p = "),  mpfprintln(stderr, _dig->p.size, _dig->p.modl);
 		    /*@switchbreak@*/ break;
 		case 1:		/* q */
-		    mp32bsethex(&_dig->q, pgpMpiHex(p));
+		    mpbsethex(&_dig->q, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t     q = "),  mp32println(stderr, _dig->q.size, _dig->q.modl);
+fprintf(stderr, "\t     q = "),  mpfprintln(stderr, _dig->q.size, _dig->q.modl);
 		    /*@switchbreak@*/ break;
 		case 2:		/* g */
-		    mp32nsethex(&_dig->g, pgpMpiHex(p));
+		    mpnsethex(&_dig->g, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t     g = "),  mp32println(stderr, _dig->g.size, _dig->g.data);
+fprintf(stderr, "\t     g = "),  mpfprintln(stderr, _dig->g.size, _dig->g.data);
 		    /*@switchbreak@*/ break;
 		case 3:		/* y */
-		    mp32nsethex(&_dig->y, pgpMpiHex(p));
+		    mpnsethex(&_dig->y, pgpMpiHex(p));
 if (_debug && _print)
-fprintf(stderr, "\t     y = "),  mp32println(stderr, _dig->y.size, _dig->y.data);
+fprintf(stderr, "\t     y = "),  mpfprintln(stderr, _dig->y.size, _dig->y.data);
 		    /*@switchbreak@*/ break;
 		default:
 		    /*@switchbreak@*/ break;
 		}
 	    }
-	    /*@=mods@*/
 	    pgpPrtStr("", pgpPublicDSA[i]);
 	} else if (pubkey_algo == PGPPUBKEYALGO_ELGAMAL_ENCRYPT) {
 	    if (i >= 3) break;
@@ -765,7 +753,6 @@ fprintf(stderr, "\t     y = "),  mp32println(stderr, _dig->y.size, _dig->y.data)
 	    if (_print)
 		fprintf(stderr, "%7d", i);
 	}
-    /*@=mods@*/
 	pgpPrtStr("", pgpMpiStr(p));
 	pgpPrtNL();
     }
@@ -848,6 +835,8 @@ static const byte * pgpPrtSeckeyParams(/*@unused@*/ byte pubkey_algo,
 }
 
 int pgpPrtKey(pgpTag tag, const byte *h, unsigned int hlen)
+	/*@globals _digp @*/
+	/*@modifies *_digp @*/
 {
     byte version = *h;
     const byte * p;
@@ -868,13 +857,11 @@ int pgpPrtKey(pgpTag tag, const byte *h, unsigned int hlen)
 	    fprintf(stderr, " valid %u days", plen);
 	pgpPrtNL();
 
-/*@-mods@*/
 	if (_digp && _digp->tag == tag) {
 	    _digp->version = v->version;
 	    memcpy(_digp->time, v->time, sizeof(_digp->time));
 	    _digp->pubkey_algo = v->pubkey_algo;
 	}
-/*@=mods@*/
 
 	p = ((byte *)v) + sizeof(*v);
 	p = pgpPrtPubkeyParams(v->pubkey_algo, p, h, hlen);
@@ -889,13 +876,11 @@ int pgpPrtKey(pgpTag tag, const byte *h, unsigned int hlen)
 	    fprintf(stderr, " %-24.24s(0x%08x)", ctime(&t), (unsigned)t);
 	pgpPrtNL();
 
-/*@-mods@*/
 	if (_digp && _digp->tag == tag) {
 	    _digp->version = v->version;
 	    memcpy(_digp->time, v->time, sizeof(_digp->time));
 	    _digp->pubkey_algo = v->pubkey_algo;
 	}
-/*@=mods@*/
 
 	p = ((byte *)v) + sizeof(*v);
 	p = pgpPrtPubkeyParams(v->pubkey_algo, p, h, hlen);
@@ -912,18 +897,18 @@ int pgpPrtKey(pgpTag tag, const byte *h, unsigned int hlen)
 
 /*@-boundswrite@*/
 int pgpPrtUserID(pgpTag tag, const byte *h, unsigned int hlen)
+	/*@globals _digp @*/
+	/*@modifies *_digp @*/
 {
     pgpPrtVal("", pgpTagTbl, tag);
     if (_print)
 	fprintf(stderr, " \"%.*s\"", (int)hlen, (const char *)h);
     pgpPrtNL();
-/*@-mods@*/
     if (_digp) {
 	char * t;
 	_digp->userid = t = memcpy(xmalloc(hlen+1), h, hlen);
 	t[hlen] = '\0';
     }
-/*@=mods@*/
     return 0;
 }
 /*@=boundswrite@*/
@@ -1053,18 +1038,18 @@ void pgpCleanDig(pgpDig dig)
 
 	dig->md5 = _free(dig->md5);
 	dig->sha1 = _free(dig->sha1);
-	mp32nfree(&dig->hm);
-	mp32nfree(&dig->r);
-	mp32nfree(&dig->s);
+	mpnfree(&dig->hm);
+	mpnfree(&dig->r);
+	mpnfree(&dig->s);
 
 	(void) rsapkFree(&dig->rsa_pk);
-	mp32nfree(&dig->m);
-	mp32nfree(&dig->c);
-	mp32nfree(&dig->rsahm);
+	mpnfree(&dig->m);
+	mpnfree(&dig->c);
+	mpnfree(&dig->rsahm);
     }
-    /*@-nullstate@*/
+/*@-nullstate@*/
     return;
-    /*@=nullstate@*/
+/*@=nullstate@*/
 }
 /*@=boundswrite@*/
 
@@ -1088,13 +1073,13 @@ pgpDig pgpFreeDig(/*@only@*/ /*@null@*/ pgpDig dig)
 	/*@=branchstate@*/
 	dig->sha1ctx = NULL;
 
-	mp32bfree(&dig->p);
-	mp32bfree(&dig->q);
-	mp32nfree(&dig->g);
-	mp32nfree(&dig->y);
-	mp32nfree(&dig->hm);
-	mp32nfree(&dig->r);
-	mp32nfree(&dig->s);
+	mpbfree(&dig->p);
+	mpbfree(&dig->q);
+	mpnfree(&dig->g);
+	mpnfree(&dig->y);
+	mpnfree(&dig->hm);
+	mpnfree(&dig->r);
+	mpnfree(&dig->s);
 
 #ifdef	NOTYET
 	/*@-branchstate@*/
@@ -1110,11 +1095,11 @@ pgpDig pgpFreeDig(/*@only@*/ /*@null@*/ pgpDig dig)
 	/*@=branchstate@*/
 	dig->md5ctx = NULL;
 
-	mp32bfree(&dig->rsa_pk.n);
-	mp32nfree(&dig->rsa_pk.e);
-	mp32nfree(&dig->m);
-	mp32nfree(&dig->c);
-	mp32nfree(&dig->hm);
+	mpbfree(&dig->rsa_pk.n);
+	mpnfree(&dig->rsa_pk.e);
+	mpnfree(&dig->m);
+	mpnfree(&dig->c);
+	mpnfree(&dig->hm);
 
 	dig = _free(dig);
     }
@@ -1122,13 +1107,14 @@ pgpDig pgpFreeDig(/*@only@*/ /*@null@*/ pgpDig dig)
 }
 
 int pgpPrtPkts(const byte * pkts, unsigned int pktlen, pgpDig dig, int printing)
+	/*@globals _dig, _digp, _print @*/
+	/*@modifies _dig, _digp, *_digp, _print @*/
 {
     unsigned int val = *pkts;
     const byte *p;
     unsigned int pleft;
     int len;
 
-    /*@-mods@*/
     _print = printing;
     _dig = dig;
     if (dig != NULL && (val & 0x80)) {
@@ -1137,7 +1123,6 @@ int pgpPrtPkts(const byte * pkts, unsigned int pktlen, pgpDig dig, int printing)
 	_digp->tag = tag;
     } else
 	_digp = NULL;
-    /*@=mods@*/
 
     for (p = pkts, pleft = pktlen; p < (pkts + pktlen); p += len, pleft -= len) {
 	len = pgpPrtPkt(p, pleft);
@@ -1160,7 +1145,7 @@ pgpArmor pgpReadPkts(const char * fn, const byte ** pkt, size_t * pktlen)
     byte * crcdec;
     size_t declen;
     size_t crclen;
-    uint32 crcpkt, crc;
+    uint32_t crcpkt, crc;
     const char * armortype = NULL;
     char * t, * te;
     int pstate = 0;
@@ -1303,7 +1288,7 @@ char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns)
     /*@-globs@*/
     t = stpcpy( stpcpy(t, "-----\nVersion: rpm-"), VERSION);
     /*@=globs@*/
-    t = stpcpy(t, " (beecrypt-2.2.0)\n\n");
+    t = stpcpy(t, " (beecrypt-3.0.0)\n\n");
 
     if ((enc = b64encode(s, ns)) != NULL) {
 	t = stpcpy(t, enc);
