@@ -12,13 +12,13 @@
 #include "ugid.h"
 #include "debug.h"
 
-/*@access DIR@*/
-/*@access FD_t@*/
-/*@access urlinfo@*/
+/*@access DIR @*/
+/*@access FD_t @*/
+/*@access urlinfo @*/
 
 /* =============================================================== */
 static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     int rc;
@@ -34,21 +34,21 @@ static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
 }
 
 static int ftpChdir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("CWD", path, NULL);
 }
 
 static int ftpRmdir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("RMD", path, NULL);
 }
 
 static int ftpRename(const char * oldpath, const char * newpath)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     int rc;
@@ -58,7 +58,7 @@ static int ftpRename(const char * oldpath, const char * newpath)
 }
 
 static int ftpUnlink(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("DELE", path, NULL);
@@ -683,7 +683,7 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 
     /* This is device */
     if (S_ISCHR (st->st_mode) || S_ISBLK (st->st_mode)){
-	int maj, min;
+	unsigned maj, min;
 	
 	if (!is_num (idx2) || sscanf(columns [idx2], " %d,", &maj) != 1)
 	    goto error;
@@ -692,7 +692,7 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 	    goto error;
 	
 #ifdef HAVE_ST_RDEV
-	st->st_rdev = ((maj & 0xff) << 8) | (min & 0xffff00ff);
+	st->st_rdev = ((maj & 0x000000ffU) << 8) | (min & 0x000000ffU);
 #endif
 	st->st_size = 0;
 	
@@ -821,12 +821,13 @@ static /*@only@*/ char * ftpBuf = NULL;
 #define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
 
 /*@-boundswrite@*/
-/*@-mods@*/
 static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 		/*@out@*/ /*@null@*/ struct stat * st,
 		/*@out@*/ /*@null@*/ char * rlbuf, size_t rlbufsiz)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, *rlbuf, fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf,
+		h_errno, fileSystem, internalState @*/
+	/*@modifies *st, *rlbuf, ftpBufAlloced, ftpBuf,
+		fileSystem, internalState @*/
 {
     FD_t fd;
     const char * path;
@@ -903,6 +904,7 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 	/* XXX FIXME: realloc ftpBuf if < ~128 chars remain */
 	if ((ftpBufAlloced - bufLength) < (1024+80)) {
 	    ftpBufAlloced <<= 2;
+	    assert(ftpBufAlloced < (8*1024*1024));
 	    ftpBuf = xrealloc(ftpBuf, ftpBufAlloced);
 	}
 	s = se = ftpBuf + bufLength;
@@ -1010,7 +1012,6 @@ exit:
     (void) ufdClose(fd);
     return rc;
 }
-/*@=mods@*/
 /*@=boundswrite@*/
 
 static const char * statstr(const struct stat * st,
@@ -1034,41 +1035,37 @@ static const char * statstr(const struct stat * st,
 static int ftp_st_ino = 0xdead0000;
 
 static int ftpStat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/
+	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_STAT, st, NULL, 0);
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
-    /*@-mods@*/
     if (st->st_ino == 0)
 	st->st_ino = ftp_st_ino++;
-    /*@=mods@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
     return rc;
 }
 
 static int ftpLstat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/
+	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_LSTAT, st, NULL, 0);
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
-    /*@-mods@*/
     if (st->st_ino == 0)
 	st->st_ino = ftp_st_ino++;
-    /*@=mods@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpLstat(%s) rc %d\n%s\n", path, rc, statstr(st, buf));
     return rc;
 }
 
 static int ftpReadlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies *buf, fileSystem, internalState @*/
 {
     int rc;
@@ -1085,7 +1082,7 @@ struct __dirstream {
     size_t size;		/* Total valid data in the block.  */
     size_t offset;		/* Current offset into the block.  */
     off_t filepos;		/* Position of next entry to read.  */
-#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
+#if defined(HAVE_PTHREAD_H)
     pthread_mutex_t lock;	/* Mutex lock for this structure.  */
 #endif
 };
@@ -1110,10 +1107,9 @@ static int ftpmagicdir = 0x8440291;
 #define	ISFTPMAGIC(_dir) (!memcmp((_dir), &ftpmagicdir, sizeof(ftpmagicdir)))
 
 /*@-boundswrite@*/
-/*@-type@*/ /* FIX: abstract DIR */
 /*@null@*/
 static DIR * ftpOpendir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     FTPDIR mydir;
@@ -1262,12 +1258,10 @@ static struct dirent * ftpReaddir(DIR * dir)
     int ac;
     int i;
 
-    /*@+voidabstract@*/
     if (mydir == NULL || !ISFTPMAGIC(mydir) || mydir->data == NULL) {
 	/* XXX TODO: EBADF errno. */
 	return NULL;
     }
-    /*@=voidabstract@*/
 
     dp = (struct dirent *) mydir->data;
     av = (const char **) (dp + 1);
@@ -1283,6 +1277,7 @@ static struct dirent * ftpReaddir(DIR * dir)
     mydir->offset = i;
 
     /* XXX glob(3) uses REAL_DIR_ENTRY(dp) test on d_ino */
+/*@-type@*/
     dp->d_ino = i + 1;		/* W2DO? */
     dp->d_reclen = 0;		/* W2DO? */
 
@@ -1292,16 +1287,14 @@ static struct dirent * ftpReaddir(DIR * dir)
     dp->d_type = dt[i];
 /*@=boundsread@*/
 #endif
+/*@=type@*/
 
     strncpy(dp->d_name, av[i], sizeof(dp->d_name));
-/*@+voidabstract@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpReaddir(%p) %p \"%s\"\n", (void *)mydir, dp, dp->d_name);
-/*@=voidabstract@*/
     
     return dp;
 }
-/*@=type@*/
 
 static int ftpClosedir(/*@only@*/ DIR * dir)
 	/*@globals fileSystem @*/
@@ -1309,7 +1302,6 @@ static int ftpClosedir(/*@only@*/ DIR * dir)
 {
     FTPDIR mydir = (FTPDIR)dir;
 
-    /*@+voidabstract@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpClosedir(%p)\n", (void *)mydir);
     if (mydir == NULL || !ISFTPMAGIC(mydir)) {
@@ -1317,7 +1309,6 @@ fprintf(stderr, "*** ftpClosedir(%p)\n", (void *)mydir);
 	return -1;
     }
     free((void *)mydir);
-    /*@=voidabstract@*/
     mydir = NULL;
     return 0;
 }
@@ -1392,7 +1383,7 @@ int Readlink(const char * path, char * buf, size_t bufsiz)
 	return -2;
 	/*@notreached@*/ break;
     }
-/*@-compdef@*/
+/*@-compdef@*/ /* FIX: *buf is undefined */
     return readlink(path, buf, bufsiz);
 /*@=compdef@*/
 }
@@ -1420,6 +1411,52 @@ fprintf(stderr, "*** Access(%s,%d)\n", path, amode);
     return access(path, amode);
 }
 
+/* glob_pattern_p() taken from bash
+ * Copyright (C) 1985, 1988, 1989 Free Software Foundation, Inc.
+ *
+ * Return nonzero if PATTERN has any special globbing chars in it.
+ */
+int Glob_pattern_p (const char * pattern, int quote)
+{
+    const char *p;
+    int open = 0;
+    char c;
+  
+    (void) urlPath(pattern, &p);
+    while ((c = *p++) != '\0')
+	switch (c) {
+	case '?':
+	case '*':
+	    return (1);
+	case '\\':
+	    if (quote && p[1] != '\0')
+		p++;
+	    continue;
+
+	case '[':
+	    open = 1;
+	    continue;
+	case ']':
+	    if (open)
+		return (1);
+	    continue;      
+
+	case '+':
+	case '@':
+	case '!':
+	    if (*p == '(')
+		return (1);
+	    continue;
+	}
+
+    return (0);
+}
+
+int Glob_error(/*@unused@*/const char * epath, /*@unused@*/ int eerrno)
+{
+    return 1;
+}
+
 int Glob(const char *pattern, int flags,
 	int errfunc(const char * epath, int eerrno), glob_t *pglob)
 {
@@ -1440,8 +1477,11 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
 	pglob->gl_stat = Stat;
 /*@=type@*/
 	flags |= GLOB_ALTDIRFUNC;
+	flags &= ~GLOB_TILDE;
 	break;
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+	flags &= ~GLOB_TILDE;
+	/*@fallthrough@*/
     case URL_IS_PATH:
 	pattern = lpath;
 	/*@fallthrough@*/
@@ -1489,7 +1529,6 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
     /*@=dependenttrans@*/
 }
 
-/*@+voidabstract@*/
 struct dirent * Readdir(DIR * dir)
 {
 if (_rpmio_debug)
@@ -1507,4 +1546,3 @@ fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
 	return ftpClosedir(dir);
     return closedir(dir);
 }
-/*@=voidabstract@*/
