@@ -26,18 +26,23 @@ int unameToUid(const char * thisUname, uid_t * uid)
 	lastUnameLen = 0;
 	return -1;
     } else if (strcmp(thisUname, "root") == 0) {
+/*@-boundswrite@*/
 	*uid = 0;
+/*@=boundswrite@*/
 	return 0;
     }
 
     thisUnameLen = strlen(thisUname);
     if (lastUname == NULL || thisUnameLen != lastUnameLen ||
-	strcmp(thisUname, lastUname) != 0) {
+	strcmp(thisUname, lastUname) != 0)
+    {
 	if (lastUnameAlloced < thisUnameLen + 1) {
 	    lastUnameAlloced = thisUnameLen + 10;
 	    lastUname = xrealloc(lastUname, lastUnameAlloced);	/* XXX memory leak */
 	}
+/*@-boundswrite@*/
 	strcpy(lastUname, thisUname);
+/*@=boundswrite@*/
 
 	pwent = getpwnam(thisUname);
 	if (pwent == NULL) {
@@ -51,7 +56,9 @@ int unameToUid(const char * thisUname, uid_t * uid)
 	lastUid = pwent->pw_uid;
     }
 
+/*@-boundswrite@*/
     *uid = lastUid;
+/*@=boundswrite@*/
 
     return 0;
 }
@@ -69,7 +76,9 @@ int gnameToGid(const char * thisGname, gid_t * gid)
 	lastGnameLen = 0;
 	return -1;
     } else if (strcmp(thisGname, "root") == 0) {
+/*@-boundswrite@*/
 	*gid = 0;
+/*@=boundswrite@*/
 	return 0;
     }
 
@@ -81,7 +90,9 @@ int gnameToGid(const char * thisGname, gid_t * gid)
 	    lastGnameAlloced = thisGnameLen + 10;
 	    lastGname = xrealloc(lastGname, lastGnameAlloced);	/* XXX memory leak */
 	}
+/*@-boundswrite@*/
 	strcpy(lastGname, thisGname);
+/*@=boundswrite@*/
 
 	grent = getgrnam(thisGname);
 	if (grent == NULL) {
@@ -89,12 +100,25 @@ int gnameToGid(const char * thisGname, gid_t * gid)
 	    endgrent();
 	    /*@=internalglobs@*/
 	    grent = getgrnam(thisGname);
-	    if (grent == NULL) return -1;
+	    if (grent == NULL) {
+		/* XXX The filesystem package needs group/lock w/o getgrnam. */
+		if (strcmp(thisGname, "lock") == 0) {
+		    *gid = lastGid = 54;
+		    return 0;
+		} else
+		if (strcmp(thisGname, "mail") == 0) {
+		    *gid = lastGid = 12;
+		    return 0;
+		} else
+		return -1;
+	    }
 	}
 	lastGid = grent->gr_gid;
     }
 
+/*@-boundswrite@*/
     *gid = lastGid;
+/*@=boundswrite@*/
 
     return 0;
 }
@@ -124,7 +148,9 @@ char * uidToUname(uid_t uid)
 	    lastUnameLen = len + 20;
 	    lastUname = xrealloc(lastUname, lastUnameLen);
 	}
+/*@-boundswrite@*/
 	strcpy(lastUname, pwent->pw_name);
+/*@=boundswrite@*/
 
 	return lastUname;
     }
@@ -155,7 +181,9 @@ char * gidToGname(gid_t gid)
 	    lastGnameLen = len + 20;
 	    lastGname = xrealloc(lastGname, lastGnameLen);
 	}
+/*@-boundswrite@*/
 	strcpy(lastGname, grent->gr_name);
+/*@=boundswrite@*/
 
 	return lastGname;
     }
